/*
  This file is part of LilyPond, the GNU music typesetter.

  Copyright (C) 1997--2026 Han-Wen Nienhuys <hanwen@xs4all.nl>

  LilyPond is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  LilyPond is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with LilyPond.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "audio-element.hh"
#include "lily-guile.hh"

#include <cassert>

Audio_element::Audio_element ()
{
}

Audio_element::~Audio_element ()
{
}

void
Audio_element::gc_mark () const
{
  if (cause_)
    {
      scm_gc_mark (cause_->self_scm ());
    }
}

char const *
Audio_element::name () const
{
  return class_name ();
}
