/* A Bison parser, made by GNU Bison 3.8.2.  */

/* Bison implementation for Yacc-like parsers in C

   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
   Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

/* As a special exception, you may create a larger work that contains
   part or all of the Bison parser skeleton and distribute that work
   under terms of your choice, so long as that work isn't itself a
   parser generator using the skeleton or a modified version thereof
   as a parser skeleton.  Alternatively, if you modify or redistribute
   the parser skeleton itself, you may (at your option) remove this
   special exception, which will cause the skeleton and the resulting
   Bison output files to be licensed under the GNU General Public
   License without this special exception.

   This special exception was added by the Free Software Foundation in
   version 2.2 of Bison.  */

/* C LALR(1) parser skeleton written by Richard Stallman, by
   simplifying the original so-called "semantic" parser.  */

/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
   especially those whose name start with YY_ or yy_.  They are
   private implementation details that can be changed or removed.  */

/* All symbols defined below should begin with yy or YY, to avoid
   infringing on user name space.  This should be done even for local
   variables, as they might otherwise be expanded by user macros.
   There are some unavoidable exceptions within include files to
   define necessary library symbols; they are noted "INFRINGES ON
   USER NAME SPACE" below.  */

/* Identify Bison output, and Bison version.  */
#define YYBISON 30802

/* Bison version string.  */
#define YYBISON_VERSION "3.8.2"

/* Skeleton name.  */
#define YYSKELETON_NAME "yacc.c"

/* Pure parsers.  */
#define YYPURE 2

/* Push parsers.  */
#define YYPUSH 0

/* Pull parsers.  */
#define YYPULL 1




/* First part of user prologue.  */
#line 30 "parser.yy"


#define yyerror Lily_parser::parser_error

/* We use custom location type: Input objects */
#define YYLTYPE Input
#define YYSTYPE SCM
#define YYLLOC_DEFAULT(Current,Rhs,N) \
	((Current).set_location ((Rhs)[1], (Rhs)[N]))

#define YYPRINT(file, type, value)					\
	do {								\
		if (scm_is_eq (value, SCM_UNSPECIFIED))			\
			break;						\
		SCM s = Display::value_to_lily_string (value);		\
		char *p = scm_to_locale_string (s);			\
		fputs (p, file);					\
		free (p);						\
	} while (0)


#define lookup_identifier(x) lookup_identifier_symbol (ly_symbol2scm (x))

#line 101 "parser.yy"
 // -*-Fundamental-*-

/*
FIXME:

   * The rules for who is protecting what are very shady.  Uniformise
     this.

   * There are too many lexical modes?
*/

#include "config.hh"

#include <cctype>
#include <cstdlib>
#include <cstdio>

#include "book.hh"
#include "context.hh"
#include "context-def.hh"
#include "context-mod.hh"
#include "dimensions.hh"
#include "file-path.hh"
#include "input.hh"
#include "international.hh"
#include "lily-guile.hh"
#include "lily-lexer.hh"
#include "lily-parser.hh"
#include "ly-module.hh"
#include "ly-scm-list.hh"
#include "misc.hh"
#include "music.hh"
#include "output-def.hh"
#include "scm-hash.hh"
#include "score.hh"
#include "std-vector.hh"
#include "text-interface.hh"
#include "warn.hh"
#include "lily-imports.hh"

void
Lily_parser::parser_error (Input const *i, Lily_parser *parser, SCM *, const std::string &s)
{
	parser->parser_error (*i, s);
}

// The following are somewhat precarious constructs as they may change
// the value of the lookahead token.  That implies that the lookahead
// token must not yet have made an impact on the state stack other
// than causing the reduction of the current rule, or switching the
// lookahead token while Bison is mulling it over will cause trouble.

#define MYBACKUP(Token, Value, Location)				\
	do {								\
		if (yychar != YYEMPTY)					\
			parser->lexer_->push_extra_token		\
				(yylloc, yychar, yylval);		\
		parser->lexer_->push_extra_token			\
			(Location, Token, Value);			\
		parser->lexer_->push_extra_token (Location, BACKUP);	\
		yychar = YYEMPTY;					\
	} while (0)


#define MYREPARSE(Location, Pred, Token, Value)				\
	do {								\
		if (yychar != YYEMPTY)					\
			parser->lexer_->push_extra_token		\
				(yylloc, yychar, yylval);		\
		parser->lexer_->push_extra_token			\
			(Location, Token, Value);			\
		parser->lexer_->push_extra_token			\
			(Location, REPARSE, Pred);			\
		yychar = YYEMPTY;					\
	} while (0)

#line 180 "parser.yy"


#define MY_MAKE_MUSIC(x, spot) \
	make_music_with_input (ly_symbol2scm (x), \
			       parser->lexer_->override_input (spot))

/* ES TODO:
- delay application of the function
*/

#define LOWLEVEL_MAKE_SYNTAX(location, proc, ...)			\
	with_location							\
		(parser->lexer_->override_input (location).smobbed_copy (), \
		 proc,							\
		 ##__VA_ARGS__)

/* Syntactic Sugar. */
#define MAKE_SYNTAX(name, location, ...)				\
	LOWLEVEL_MAKE_SYNTAX (location, Syntax::name, ##__VA_ARGS__)

#define START_MAKE_SYNTAX(name, ...)					\
	ly_list (Syntax::name, ##__VA_ARGS__)

#define FINISH_MAKE_SYNTAX(start, location, ...)			\
	LOWLEVEL_MAKE_SYNTAX						\
		(location,						\
		 Guile_user::apply,					\
		 scm_car (start),					\
		 scm_append_x						\
		 (ly_list (scm_cdr (start),				\
		           ly_list (__VA_ARGS__))))

#undef _
#if !HAVE_GETTEXT
#define _(x) x
#else
#include <libintl.h>
#define _(x) gettext (x)
#endif

using std::string;

static Music *make_music_with_input (SCM name, Input where);
bool add_post_events (Music *music, SCM events);
SCM reverse_music_list (Lily_parser *parser, Input loc, SCM lst, bool preserve, bool compress);
SCM check_scheme_arg (Lily_parser *parser, Input loc,
		      SCM arg, SCM args, SCM pred, SCM disp = SCM_UNDEFINED);
SCM make_music_from_simple (Lily_parser *parser, Input loc, SCM pitch);
SCM loc_on_copy (Lily_parser *parser, Input loc, SCM arg);
SCM make_chord_elements (Input loc, SCM pitch, SCM dur, SCM modification_list);
SCM make_chord_step (SCM step, Rational alter);
SCM make_simple_markup (SCM a);
SCM make_duration (SCM t, int dots = 0, SCM factor = SCM_UNDEFINED);
bool is_regular_identifier (SCM id, bool multiple=false);
SCM make_reverse_key_list (SCM keys);
SCM try_word_variants (SCM pred, SCM str);
SCM try_string_variants (SCM pred, SCM str);
SCM post_event_cons (SCM ev, SCM tail);
void property_path_dot_warning (Input loc, SCM lst);
int yylex (YYSTYPE *s, YYLTYPE *loc, Lily_parser *parser);


// generated code may trigger conversion warnings
#pragma GCC diagnostic ignored "-Wconversion"

// generated code triggers a false positive in GCC 11
// https://gcc.gnu.org/bugzilla/show_bug.cgi?id=98753
#pragma GCC diagnostic ignored "-Wfree-nonheap-object"

// generated code contains some old-style casts
#pragma GCC diagnostic ignored "-Wold-style-cast"

// generated `int yynerrs = 0;` triggers this in Clang >= 15
#pragma GCC diagnostic ignored "-Wunused-but-set-variable"

// generated code contains some useless casts
#if !defined(__clang__)
#pragma GCC diagnostic ignored "-Wuseless-cast"
#endif


#line 254 "./out/parser.cc"

# ifndef YY_CAST
#  ifdef __cplusplus
#   define YY_CAST(Type, Val) static_cast<Type> (Val)
#   define YY_REINTERPRET_CAST(Type, Val) reinterpret_cast<Type> (Val)
#  else
#   define YY_CAST(Type, Val) ((Type) (Val))
#   define YY_REINTERPRET_CAST(Type, Val) ((Type) (Val))
#  endif
# endif
# ifndef YY_NULLPTR
#  if defined __cplusplus
#   if 201103L <= __cplusplus
#    define YY_NULLPTR nullptr
#   else
#    define YY_NULLPTR 0
#   endif
#  else
#   define YY_NULLPTR ((void*)0)
#  endif
# endif

#include "parser.hh"
/* Symbol kind.  */
enum yysymbol_kind_t
{
  YYSYMBOL_YYEMPTY = -2,
  YYSYMBOL_YYEOF = 0,                      /* "end of input"  */
  YYSYMBOL_YYerror = 1,                    /* error  */
  YYSYMBOL_YYUNDEF = 2,                    /* "invalid token"  */
  YYSYMBOL_PREC_BOT = 3,                   /* PREC_BOT  */
  YYSYMBOL_COMPOSITE = 4,                  /* COMPOSITE  */
  YYSYMBOL_5_ = 5,                         /* ':'  */
  YYSYMBOL_6_ = 6,                         /* '^'  */
  YYSYMBOL_7___ = 7,                       /* '_'  */
  YYSYMBOL_8_ = 8,                         /* '!'  */
  YYSYMBOL_9_ = 9,                         /* '\''  */
  YYSYMBOL_10_ = 10,                       /* ','  */
  YYSYMBOL_PREC_TOP = 11,                  /* PREC_TOP  */
  YYSYMBOL_ACCEPTS = 12,                   /* "\\accepts"  */
  YYSYMBOL_ADDLYRICS = 13,                 /* "\\addlyrics"  */
  YYSYMBOL_ALIAS = 14,                     /* "\\alias"  */
  YYSYMBOL_ALTERNATIVE = 15,               /* "\\alternative"  */
  YYSYMBOL_BOOK = 16,                      /* "\\book"  */
  YYSYMBOL_BOOKPART = 17,                  /* "\\bookpart"  */
  YYSYMBOL_CHANGE = 18,                    /* "\\change"  */
  YYSYMBOL_CHORDMODE = 19,                 /* "\\chordmode"  */
  YYSYMBOL_CHORDS = 20,                    /* "\\chords"  */
  YYSYMBOL_CONSISTS = 21,                  /* "\\consists"  */
  YYSYMBOL_CONTEXT = 22,                   /* "\\context"  */
  YYSYMBOL_DEFAULT = 23,                   /* "\\default"  */
  YYSYMBOL_DEFAULTCHILD = 24,              /* "\\defaultchild"  */
  YYSYMBOL_DENIES = 25,                    /* "\\denies"  */
  YYSYMBOL_DESCRIPTION = 26,               /* "\\description"  */
  YYSYMBOL_DRUMMODE = 27,                  /* "\\drummode"  */
  YYSYMBOL_DRUMS = 28,                     /* "\\drums"  */
  YYSYMBOL_ETC = 29,                       /* "\\etc"  */
  YYSYMBOL_FIGUREMODE = 30,                /* "\\figuremode"  */
  YYSYMBOL_FIGURES = 31,                   /* "\\figures"  */
  YYSYMBOL_HEADER = 32,                    /* "\\header"  */
  YYSYMBOL_INVALID = 33,                   /* "\\version-error"  */
  YYSYMBOL_LAYOUT = 34,                    /* "\\layout"  */
  YYSYMBOL_LYRICMODE = 35,                 /* "\\lyricmode"  */
  YYSYMBOL_LYRICS = 36,                    /* "\\lyrics"  */
  YYSYMBOL_LYRICSTO = 37,                  /* "\\lyricsto"  */
  YYSYMBOL_MARKUP = 38,                    /* "\\markup"  */
  YYSYMBOL_MARKUPLIST = 39,                /* "\\markuplist"  */
  YYSYMBOL_MIDI = 40,                      /* "\\midi"  */
  YYSYMBOL_NAME = 41,                      /* "\\name"  */
  YYSYMBOL_NOTEMODE = 42,                  /* "\\notemode"  */
  YYSYMBOL_OVERRIDE = 43,                  /* "\\override"  */
  YYSYMBOL_PAPER = 44,                     /* "\\paper"  */
  YYSYMBOL_REMOVE = 45,                    /* "\\remove"  */
  YYSYMBOL_REPEAT = 46,                    /* "\\repeat"  */
  YYSYMBOL_REST = 47,                      /* "\\rest"  */
  YYSYMBOL_REVERT = 48,                    /* "\\revert"  */
  YYSYMBOL_SCORE = 49,                     /* "\\score"  */
  YYSYMBOL_SCORELINES = 50,                /* "\\score-lines"  */
  YYSYMBOL_SEQUENTIAL = 51,                /* "\\sequential"  */
  YYSYMBOL_SET = 52,                       /* "\\set"  */
  YYSYMBOL_SIMULTANEOUS = 53,              /* "\\simultaneous"  */
  YYSYMBOL_TEMPO = 54,                     /* "\\tempo"  */
  YYSYMBOL_TYPE = 55,                      /* "\\type"  */
  YYSYMBOL_UNSET = 56,                     /* "\\unset"  */
  YYSYMBOL_WITH = 57,                      /* "\\with"  */
  YYSYMBOL_NEWCONTEXT = 58,                /* "\\new"  */
  YYSYMBOL_CHORD_BASS = 59,                /* "/+"  */
  YYSYMBOL_CHORD_CARET = 60,               /* "^"  */
  YYSYMBOL_CHORD_COLON = 61,               /* ":"  */
  YYSYMBOL_CHORD_MINUS = 62,               /* "-"  */
  YYSYMBOL_CHORD_SLASH = 63,               /* "/"  */
  YYSYMBOL_ANGLE_OPEN = 64,                /* "<"  */
  YYSYMBOL_ANGLE_CLOSE = 65,               /* ">"  */
  YYSYMBOL_DOUBLE_ANGLE_OPEN = 66,         /* "<<"  */
  YYSYMBOL_DOUBLE_ANGLE_CLOSE = 67,        /* ">>"  */
  YYSYMBOL_E_BACKSLASH = 68,               /* "\\\\"  */
  YYSYMBOL_E_EXCLAMATION = 69,             /* "\\!"  */
  YYSYMBOL_E_PLUS = 70,                    /* "\\+"  */
  YYSYMBOL_EXTENDER = 71,                  /* "__"  */
  YYSYMBOL_FIGURE_CLOSE = 72,              /* FIGURE_CLOSE  */
  YYSYMBOL_FIGURE_OPEN = 73,               /* FIGURE_OPEN  */
  YYSYMBOL_FIGURE_SPACE = 74,              /* "_"  */
  YYSYMBOL_FIGURE_ALTERATION_EXPR = 75,    /* FIGURE_ALTERATION_EXPR  */
  YYSYMBOL_HYPHEN = 76,                    /* "--"  */
  YYSYMBOL_MULTI_MEASURE_REST = 77,        /* MULTI_MEASURE_REST  */
  YYSYMBOL_E_UNSIGNED = 78,                /* E_UNSIGNED  */
  YYSYMBOL_UNSIGNED = 79,                  /* UNSIGNED  */
  YYSYMBOL_EXPECT_MARKUP = 80,             /* "markup?"  */
  YYSYMBOL_EXPECT_SCM = 81,                /* "scheme?"  */
  YYSYMBOL_BACKUP = 82,                    /* "(backed-up?)"  */
  YYSYMBOL_REPARSE = 83,                   /* "(reparsed?)"  */
  YYSYMBOL_EXPECT_MARKUP_LIST = 84,        /* "markup-list?"  */
  YYSYMBOL_EXPECT_OPTIONAL = 85,           /* "optional?"  */
  YYSYMBOL_EXPECT_NO_MORE_ARGS = 86,       /* EXPECT_NO_MORE_ARGS  */
  YYSYMBOL_EMBEDDED_LILY = 87,             /* "#{"  */
  YYSYMBOL_BOOK_IDENTIFIER = 88,           /* BOOK_IDENTIFIER  */
  YYSYMBOL_CHORD_MODIFIER = 89,            /* CHORD_MODIFIER  */
  YYSYMBOL_CHORD_REPETITION = 90,          /* CHORD_REPETITION  */
  YYSYMBOL_DRUM_PITCH = 91,                /* DRUM_PITCH  */
  YYSYMBOL_DURATION_ARG = 92,              /* DURATION_ARG  */
  YYSYMBOL_DURATION_IDENTIFIER = 93,       /* DURATION_IDENTIFIER  */
  YYSYMBOL_EVENT_IDENTIFIER = 94,          /* EVENT_IDENTIFIER  */
  YYSYMBOL_EVENT_FUNCTION = 95,            /* EVENT_FUNCTION  */
  YYSYMBOL_FRACTION = 96,                  /* FRACTION  */
  YYSYMBOL_LOOKUP_IDENTIFIER = 97,         /* LOOKUP_IDENTIFIER  */
  YYSYMBOL_LYRIC_ELEMENT = 98,             /* LYRIC_ELEMENT  */
  YYSYMBOL_MARKUP_FUNCTION = 99,           /* MARKUP_FUNCTION  */
  YYSYMBOL_MARKUP_LIST_FUNCTION = 100,     /* MARKUP_LIST_FUNCTION  */
  YYSYMBOL_MARKUP_IDENTIFIER = 101,        /* MARKUP_IDENTIFIER  */
  YYSYMBOL_MARKUPLIST_IDENTIFIER = 102,    /* MARKUPLIST_IDENTIFIER  */
  YYSYMBOL_MODULE_IDENTIFIER = 103,        /* MODULE_IDENTIFIER  */
  YYSYMBOL_MUSIC_FUNCTION = 104,           /* MUSIC_FUNCTION  */
  YYSYMBOL_MUSIC_IDENTIFIER = 105,         /* MUSIC_IDENTIFIER  */
  YYSYMBOL_NOTENAME_PITCH = 106,           /* NOTENAME_PITCH  */
  YYSYMBOL_NUMBER_IDENTIFIER = 107,        /* NUMBER_IDENTIFIER  */
  YYSYMBOL_PITCH_IDENTIFIER = 108,         /* PITCH_IDENTIFIER  */
  YYSYMBOL_REAL = 109,                     /* REAL  */
  YYSYMBOL_RESTNAME = 110,                 /* RESTNAME  */
  YYSYMBOL_SCM_ARG = 111,                  /* SCM_ARG  */
  YYSYMBOL_SCM_FUNCTION = 112,             /* SCM_FUNCTION  */
  YYSYMBOL_SCM_IDENTIFIER = 113,           /* SCM_IDENTIFIER  */
  YYSYMBOL_SCM_TOKEN = 114,                /* SCM_TOKEN  */
  YYSYMBOL_STRING = 115,                   /* STRING  */
  YYSYMBOL_SYMBOL_LIST = 116,              /* SYMBOL_LIST  */
  YYSYMBOL_TONICNAME_PITCH = 117,          /* TONICNAME_PITCH  */
  YYSYMBOL_SYMBOL = 118,                   /* SYMBOL  */
  YYSYMBOL_119_ = 119,                     /* '-'  */
  YYSYMBOL_120_ = 120,                     /* '+'  */
  YYSYMBOL_UNARY_MINUS = 121,              /* UNARY_MINUS  */
  YYSYMBOL_122_ = 122,                     /* '.'  */
  YYSYMBOL_123_ = 123,                     /* '{'  */
  YYSYMBOL_124_ = 124,                     /* '}'  */
  YYSYMBOL_125_ = 125,                     /* '='  */
  YYSYMBOL_126_ = 126,                     /* '*'  */
  YYSYMBOL_127_ = 127,                     /* ']'  */
  YYSYMBOL_128_ = 128,                     /* '/'  */
  YYSYMBOL_129_ = 129,                     /* '['  */
  YYSYMBOL_130_ = 130,                     /* '?'  */
  YYSYMBOL_YYACCEPT = 131,                 /* $accept  */
  YYSYMBOL_start_symbol = 132,             /* start_symbol  */
  YYSYMBOL_133_1 = 133,                    /* $@1  */
  YYSYMBOL_lilypond = 134,                 /* lilypond  */
  YYSYMBOL_toplevel_expression = 135,      /* toplevel_expression  */
  YYSYMBOL_lookup = 136,                   /* lookup  */
  YYSYMBOL_embedded_scm_bare = 137,        /* embedded_scm_bare  */
  YYSYMBOL_embedded_scm_active = 138,      /* embedded_scm_active  */
  YYSYMBOL_embedded_scm_bare_arg = 139,    /* embedded_scm_bare_arg  */
  YYSYMBOL_embedded_scm = 140,             /* embedded_scm  */
  YYSYMBOL_embedded_scm_arg = 141,         /* embedded_scm_arg  */
  YYSYMBOL_scm_function_call = 142,        /* scm_function_call  */
  YYSYMBOL_embedded_lilypond_number = 143, /* embedded_lilypond_number  */
  YYSYMBOL_embedded_lilypond = 144,        /* embedded_lilypond  */
  YYSYMBOL_lilypond_header_body = 145,     /* lilypond_header_body  */
  YYSYMBOL_lilypond_header = 146,          /* lilypond_header  */
  YYSYMBOL_header_block = 147,             /* header_block  */
  YYSYMBOL_148_2 = 148,                    /* $@2  */
  YYSYMBOL_header_modification = 149,      /* header_modification  */
  YYSYMBOL_150_3 = 150,                    /* $@3  */
  YYSYMBOL_assignment_id = 151,            /* assignment_id  */
  YYSYMBOL_assignment = 152,               /* assignment  */
  YYSYMBOL_identifier_init = 153,          /* identifier_init  */
  YYSYMBOL_identifier_init_nonumber = 154, /* identifier_init_nonumber  */
  YYSYMBOL_partial_function_scriptable = 155, /* partial_function_scriptable  */
  YYSYMBOL_partial_function = 156,         /* partial_function  */
  YYSYMBOL_context_def_spec_block = 157,   /* context_def_spec_block  */
  YYSYMBOL_context_mod_arg = 158,          /* context_mod_arg  */
  YYSYMBOL_159_4 = 159,                    /* $@4  */
  YYSYMBOL_context_def_spec_body = 160,    /* context_def_spec_body  */
  YYSYMBOL_book_block = 161,               /* book_block  */
  YYSYMBOL_book_body = 162,                /* book_body  */
  YYSYMBOL_163_5 = 163,                    /* $@5  */
  YYSYMBOL_bookpart_block = 164,           /* bookpart_block  */
  YYSYMBOL_bookpart_body = 165,            /* bookpart_body  */
  YYSYMBOL_166_6 = 166,                    /* $@6  */
  YYSYMBOL_score_block = 167,              /* score_block  */
  YYSYMBOL_score_body = 168,               /* score_body  */
  YYSYMBOL_score_item = 169,               /* score_item  */
  YYSYMBOL_score_items = 170,              /* score_items  */
  YYSYMBOL_171_7 = 171,                    /* $@7  */
  YYSYMBOL_paper_block = 172,              /* paper_block  */
  YYSYMBOL_output_def = 173,               /* output_def  */
  YYSYMBOL_output_def_head = 174,          /* output_def_head  */
  YYSYMBOL_output_def_head_with_mode_switch = 175, /* output_def_head_with_mode_switch  */
  YYSYMBOL_music_or_context_def = 176,     /* music_or_context_def  */
  YYSYMBOL_output_def_body = 177,          /* output_def_body  */
  YYSYMBOL_178_8 = 178,                    /* $@8  */
  YYSYMBOL_tempo_event = 179,              /* tempo_event  */
  YYSYMBOL_music_list = 180,               /* music_list  */
  YYSYMBOL_braced_music_list = 181,        /* braced_music_list  */
  YYSYMBOL_music = 182,                    /* music  */
  YYSYMBOL_pitch_as_music = 183,           /* pitch_as_music  */
  YYSYMBOL_music_embedded = 184,           /* music_embedded  */
  YYSYMBOL_music_embedded_backup = 185,    /* music_embedded_backup  */
  YYSYMBOL_music_assign = 186,             /* music_assign  */
  YYSYMBOL_repeated_music = 187,           /* repeated_music  */
  YYSYMBOL_alternative_music = 188,        /* alternative_music  */
  YYSYMBOL_sequential_music = 189,         /* sequential_music  */
  YYSYMBOL_simultaneous_music = 190,       /* simultaneous_music  */
  YYSYMBOL_simple_music = 191,             /* simple_music  */
  YYSYMBOL_context_modification = 192,     /* context_modification  */
  YYSYMBOL_193_9 = 193,                    /* $@9  */
  YYSYMBOL_context_modification_arg = 194, /* context_modification_arg  */
  YYSYMBOL_optional_context_mods = 195,    /* optional_context_mods  */
  YYSYMBOL_context_modification_mods_list = 196, /* context_modification_mods_list  */
  YYSYMBOL_context_mod_list = 197,         /* context_mod_list  */
  YYSYMBOL_context_prefix = 198,           /* context_prefix  */
  YYSYMBOL_new_lyrics = 199,               /* new_lyrics  */
  YYSYMBOL_basic_music = 200,              /* basic_music  */
  YYSYMBOL_contextable_music = 201,        /* contextable_music  */
  YYSYMBOL_contexted_basic_music = 202,    /* contexted_basic_music  */
  YYSYMBOL_composite_music = 203,          /* composite_music  */
  YYSYMBOL_music_bare = 204,               /* music_bare  */
  YYSYMBOL_grouped_music_list = 205,       /* grouped_music_list  */
  YYSYMBOL_symbol_list_arg = 206,          /* symbol_list_arg  */
  YYSYMBOL_symbol_list_rev = 207,          /* symbol_list_rev  */
  YYSYMBOL_symbol_list_part = 208,         /* symbol_list_part  */
  YYSYMBOL_symbol_list_element = 209,      /* symbol_list_element  */
  YYSYMBOL_symbol_list_part_bare = 210,    /* symbol_list_part_bare  */
  YYSYMBOL_function_arglist_nonbackup = 211, /* function_arglist_nonbackup  */
  YYSYMBOL_reparsed_rhythm = 212,          /* reparsed_rhythm  */
  YYSYMBOL_function_arglist_nonbackup_reparse = 213, /* function_arglist_nonbackup_reparse  */
  YYSYMBOL_function_arglist_backup = 214,  /* function_arglist_backup  */
  YYSYMBOL_function_arglist = 215,         /* function_arglist  */
  YYSYMBOL_function_arglist_skip_nonbackup = 216, /* function_arglist_skip_nonbackup  */
  YYSYMBOL_function_arglist_partial = 217, /* function_arglist_partial  */
  YYSYMBOL_function_arglist_partial_optional = 218, /* function_arglist_partial_optional  */
  YYSYMBOL_function_arglist_common = 219,  /* function_arglist_common  */
  YYSYMBOL_function_arglist_common_reparse = 220, /* function_arglist_common_reparse  */
  YYSYMBOL_function_arglist_optional = 221, /* function_arglist_optional  */
  YYSYMBOL_function_arglist_skip_backup = 222, /* function_arglist_skip_backup  */
  YYSYMBOL_music_function_call = 223,      /* music_function_call  */
  YYSYMBOL_optional_id = 224,              /* optional_id  */
  YYSYMBOL_lyric_mode_music = 225,         /* lyric_mode_music  */
  YYSYMBOL_226_10 = 226,                   /* $@10  */
  YYSYMBOL_mode_changed_music = 227,       /* mode_changed_music  */
  YYSYMBOL_mode_changing_head = 228,       /* mode_changing_head  */
  YYSYMBOL_mode_changing_head_with_context = 229, /* mode_changing_head_with_context  */
  YYSYMBOL_context_change = 230,           /* context_change  */
  YYSYMBOL_property_path = 231,            /* property_path  */
  YYSYMBOL_property_operation = 232,       /* property_operation  */
  YYSYMBOL_revert_arg = 233,               /* revert_arg  */
  YYSYMBOL_revert_arg_backup = 234,        /* revert_arg_backup  */
  YYSYMBOL_revert_arg_part = 235,          /* revert_arg_part  */
  YYSYMBOL_context_def_mod = 236,          /* context_def_mod  */
  YYSYMBOL_context_mod = 237,              /* context_mod  */
  YYSYMBOL_grob_prop_spec = 238,           /* grob_prop_spec  */
  YYSYMBOL_grob_prop_path = 239,           /* grob_prop_path  */
  YYSYMBOL_context_prop_spec = 240,        /* context_prop_spec  */
  YYSYMBOL_simple_revert_context = 241,    /* simple_revert_context  */
  YYSYMBOL_music_property_def = 242,       /* music_property_def  */
  YYSYMBOL_string = 243,                   /* string  */
  YYSYMBOL_text = 244,                     /* text  */
  YYSYMBOL_simple_string = 245,            /* simple_string  */
  YYSYMBOL_symbol = 246,                   /* symbol  */
  YYSYMBOL_scalar = 247,                   /* scalar  */
  YYSYMBOL_event_chord = 248,              /* event_chord  */
  YYSYMBOL_note_chord_element = 249,       /* note_chord_element  */
  YYSYMBOL_chord_body = 250,               /* chord_body  */
  YYSYMBOL_chord_body_elements = 251,      /* chord_body_elements  */
  YYSYMBOL_chord_body_element = 252,       /* chord_body_element  */
  YYSYMBOL_music_function_chord_body = 253, /* music_function_chord_body  */
  YYSYMBOL_event_function_event = 254,     /* event_function_event  */
  YYSYMBOL_post_events = 255,              /* post_events  */
  YYSYMBOL_post_event_nofinger = 256,      /* post_event_nofinger  */
  YYSYMBOL_post_event = 257,               /* post_event  */
  YYSYMBOL_string_number_event = 258,      /* string_number_event  */
  YYSYMBOL_direction_less_event = 259,     /* direction_less_event  */
  YYSYMBOL_direction_reqd_event = 260,     /* direction_reqd_event  */
  YYSYMBOL_octave_check = 261,             /* octave_check  */
  YYSYMBOL_quotes = 262,                   /* quotes  */
  YYSYMBOL_erroneous_quotes = 263,         /* erroneous_quotes  */
  YYSYMBOL_sup_quotes = 264,               /* sup_quotes  */
  YYSYMBOL_sub_quotes = 265,               /* sub_quotes  */
  YYSYMBOL_steno_pitch = 266,              /* steno_pitch  */
  YYSYMBOL_steno_tonic_pitch = 267,        /* steno_tonic_pitch  */
  YYSYMBOL_pitch = 268,                    /* pitch  */
  YYSYMBOL_pitch_or_tonic_pitch = 269,     /* pitch_or_tonic_pitch  */
  YYSYMBOL_gen_text_def = 270,             /* gen_text_def  */
  YYSYMBOL_fingering = 271,                /* fingering  */
  YYSYMBOL_script_abbreviation = 272,      /* script_abbreviation  */
  YYSYMBOL_script_dir = 273,               /* script_dir  */
  YYSYMBOL_maybe_notemode_duration = 274,  /* maybe_notemode_duration  */
  YYSYMBOL_optional_notemode_duration = 275, /* optional_notemode_duration  */
  YYSYMBOL_steno_duration = 276,           /* steno_duration  */
  YYSYMBOL_duration = 277,                 /* duration  */
  YYSYMBOL_dots = 278,                     /* dots  */
  YYSYMBOL_multiplier_scm = 279,           /* multiplier_scm  */
  YYSYMBOL_multipliers = 280,              /* multipliers  */
  YYSYMBOL_tremolo_type = 281,             /* tremolo_type  */
  YYSYMBOL_bass_number = 282,              /* bass_number  */
  YYSYMBOL_bass_figure = 283,              /* bass_figure  */
  YYSYMBOL_figured_bass_modification = 284, /* figured_bass_modification  */
  YYSYMBOL_br_bass_figure = 285,           /* br_bass_figure  */
  YYSYMBOL_figure_list = 286,              /* figure_list  */
  YYSYMBOL_optional_rest = 287,            /* optional_rest  */
  YYSYMBOL_pitch_or_music = 288,           /* pitch_or_music  */
  YYSYMBOL_simple_element = 289,           /* simple_element  */
  YYSYMBOL_lyric_element = 290,            /* lyric_element  */
  YYSYMBOL_lyric_element_music = 291,      /* lyric_element_music  */
  YYSYMBOL_new_chord = 292,                /* new_chord  */
  YYSYMBOL_chord_items = 293,              /* chord_items  */
  YYSYMBOL_chord_separator = 294,          /* chord_separator  */
  YYSYMBOL_chord_item = 295,               /* chord_item  */
  YYSYMBOL_step_numbers = 296,             /* step_numbers  */
  YYSYMBOL_step_number = 297,              /* step_number  */
  YYSYMBOL_tempo_range = 298,              /* tempo_range  */
  YYSYMBOL_number_expression = 299,        /* number_expression  */
  YYSYMBOL_number_term = 300,              /* number_term  */
  YYSYMBOL_number_factor = 301,            /* number_factor  */
  YYSYMBOL_bare_number_common = 302,       /* bare_number_common  */
  YYSYMBOL_bare_number = 303,              /* bare_number  */
  YYSYMBOL_exact_unsigned_number = 304,    /* exact_unsigned_number  */
  YYSYMBOL_unsigned_integer = 305,         /* unsigned_integer  */
  YYSYMBOL_exclamations = 306,             /* exclamations  */
  YYSYMBOL_questions = 307,                /* questions  */
  YYSYMBOL_full_markup_list = 308,         /* full_markup_list  */
  YYSYMBOL_309_11 = 309,                   /* $@11  */
  YYSYMBOL_markup_mode = 310,              /* markup_mode  */
  YYSYMBOL_markup_mode_word = 311,         /* markup_mode_word  */
  YYSYMBOL_full_markup = 312,              /* full_markup  */
  YYSYMBOL_partial_markup = 313,           /* partial_markup  */
  YYSYMBOL_markup_top = 314,               /* markup_top  */
  YYSYMBOL_markup_scm = 315,               /* markup_scm  */
  YYSYMBOL_316_12 = 316,                   /* $@12  */
  YYSYMBOL_markup_list = 317,              /* markup_list  */
  YYSYMBOL_markup_uncomposed_list = 318,   /* markup_uncomposed_list  */
  YYSYMBOL_319_13 = 319,                   /* $@13  */
  YYSYMBOL_markup_composed_list = 320,     /* markup_composed_list  */
  YYSYMBOL_markup_braced_list = 321,       /* markup_braced_list  */
  YYSYMBOL_markup_braced_list_body = 322,  /* markup_braced_list_body  */
  YYSYMBOL_markup_command_list = 323,      /* markup_command_list  */
  YYSYMBOL_markup_command_basic_arguments = 324, /* markup_command_basic_arguments  */
  YYSYMBOL_325_14 = 325,                   /* $@14  */
  YYSYMBOL_markup_command_list_arguments = 326, /* markup_command_list_arguments  */
  YYSYMBOL_markup_partial_function = 327,  /* markup_partial_function  */
  YYSYMBOL_markup_arglist_partial = 328,   /* markup_arglist_partial  */
  YYSYMBOL_markup_head_1_item = 329,       /* markup_head_1_item  */
  YYSYMBOL_markup_head_1_list = 330,       /* markup_head_1_list  */
  YYSYMBOL_markup_word = 331,              /* markup_word  */
  YYSYMBOL_simple_markup = 332,            /* simple_markup  */
  YYSYMBOL_simple_markup_noword = 333,     /* simple_markup_noword  */
  YYSYMBOL_334_15 = 334,                   /* $@15  */
  YYSYMBOL_markup = 335                    /* markup  */
};
typedef enum yysymbol_kind_t yysymbol_kind_t;




#ifdef short
# undef short
#endif

/* On compilers that do not define __PTRDIFF_MAX__ etc., make sure
   <limits.h> and (if available) <stdint.h> are included
   so that the code can choose integer types of a good width.  */

#ifndef __PTRDIFF_MAX__
# include <limits.h> /* INFRINGES ON USER NAME SPACE */
# if defined __STDC_VERSION__ && 199901 <= __STDC_VERSION__
#  include <stdint.h> /* INFRINGES ON USER NAME SPACE */
#  define YY_STDINT_H
# endif
#endif

/* Narrow types that promote to a signed type and that can represent a
   signed or unsigned integer of at least N bits.  In tables they can
   save space and decrease cache pressure.  Promoting to a signed type
   helps avoid bugs in integer arithmetic.  */

#ifdef __INT_LEAST8_MAX__
typedef __INT_LEAST8_TYPE__ yytype_int8;
#elif defined YY_STDINT_H
typedef int_least8_t yytype_int8;
#else
typedef signed char yytype_int8;
#endif

#ifdef __INT_LEAST16_MAX__
typedef __INT_LEAST16_TYPE__ yytype_int16;
#elif defined YY_STDINT_H
typedef int_least16_t yytype_int16;
#else
typedef short yytype_int16;
#endif

/* Work around bug in HP-UX 11.23, which defines these macros
   incorrectly for preprocessor constants.  This workaround can likely
   be removed in 2023, as HPE has promised support for HP-UX 11.23
   (aka HP-UX 11i v2) only through the end of 2022; see Table 2 of
   <https://h20195.www2.hpe.com/V2/getpdf.aspx/4AA4-7673ENW.pdf>.  */
#ifdef __hpux
# undef UINT_LEAST8_MAX
# undef UINT_LEAST16_MAX
# define UINT_LEAST8_MAX 255
# define UINT_LEAST16_MAX 65535
#endif

#if defined __UINT_LEAST8_MAX__ && __UINT_LEAST8_MAX__ <= __INT_MAX__
typedef __UINT_LEAST8_TYPE__ yytype_uint8;
#elif (!defined __UINT_LEAST8_MAX__ && defined YY_STDINT_H \
       && UINT_LEAST8_MAX <= INT_MAX)
typedef uint_least8_t yytype_uint8;
#elif !defined __UINT_LEAST8_MAX__ && UCHAR_MAX <= INT_MAX
typedef unsigned char yytype_uint8;
#else
typedef short yytype_uint8;
#endif

#if defined __UINT_LEAST16_MAX__ && __UINT_LEAST16_MAX__ <= __INT_MAX__
typedef __UINT_LEAST16_TYPE__ yytype_uint16;
#elif (!defined __UINT_LEAST16_MAX__ && defined YY_STDINT_H \
       && UINT_LEAST16_MAX <= INT_MAX)
typedef uint_least16_t yytype_uint16;
#elif !defined __UINT_LEAST16_MAX__ && USHRT_MAX <= INT_MAX
typedef unsigned short yytype_uint16;
#else
typedef int yytype_uint16;
#endif

#ifndef YYPTRDIFF_T
# if defined __PTRDIFF_TYPE__ && defined __PTRDIFF_MAX__
#  define YYPTRDIFF_T __PTRDIFF_TYPE__
#  define YYPTRDIFF_MAXIMUM __PTRDIFF_MAX__
# elif defined PTRDIFF_MAX
#  ifndef ptrdiff_t
#   include <stddef.h> /* INFRINGES ON USER NAME SPACE */
#  endif
#  define YYPTRDIFF_T ptrdiff_t
#  define YYPTRDIFF_MAXIMUM PTRDIFF_MAX
# else
#  define YYPTRDIFF_T long
#  define YYPTRDIFF_MAXIMUM LONG_MAX
# endif
#endif

#ifndef YYSIZE_T
# ifdef __SIZE_TYPE__
#  define YYSIZE_T __SIZE_TYPE__
# elif defined size_t
#  define YYSIZE_T size_t
# elif defined __STDC_VERSION__ && 199901 <= __STDC_VERSION__
#  include <stddef.h> /* INFRINGES ON USER NAME SPACE */
#  define YYSIZE_T size_t
# else
#  define YYSIZE_T unsigned
# endif
#endif

#define YYSIZE_MAXIMUM                                  \
  YY_CAST (YYPTRDIFF_T,                                 \
           (YYPTRDIFF_MAXIMUM < YY_CAST (YYSIZE_T, -1)  \
            ? YYPTRDIFF_MAXIMUM                         \
            : YY_CAST (YYSIZE_T, -1)))

#define YYSIZEOF(X) YY_CAST (YYPTRDIFF_T, sizeof (X))


/* Stored state numbers (used for stacks). */
typedef yytype_int16 yy_state_t;

/* State numbers in computations.  */
typedef int yy_state_fast_t;

#ifndef YY_
# if defined YYENABLE_NLS && YYENABLE_NLS
#  if ENABLE_NLS
#   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
#   define YY_(Msgid) dgettext ("bison-runtime", Msgid)
#  endif
# endif
# ifndef YY_
#  define YY_(Msgid) Msgid
# endif
#endif


#ifndef YY_ATTRIBUTE_PURE
# if defined __GNUC__ && 2 < __GNUC__ + (96 <= __GNUC_MINOR__)
#  define YY_ATTRIBUTE_PURE __attribute__ ((__pure__))
# else
#  define YY_ATTRIBUTE_PURE
# endif
#endif

#ifndef YY_ATTRIBUTE_UNUSED
# if defined __GNUC__ && 2 < __GNUC__ + (7 <= __GNUC_MINOR__)
#  define YY_ATTRIBUTE_UNUSED __attribute__ ((__unused__))
# else
#  define YY_ATTRIBUTE_UNUSED
# endif
#endif

/* Suppress unused-variable warnings by "using" E.  */
#if ! defined lint || defined __GNUC__
# define YY_USE(E) ((void) (E))
#else
# define YY_USE(E) /* empty */
#endif

/* Suppress an incorrect diagnostic about yylval being uninitialized.  */
#if defined __GNUC__ && ! defined __ICC && 406 <= __GNUC__ * 100 + __GNUC_MINOR__
# if __GNUC__ * 100 + __GNUC_MINOR__ < 407
#  define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                           \
    _Pragma ("GCC diagnostic push")                                     \
    _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")
# else
#  define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                           \
    _Pragma ("GCC diagnostic push")                                     \
    _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")              \
    _Pragma ("GCC diagnostic ignored \"-Wmaybe-uninitialized\"")
# endif
# define YY_IGNORE_MAYBE_UNINITIALIZED_END      \
    _Pragma ("GCC diagnostic pop")
#else
# define YY_INITIAL_VALUE(Value) Value
#endif
#ifndef YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
# define YY_IGNORE_MAYBE_UNINITIALIZED_END
#endif
#ifndef YY_INITIAL_VALUE
# define YY_INITIAL_VALUE(Value) /* Nothing. */
#endif

#if defined __cplusplus && defined __GNUC__ && ! defined __ICC && 6 <= __GNUC__
# define YY_IGNORE_USELESS_CAST_BEGIN                          \
    _Pragma ("GCC diagnostic push")                            \
    _Pragma ("GCC diagnostic ignored \"-Wuseless-cast\"")
# define YY_IGNORE_USELESS_CAST_END            \
    _Pragma ("GCC diagnostic pop")
#endif
#ifndef YY_IGNORE_USELESS_CAST_BEGIN
# define YY_IGNORE_USELESS_CAST_BEGIN
# define YY_IGNORE_USELESS_CAST_END
#endif


#define YY_ASSERT(E) ((void) (0 && (E)))

#if 1

/* The parser invokes alloca or malloc; define the necessary symbols.  */

# ifdef YYSTACK_USE_ALLOCA
#  if YYSTACK_USE_ALLOCA
#   ifdef __GNUC__
#    define YYSTACK_ALLOC __builtin_alloca
#   elif defined __BUILTIN_VA_ARG_INCR
#    include <alloca.h> /* INFRINGES ON USER NAME SPACE */
#   elif defined _AIX
#    define YYSTACK_ALLOC __alloca
#   elif defined _MSC_VER
#    include <malloc.h> /* INFRINGES ON USER NAME SPACE */
#    define alloca _alloca
#   else
#    define YYSTACK_ALLOC alloca
#    if ! defined _ALLOCA_H && ! defined EXIT_SUCCESS
#     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
      /* Use EXIT_SUCCESS as a witness for stdlib.h.  */
#     ifndef EXIT_SUCCESS
#      define EXIT_SUCCESS 0
#     endif
#    endif
#   endif
#  endif
# endif

# ifdef YYSTACK_ALLOC
   /* Pacify GCC's 'empty if-body' warning.  */
#  define YYSTACK_FREE(Ptr) do { /* empty */; } while (0)
#  ifndef YYSTACK_ALLOC_MAXIMUM
    /* The OS might guarantee only one guard page at the bottom of the stack,
       and a page size can be as small as 4096 bytes.  So we cannot safely
       invoke alloca (N) if N exceeds 4096.  Use a slightly smaller number
       to allow for a few compiler-allocated temporary stack slots.  */
#   define YYSTACK_ALLOC_MAXIMUM 4032 /* reasonable circa 2006 */
#  endif
# else
#  define YYSTACK_ALLOC YYMALLOC
#  define YYSTACK_FREE YYFREE
#  ifndef YYSTACK_ALLOC_MAXIMUM
#   define YYSTACK_ALLOC_MAXIMUM YYSIZE_MAXIMUM
#  endif
#  if (defined __cplusplus && ! defined EXIT_SUCCESS \
       && ! ((defined YYMALLOC || defined malloc) \
             && (defined YYFREE || defined free)))
#   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
#   ifndef EXIT_SUCCESS
#    define EXIT_SUCCESS 0
#   endif
#  endif
#  ifndef YYMALLOC
#   define YYMALLOC malloc
#   if ! defined malloc && ! defined EXIT_SUCCESS
void *malloc (YYSIZE_T); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
#  ifndef YYFREE
#   define YYFREE free
#   if ! defined free && ! defined EXIT_SUCCESS
void free (void *); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
# endif
#endif /* 1 */

#if (! defined yyoverflow \
     && (! defined __cplusplus \
         || (defined YYLTYPE_IS_TRIVIAL && YYLTYPE_IS_TRIVIAL \
             && defined YYSTYPE_IS_TRIVIAL && YYSTYPE_IS_TRIVIAL)))

/* A type that is properly aligned for any stack member.  */
union yyalloc
{
  yy_state_t yyss_alloc;
  YYSTYPE yyvs_alloc;
  YYLTYPE yyls_alloc;
};

/* The size of the maximum gap between one aligned stack and the next.  */
# define YYSTACK_GAP_MAXIMUM (YYSIZEOF (union yyalloc) - 1)

/* The size of an array large to enough to hold all stacks, each with
   N elements.  */
# define YYSTACK_BYTES(N) \
     ((N) * (YYSIZEOF (yy_state_t) + YYSIZEOF (YYSTYPE) \
             + YYSIZEOF (YYLTYPE)) \
      + 2 * YYSTACK_GAP_MAXIMUM)

# define YYCOPY_NEEDED 1

/* Relocate STACK from its old location to the new one.  The
   local variables YYSIZE and YYSTACKSIZE give the old and new number of
   elements in the stack, and YYPTR gives the new location of the
   stack.  Advance YYPTR to a properly aligned location for the next
   stack.  */
# define YYSTACK_RELOCATE(Stack_alloc, Stack)                           \
    do                                                                  \
      {                                                                 \
        YYPTRDIFF_T yynewbytes;                                         \
        YYCOPY (&yyptr->Stack_alloc, Stack, yysize);                    \
        Stack = &yyptr->Stack_alloc;                                    \
        yynewbytes = yystacksize * YYSIZEOF (*Stack) + YYSTACK_GAP_MAXIMUM; \
        yyptr += yynewbytes / YYSIZEOF (*yyptr);                        \
      }                                                                 \
    while (0)

#endif

#if defined YYCOPY_NEEDED && YYCOPY_NEEDED
/* Copy COUNT objects from SRC to DST.  The source and destination do
   not overlap.  */
# ifndef YYCOPY
#  if defined __GNUC__ && 1 < __GNUC__
#   define YYCOPY(Dst, Src, Count) \
      __builtin_memcpy (Dst, Src, YY_CAST (YYSIZE_T, (Count)) * sizeof (*(Src)))
#  else
#   define YYCOPY(Dst, Src, Count)              \
      do                                        \
        {                                       \
          YYPTRDIFF_T yyi;                      \
          for (yyi = 0; yyi < (Count); yyi++)   \
            (Dst)[yyi] = (Src)[yyi];            \
        }                                       \
      while (0)
#  endif
# endif
#endif /* !YYCOPY_NEEDED */

/* YYFINAL -- State number of the termination state.  */
#define YYFINAL  5
/* YYLAST -- Last index in YYTABLE.  */
#define YYLAST   4962

/* YYNTOKENS -- Number of terminals.  */
#define YYNTOKENS  131
/* YYNNTS -- Number of nonterminals.  */
#define YYNNTS  205
/* YYNRULES -- Number of rules.  */
#define YYNRULES  617
/* YYNSTATES -- Number of states.  */
#define YYNSTATES  913

/* YYMAXUTOK -- Last valid token kind.  */
#define YYMAXUTOK   368


/* YYTRANSLATE(TOKEN-NUM) -- Symbol number corresponding to TOKEN-NUM
   as returned by yylex, with out-of-bounds checking.  */
#define YYTRANSLATE(YYX)                                \
  (0 <= (YYX) && (YYX) <= YYMAXUTOK                     \
   ? YY_CAST (yysymbol_kind_t, yytranslate[YYX])        \
   : YYSYMBOL_YYUNDEF)

/* YYTRANSLATE[TOKEN-NUM] -- Symbol number corresponding to TOKEN-NUM
   as returned by yylex.  */
static const yytype_uint8 yytranslate[] =
{
       0,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     8,     2,     2,     2,     2,     2,     9,
       2,     2,   126,   120,    10,   119,   122,   128,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     5,     2,
       2,   125,     2,   130,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,   129,     2,   127,     6,     7,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,   123,     2,   124,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     1,     2,     3,     4,
      11,    12,    13,    14,    15,    16,    17,    18,    19,    20,
      21,    22,    23,    24,    25,    26,    27,    28,    29,    30,
      31,    32,    33,    34,    35,    36,    37,    38,    39,    40,
      41,    42,    43,    44,    45,    46,    47,    48,    49,    50,
      51,    52,    53,    54,    55,    56,    57,    58,    59,    60,
      61,    62,    63,    64,    65,    66,    67,    68,    69,    70,
      71,    72,    73,    74,    75,    76,    77,    78,    79,    80,
      81,    82,    83,    84,    85,    86,    87,    88,    89,    90,
      91,    92,    93,    94,    95,    96,    97,    98,    99,   100,
     101,   102,   103,   104,   105,   106,   107,   108,   109,   110,
     111,   112,   113,   114,   115,   116,   117,   118,   121
};

#if YYDEBUG
/* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
static const yytype_int16 yyrline[] =
{
       0,   409,   409,   410,   410,   418,   419,   421,   423,   426,
     433,   436,   440,   444,   452,   456,   460,   464,   468,   472,
     507,   523,   524,   529,   530,   566,   570,   574,   575,   576,
     580,   581,   585,   586,   587,   588,   589,   590,   591,   592,
     593,   594,   595,   601,   602,   603,   607,   608,   609,   613,
     620,   624,   625,   633,   640,   641,   642,   647,   659,   677,
     681,   689,   690,   693,   697,   706,   714,   714,   724,   724,
     735,   739,   746,   750,   755,   770,   771,   772,   776,   780,
     796,   797,   798,   799,   800,   801,   802,   803,   804,   805,
     806,   807,   808,   809,   810,   820,   824,   828,   832,   836,
     840,   844,   848,   852,   859,   860,   870,   880,   890,   900,
     904,   908,   912,   917,   927,   933,   942,   955,   957,   957,
     969,   972,   982,   993,  1021,  1033,  1043,  1046,  1050,  1054,
    1058,  1062,  1066,  1070,  1074,  1104,  1103,  1107,  1116,  1124,
    1129,  1132,  1135,  1139,  1143,  1147,  1151,  1155,  1186,  1185,
    1192,  1200,  1207,  1224,  1230,  1231,  1232,  1237,  1240,  1301,
    1300,  1323,  1337,  1347,  1353,  1358,  1367,  1379,  1380,  1384,
    1391,  1395,  1423,  1430,  1429,  1446,  1452,  1455,  1458,  1467,
    1470,  1474,  1484,  1490,  1491,  1492,  1496,  1508,  1509,  1510,
    1514,  1518,  1533,  1551,  1552,  1556,  1560,  1567,  1573,  1576,
    1582,  1585,  1593,  1594,  1595,  1600,  1599,  1607,  1626,  1627,
    1635,  1649,  1652,  1665,  1668,  1672,  1686,  1689,  1695,  1698,
    1720,  1721,  1722,  1723,  1724,  1727,  1734,  1735,  1736,  1740,
    1747,  1751,  1758,  1759,  1760,  1767,  1768,  1769,  1773,  1774,
    1831,  1832,  1836,  1843,  1844,  1848,  1857,  1858,  1870,  1874,
    1879,  1888,  1895,  1896,  1900,  1912,  1918,  1924,  1931,  1935,
    1945,  1949,  1953,  1957,  1961,  1965,  1973,  1987,  2004,  2015,
    2026,  2043,  2060,  2073,  2096,  2114,  2115,  2124,  2134,  2150,
    2166,  2175,  2202,  2213,  2223,  2242,  2253,  2263,  2277,  2292,
    2307,  2322,  2332,  2337,  2342,  2347,  2354,  2355,  2362,  2363,
    2382,  2386,  2390,  2394,  2401,  2405,  2409,  2413,  2420,  2423,
    2430,  2435,  2440,  2445,  2450,  2455,  2465,  2470,  2475,  2480,
    2487,  2505,  2516,  2527,  2545,  2563,  2577,  2600,  2612,  2628,
    2637,  2638,  2642,  2646,  2647,  2654,  2662,  2663,  2675,  2675,
    2682,  2686,  2697,  2708,  2713,  2718,  2723,  2730,  2737,  2741,
    2746,  2752,  2759,  2766,  2772,  2775,  2778,  2789,  2818,  2825,
    2837,  2838,  2842,  2846,  2854,  2855,  2857,  2858,  2859,  2861,
    2862,  2863,  2864,  2868,  2869,  2872,  2875,  2892,  2900,  2915,
    2938,  2970,  2988,  2997,  3000,  3009,  3020,  3021,  3022,  3026,
    3027,  3028,  3029,  3040,  3041,  3042,  3054,  3057,  3063,  3079,
    3080,  3081,  3082,  3090,  3094,  3095,  3099,  3106,  3113,  3117,
    3121,  3122,  3127,  3145,  3151,  3158,  3159,  3166,  3192,  3202,
    3218,  3222,  3223,  3224,  3228,  3235,  3238,  3244,  3247,  3259,
    3264,  3269,  3279,  3288,  3295,  3305,  3306,  3313,  3321,  3322,
    3325,  3330,  3334,  3337,  3353,  3354,  3359,  3362,  3363,  3368,
    3375,  3378,  3384,  3387,  3393,  3408,  3419,  3420,  3431,  3432,
    3436,  3441,  3447,  3454,  3462,  3470,  3473,  3476,  3479,  3482,
    3485,  3488,  3494,  3495,  3496,  3500,  3503,  3511,  3519,  3527,
    3533,  3539,  3542,  3548,  3549,  3554,  3557,  3564,  3572,  3585,
    3588,  3600,  3601,  3602,  3603,  3604,  3619,  3623,  3634,  3638,
    3667,  3675,  3678,  3681,  3684,  3690,  3693,  3700,  3703,  3709,
    3710,  3718,  3777,  3797,  3804,  3820,  3825,  3830,  3835,  3839,
    3849,  3855,  3862,  3865,  3871,  3874,  3877,  3880,  3886,  3889,
    3892,  3898,  3899,  3905,  3908,  3911,  3917,  3920,  3932,  3935,
    3938,  3942,  3945,  3948,  3954,  3957,  3961,  3962,  3963,  3970,
    3971,  3972,  3978,  3979,  3988,  4001,  4002,  4011,  4023,  4024,
    4040,  4041,  4052,  4051,  4060,  4076,  4085,  4089,  4096,  4104,
    4107,  4112,  4119,  4118,  4135,  4138,  4142,  4145,  4148,  4152,
    4152,  4168,  4175,  4181,  4182,  4185,  4191,  4197,  4200,  4211,
    4211,  4232,  4235,  4238,  4241,  4247,  4248,  4254,  4258,  4266,
    4270,  4274,  4280,  4284,  4288,  4295,  4301,  4304,  4310,  4311,
    4315,  4319,  4323,  4323,  4336,  4339,  4346,  4351
};
#endif

/** Accessing symbol of state STATE.  */
#define YY_ACCESSING_SYMBOL(State) YY_CAST (yysymbol_kind_t, yystos[State])

#if 1
/* The user-facing name of the symbol whose (internal) number is
   YYSYMBOL.  No bounds checking.  */
static const char *yysymbol_name (yysymbol_kind_t yysymbol) YY_ATTRIBUTE_UNUSED;

/* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
   First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
static const char *const yytname[] =
{
  "\"end of input\"", "error", "\"invalid token\"", "PREC_BOT",
  "COMPOSITE", "':'", "'^'", "'_'", "'!'", "'\\''", "','", "PREC_TOP",
  "\"\\\\accepts\"", "\"\\\\addlyrics\"", "\"\\\\alias\"",
  "\"\\\\alternative\"", "\"\\\\book\"", "\"\\\\bookpart\"",
  "\"\\\\change\"", "\"\\\\chordmode\"", "\"\\\\chords\"",
  "\"\\\\consists\"", "\"\\\\context\"", "\"\\\\default\"",
  "\"\\\\defaultchild\"", "\"\\\\denies\"", "\"\\\\description\"",
  "\"\\\\drummode\"", "\"\\\\drums\"", "\"\\\\etc\"", "\"\\\\figuremode\"",
  "\"\\\\figures\"", "\"\\\\header\"", "\"\\\\version-error\"",
  "\"\\\\layout\"", "\"\\\\lyricmode\"", "\"\\\\lyrics\"",
  "\"\\\\lyricsto\"", "\"\\\\markup\"", "\"\\\\markuplist\"",
  "\"\\\\midi\"", "\"\\\\name\"", "\"\\\\notemode\"", "\"\\\\override\"",
  "\"\\\\paper\"", "\"\\\\remove\"", "\"\\\\repeat\"", "\"\\\\rest\"",
  "\"\\\\revert\"", "\"\\\\score\"", "\"\\\\score-lines\"",
  "\"\\\\sequential\"", "\"\\\\set\"", "\"\\\\simultaneous\"",
  "\"\\\\tempo\"", "\"\\\\type\"", "\"\\\\unset\"", "\"\\\\with\"",
  "\"\\\\new\"", "\"/+\"", "\"^\"", "\":\"", "\"-\"", "\"/\"", "\"<\"",
  "\">\"", "\"<<\"", "\">>\"", "\"\\\\\\\\\"", "\"\\\\!\"", "\"\\\\+\"",
  "\"__\"", "FIGURE_CLOSE", "FIGURE_OPEN", "\"_\"",
  "FIGURE_ALTERATION_EXPR", "\"--\"", "MULTI_MEASURE_REST", "E_UNSIGNED",
  "UNSIGNED", "\"markup?\"", "\"scheme?\"", "\"(backed-up?)\"",
  "\"(reparsed?)\"", "\"markup-list?\"", "\"optional?\"",
  "EXPECT_NO_MORE_ARGS", "\"#{\"", "BOOK_IDENTIFIER", "CHORD_MODIFIER",
  "CHORD_REPETITION", "DRUM_PITCH", "DURATION_ARG", "DURATION_IDENTIFIER",
  "EVENT_IDENTIFIER", "EVENT_FUNCTION", "FRACTION", "LOOKUP_IDENTIFIER",
  "LYRIC_ELEMENT", "MARKUP_FUNCTION", "MARKUP_LIST_FUNCTION",
  "MARKUP_IDENTIFIER", "MARKUPLIST_IDENTIFIER", "MODULE_IDENTIFIER",
  "MUSIC_FUNCTION", "MUSIC_IDENTIFIER", "NOTENAME_PITCH",
  "NUMBER_IDENTIFIER", "PITCH_IDENTIFIER", "REAL", "RESTNAME", "SCM_ARG",
  "SCM_FUNCTION", "SCM_IDENTIFIER", "SCM_TOKEN", "STRING", "SYMBOL_LIST",
  "TONICNAME_PITCH", "SYMBOL", "'-'", "'+'", "UNARY_MINUS", "'.'", "'{'",
  "'}'", "'='", "'*'", "']'", "'/'", "'['", "'?'", "$accept",
  "start_symbol", "$@1", "lilypond", "toplevel_expression", "lookup",
  "embedded_scm_bare", "embedded_scm_active", "embedded_scm_bare_arg",
  "embedded_scm", "embedded_scm_arg", "scm_function_call",
  "embedded_lilypond_number", "embedded_lilypond", "lilypond_header_body",
  "lilypond_header", "header_block", "$@2", "header_modification", "$@3",
  "assignment_id", "assignment", "identifier_init",
  "identifier_init_nonumber", "partial_function_scriptable",
  "partial_function", "context_def_spec_block", "context_mod_arg", "$@4",
  "context_def_spec_body", "book_block", "book_body", "$@5",
  "bookpart_block", "bookpart_body", "$@6", "score_block", "score_body",
  "score_item", "score_items", "$@7", "paper_block", "output_def",
  "output_def_head", "output_def_head_with_mode_switch",
  "music_or_context_def", "output_def_body", "$@8", "tempo_event",
  "music_list", "braced_music_list", "music", "pitch_as_music",
  "music_embedded", "music_embedded_backup", "music_assign",
  "repeated_music", "alternative_music", "sequential_music",
  "simultaneous_music", "simple_music", "context_modification", "$@9",
  "context_modification_arg", "optional_context_mods",
  "context_modification_mods_list", "context_mod_list", "context_prefix",
  "new_lyrics", "basic_music", "contextable_music",
  "contexted_basic_music", "composite_music", "music_bare",
  "grouped_music_list", "symbol_list_arg", "symbol_list_rev",
  "symbol_list_part", "symbol_list_element", "symbol_list_part_bare",
  "function_arglist_nonbackup", "reparsed_rhythm",
  "function_arglist_nonbackup_reparse", "function_arglist_backup",
  "function_arglist", "function_arglist_skip_nonbackup",
  "function_arglist_partial", "function_arglist_partial_optional",
  "function_arglist_common", "function_arglist_common_reparse",
  "function_arglist_optional", "function_arglist_skip_backup",
  "music_function_call", "optional_id", "lyric_mode_music", "$@10",
  "mode_changed_music", "mode_changing_head",
  "mode_changing_head_with_context", "context_change", "property_path",
  "property_operation", "revert_arg", "revert_arg_backup",
  "revert_arg_part", "context_def_mod", "context_mod", "grob_prop_spec",
  "grob_prop_path", "context_prop_spec", "simple_revert_context",
  "music_property_def", "string", "text", "simple_string", "symbol",
  "scalar", "event_chord", "note_chord_element", "chord_body",
  "chord_body_elements", "chord_body_element", "music_function_chord_body",
  "event_function_event", "post_events", "post_event_nofinger",
  "post_event", "string_number_event", "direction_less_event",
  "direction_reqd_event", "octave_check", "quotes", "erroneous_quotes",
  "sup_quotes", "sub_quotes", "steno_pitch", "steno_tonic_pitch", "pitch",
  "pitch_or_tonic_pitch", "gen_text_def", "fingering",
  "script_abbreviation", "script_dir", "maybe_notemode_duration",
  "optional_notemode_duration", "steno_duration", "duration", "dots",
  "multiplier_scm", "multipliers", "tremolo_type", "bass_number",
  "bass_figure", "figured_bass_modification", "br_bass_figure",
  "figure_list", "optional_rest", "pitch_or_music", "simple_element",
  "lyric_element", "lyric_element_music", "new_chord", "chord_items",
  "chord_separator", "chord_item", "step_numbers", "step_number",
  "tempo_range", "number_expression", "number_term", "number_factor",
  "bare_number_common", "bare_number", "exact_unsigned_number",
  "unsigned_integer", "exclamations", "questions", "full_markup_list",
  "$@11", "markup_mode", "markup_mode_word", "full_markup",
  "partial_markup", "markup_top", "markup_scm", "$@12", "markup_list",
  "markup_uncomposed_list", "$@13", "markup_composed_list",
  "markup_braced_list", "markup_braced_list_body", "markup_command_list",
  "markup_command_basic_arguments", "$@14",
  "markup_command_list_arguments", "markup_partial_function",
  "markup_arglist_partial", "markup_head_1_item", "markup_head_1_list",
  "markup_word", "simple_markup", "simple_markup_noword", "$@15", "markup", YY_NULLPTR
};

static const char *
yysymbol_name (yysymbol_kind_t yysymbol)
{
  return yytname[yysymbol];
}
#endif

#define YYPACT_NINF (-647)

#define yypact_value_is_default(Yyn) \
  ((Yyn) == YYPACT_NINF)

#define YYTABLE_NINF (-603)

#define yytable_value_is_error(Yyn) \
  0

/* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
   STATE-NUM.  */
static const yytype_int16 yypact[] =
{
      95,  -647,   167,  2098,  1642,  -647,  -647,  1292,   -21,    68,
    -647,  -647,   592,  -647,  -647,  -647,  -647,  -647,  -647,  -647,
    -647,   736,  -647,  -647,  -647,  -647,  -647,   845,    73,   105,
     105,   592,  -647,  -647,   112,   140,   231,  -647,   231,  -647,
    -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,   216,
     182,  -647,  -647,  -647,  -647,  -647,  -647,   149,  1982,  -647,
    -647,  -647,  -647,  -647,  4839,   273,  -647,  -647,  -647,  -647,
    -647,  -647,   237,  -647,  -647,  1058,   173,  -647,  -647,   244,
     249,   249,   592,   716,  1402,   549,   845,   549,   549,   401,
     549,   556,  -647,  -647,  -647,  -647,    33,  -647,   230,    33,
      33,  -647,  -647,   406,  -647,  -647,   460,   120,  -647,   120,
     242,    33,   473,  -647,  -647,    62,   120,    75,    56,  -647,
    -647,    76,  -647,  -647,  -647,  -647,   216,  -647,  -647,   322,
    -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,  3728,   289,
      80,  -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,    33,
    -647,  -647,    90,  -647,  -647,  -647,    33,  -647,  4125,  -647,
    -647,  -647,    92,  -647,    33,  -647,  -647,  -647,  -647,  3981,
    -647,    94,  -647,  -647,   316,   320,  -647,  -647,  -647,   317,
     324,   328,   334,   342,   354,  1103,  -647,  -647,  -647,   889,
    -647,  -647,  -647,   317,  2348,   549,   549,   488,   400,  -647,
    -647,   412,  -647,  -647,   439,  -647,  1872,   360,  -647,   549,
    3613,  -647,  -647,  -647,  -647,  -647,  -647,  4773,   326,   173,
    -647,  -647,   273,  -647,  -647,  -647,  -647,   476,  -647,   237,
     471,  -647,  -647,   574,   605,  -647,  -647,  -647,  -647,  -647,
     121,  -647,  -647,  -647,  -647,  -647,  -647,  1058,  -647,  -647,
    3613,  -647,  -647,  -647,  -647,   404,  -647,  -647,  -647,  -647,
    -647,  -647,   134,  -647,  -647,  -647,   549,   411,   967,  -647,
     549,   134,   414,  -647,  -647,  -647,  -647,    33,   417,  -647,
    -647,  -647,  -647,   425,  -647,  4245,  1016,  -647,  -647,  -647,
    -647,   428,  -647,  -647,   428,   505,   472,  -647,  -647,   505,
     481,  -647,  -647,  -647,  -647,   558,   555,  -647,  -647,  -647,
     505,   487,  -647,  -647,   230,  -647,  -647,  -647,  -647,   549,
     549,   231,  -647,  -647,   249,  -647,  -647,  -647,  -647,  4189,
    -647,  -647,   356,  -647,  1024,  1033,   568,  -647,  -647,  -647,
    -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,
    -647,  -647,  -647,  3981,  -647,   453,   949,   174,  -647,   174,
     672,   552,  4215,  -647,  2138,  -647,  2238,   845,  -647,  -647,
    -647,   237,   845,   504,   485,  -647,  1103,  -647,  -647,  -647,
    4679,    84,  4573,  -647,  -647,  -647,  -647,   134,   134,   508,
     172,  -647,  3498,   514,   521,  1960,  2200,  -647,  -647,   134,
     479,   135,   141,   156,   227,  -647,  -647,  -647,  -647,   158,
    -647,  -647,   277,  -647,    30,  -647,  -647,  -647,  -647,  -647,
    -647,   476,   342,  -647,  -647,  -647,   482,   489,   605,   605,
     605,  -647,  -647,   605,  -647,  -647,  4156,   529,  -647,  -647,
    -647,  -647,  -647,  -647,  -647,  -647,   845,  4320,   549,   549,
    -647,  3842,  -647,  -647,   549,   845,   549,   234,   283,  -647,
    -647,    81,  4070,  -647,  -647,   532,  -647,  3956,   493,   945,
    -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,
    -647,  -647,  -647,  -647,  -647,  -647,  -647,   681,  -647,  -647,
     700,  -647,  -647,   174,  -647,   174,   505,   539,  -647,  2923,
     560,  2923,   560,  -647,  -647,  2923,   560,   510,   511,  1522,
     174,  -647,   174,   535,  -647,  -647,   535,  -647,  -647,     2,
    -647,   386,  -647,   174,   717,   717,   717,  -647,  -647,   672,
    -647,  -647,  -647,  -647,   216,  -647,  -647,  -647,  -647,  -647,
    -647,  -647,  -647,  -647,  -647,  -647,   216,  -647,  -647,  -647,
    -647,  -647,  -647,  -647,  -647,   342,   623,  -647,  -647,  -647,
    -647,   216,  -647,  -647,  -647,   607,  1152,  -647,  -647,  -647,
    -647,  -647,  -647,  -647,  -647,   286,  -647,  -647,  -647,  -647,
    -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,
    -647,  -647,  -647,  -647,  -647,  -647,   593,  3038,   626,  -647,
    -647,   159,  -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,
    -647,  -647,  -647,  -647,  -647,   826,  3613,  -647,   575,   220,
    -647,   549,   549,   174,   220,   220,   220,   220,  -647,   342,
    -647,  -647,  -647,  1259,  1103,  1218,  -647,  -647,  1058,  -647,
    -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,
     549,  -647,   549,  -647,   592,  -647,   334,  -647,  -647,  1132,
    -647,  -647,  1272,  -647,   550,  -647,  -647,  -647,   227,  -647,
    -647,   162,  -647,  -647,  -647,  -647,   572,   967,   583,   505,
     600,   505,   618,   305,   612,  -647,    37,  -647,  -647,   945,
    -647,  -647,  -647,  -647,   601,  4357,   174,   568,   700,  -647,
    -647,  -647,  -647,  -647,  -647,  -647,  -647,  3153,   829,  -647,
     505,   647,  2578,  -647,  -647,  2578,  -647,  2578,  4467,  4467,
    -647,  -647,   784,   120,  -647,    33,  -647,  -647,  -647,  -647,
    -647,   717,   703,  -647,  1259,  -647,  1103,  -647,  -647,  -647,
    -647,  -647,  -647,  -647,   659,  2693,   718,  -647,  -647,  -647,
    -647,  -647,  -647,  -647,  -647,   521,  -647,  -647,  -647,  -647,
    -647,   303,  -647,  -647,  -647,  -647,  -647,  -647,  -647,   428,
     549,   549,  -647,  -647,  -647,  -647,  -647,  -647,  -647,  -647,
    -647,  -647,  -647,  -647,   102,   117,  -647,  -647,  -647,  -647,
    -647,  -647,   642,  1218,  -647,  -647,   634,  -647,  -647,  -647,
    -647,  -647,  -647,  4467,  -647,   549,   549,   941,   941,   941,
     941,   850,   941,   850,   505,   667,   806,   662,   565,  -647,
    -647,   945,  -647,  -647,  -647,     2,   682,  2463,  -647,  3268,
     560,  -647,  -647,  -647,   227,    34,  -647,  -647,  -647,   649,
    -647,  -647,   120,  1218,   607,  -647,  -647,  -647,  -647,  -647,
    -647,  -647,   323,  -647,  -647,  -647,  -647,  -647,  -647,  -647,
    -647,  -647,  -647,   453,   134,   134,  -647,  -647,  1757,  4467,
    -647,  -647,  -647,   692,   941,   941,   941,   850,   717,   717,
     806,   549,   549,  -647,  -647,  -647,   829,  2808,  -647,  -647,
     695,  -647,   729,  -647,  -647,  -647,  -647,   174,   658,  -647,
     850,   941,  -647,  -647,  -647,  -647,   174,  -647,  -647,  -647,
    -647,  3383,   174
};

/* YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
   Performed when YYTABLE does not specify something else to do.  Zero
   means the default is an error.  */
static const yytype_int16 yydefact[] =
{
       5,     3,     0,     0,     0,     1,     8,     0,     0,     0,
     346,   350,     0,   344,   348,   345,   349,     9,   165,   347,
     351,     0,   564,   562,   164,   343,   163,     0,     0,     0,
       0,     0,   179,    13,    21,    23,     0,   236,     0,    27,
      18,    70,    71,   179,     6,    29,    19,    28,    10,     0,
       0,     7,    11,    12,    14,    20,   166,     0,     0,   199,
     221,   222,   239,   238,     0,   232,   233,    15,   223,   237,
     220,   235,     0,   211,    17,     0,   567,    16,    59,   489,
     473,   472,     0,     0,     0,     0,     0,     0,     0,     0,
       0,   205,   415,   430,   507,   429,   475,   437,   481,   475,
     475,   481,   439,     0,    88,   518,     0,   446,   547,   446,
     546,   475,     0,    26,    25,   517,   446,   516,   474,    45,
      43,   192,    44,    55,     4,    80,     0,    54,   104,     0,
      85,    82,    83,    81,    84,   410,   187,   185,     0,   189,
     183,   193,    93,   194,   204,   203,    89,   202,   411,   475,
     441,   435,   188,   438,   427,   456,   475,   558,   115,   485,
     425,   440,   186,   425,   475,   184,   425,    51,    92,     0,
     567,   515,    91,   197,   125,   139,   396,   397,   398,   336,
     393,   394,   395,   338,     0,     0,   393,   394,   395,     0,
     157,   198,   200,   336,     0,     0,     0,     0,     0,   308,
     296,     0,   335,   252,     0,    49,     0,     0,    67,     0,
      68,   169,   175,   172,   162,   171,   170,     0,     0,     0,
     227,   226,   230,   231,   228,   186,   211,   234,   341,     0,
     210,   612,   579,     0,     0,   608,   609,   583,   572,   566,
       0,   569,   575,   574,   576,   577,   606,     0,   565,   571,
      68,   490,   464,   433,   434,     0,   120,    60,   249,   248,
     250,   247,   377,   243,   251,   246,   378,     0,   111,   381,
       0,   380,     0,   481,   389,   390,   392,   178,     0,   391,
     385,   209,   208,     0,   207,     0,     0,   477,   425,   476,
      52,   478,   425,   513,   479,     0,     0,   424,    96,     0,
       0,    95,   450,   452,   454,   448,   447,   457,   548,   514,
       0,     0,    97,   455,   464,    50,   436,    69,    94,     0,
       0,     0,   517,   516,   474,   192,   179,   183,   188,     0,
     425,   515,     0,   425,   520,     0,   560,   465,   471,   468,
     469,   461,   462,   467,   466,   470,   463,   114,   428,   432,
     431,   442,   443,     0,   460,   480,   191,   407,   425,   512,
       0,     0,     0,   126,     0,   140,     0,     0,   211,   340,
     224,     0,     0,     0,     0,   563,     0,   555,   556,   557,
       0,     0,   152,   211,   181,   201,   180,    22,    24,     0,
     330,   275,    68,     0,     0,     0,     0,   182,    61,   353,
       0,   550,   386,   387,   474,    90,    72,    75,    86,     0,
     425,    87,    76,   540,   541,   549,   545,   388,   168,   174,
     167,   229,   338,   211,   342,   212,     0,     0,     0,     0,
       0,   594,   614,     0,   595,   586,     0,     0,   615,   578,
     581,   607,   610,   570,   611,    74,     0,   118,     0,     0,
     379,    68,   473,   472,     0,     0,     0,     0,     0,   474,
     112,   115,   109,   360,   383,     0,   359,    68,     0,     0,
     213,   413,   425,   422,   423,   421,   416,   419,   420,   459,
     458,   558,   414,   496,   491,   492,   493,     0,   495,   497,
     505,   508,   494,   409,   482,   408,     0,     0,   301,    68,
       0,    68,     0,   451,   453,    68,     0,     0,     0,     0,
     191,   190,   412,     0,   525,   524,     0,   522,   559,   444,
     113,     0,   426,   519,     0,     0,     0,   597,   568,     0,
     137,   133,   124,   134,     0,   128,   129,   127,   161,   130,
     132,   131,   150,   146,   138,   147,     0,   142,   141,   143,
     145,   144,   337,   216,   339,   338,   195,   153,   151,   154,
     158,     0,   156,   155,   217,     0,     0,   326,   327,    32,
      30,   320,    31,   323,   324,   474,    42,    46,   309,    47,
      36,    38,    39,    40,    37,    41,    48,    35,   311,   322,
     321,   310,    34,   325,    33,   332,     0,    68,     0,   481,
     263,   240,   265,   261,   260,   259,   264,   262,   313,   319,
     318,   317,   315,   314,   316,     0,    68,   551,   550,     0,
     544,     0,     0,    79,     0,     0,     0,     0,   218,   338,
     157,   157,   605,     0,     0,     0,   582,   585,     0,   617,
     584,   573,   352,   366,   369,   364,   367,   368,   371,   372,
       0,   365,     0,   370,     0,   116,    43,   117,   123,     0,
     122,   373,     0,   121,     0,   245,   244,   401,   474,   399,
     107,     0,   404,   382,   400,   402,     0,   111,     0,     0,
       0,     0,     0,     0,     0,   110,     0,   108,   384,     0,
     552,   553,   554,   176,   536,   118,   418,   560,   506,   504,
     502,   501,   499,   498,   503,   500,   305,    68,     0,    99,
       0,     0,    68,   303,    98,    68,   100,    68,    68,    68,
     527,   526,   521,   446,   561,   475,   486,   487,   483,   484,
     488,     0,   605,   599,   604,   601,   603,   600,   598,   136,
     149,   225,   196,   160,     0,   333,   334,   295,   294,   293,
     291,   292,   328,   312,   329,     0,   273,   274,   267,   270,
     271,   474,   257,   253,   269,   268,   258,   272,   297,   485,
       0,     0,    63,    65,    64,    62,    73,    78,    77,   539,
     538,   542,   543,   219,     0,     0,   592,   593,   589,   588,
     591,   587,     0,     0,   596,   616,     0,   357,   355,   119,
     374,   375,   376,    68,   403,     0,     0,   105,   109,   106,
     300,     0,   300,     0,     0,     0,     0,     0,     0,   358,
     177,     0,   206,   215,   214,   444,     0,   333,   307,    68,
       0,   102,   101,   103,     0,   533,   530,   528,   523,   529,
     531,   445,   446,   602,     0,   281,   287,   283,   282,   288,
     289,   290,   474,   276,   277,   279,   278,   280,   331,   299,
     254,   256,   255,   425,   242,   241,   613,   580,     0,    68,
     354,   406,   405,     0,   302,   302,   300,     0,     0,     0,
       0,     0,     0,   363,   537,   425,     0,    68,   535,   534,
       0,   449,   509,   334,   284,   286,   285,   266,     0,   356,
       0,   302,   604,   603,   362,   361,   417,   532,   510,   425,
     590,    68,   511
};

/* YYPGOTO[NTERM-NUM].  */
static const yytype_int16 yypgoto[] =
{
    -647,  -647,  -647,  -647,  -647,     6,   331,   -44,  -647,   372,
      93,    19,   665,   -79,  -647,  -100,  -647,  -647,    26,  -647,
    -647,   -30,  -179,  -171,  -111,   465,    15,    89,  -647,  -647,
      -1,  -647,  -647,    17,  -647,  -647,     3,  -274,  -647,  -647,
    -647,   422,     9,  -647,  -647,  -647,  -647,  -647,  -647,     7,
     437,  -259,   740,   -66,  -647,    23,  -647,   251,  -647,  -647,
    -647,    21,  -647,  -647,  -148,  -647,  -647,  -647,   571,   126,
    -647,   749,    13,  -647,   -17,  -351,   -25,   -26,  -647,  -119,
    -305,  -327,  -647,  -533,    10,  -646,    -6,  -450,    -8,  -647,
    -158,  -531,   -15,   630,  -363,  -647,  -551,  -647,  -647,  -647,
    -196,  -647,  -503,  -647,  -647,  -647,   130,  -647,  -211,   -41,
    -647,  -647,  -331,  -647,    88,    22,  -411,   768,  -647,  -647,
    -647,  -647,  -647,  -647,  -128,   398,     0,  -647,   -91,  -647,
      12,   -76,  -647,  -647,  -647,  -647,   517,   536,  -647,  -647,
     396,  -647,    50,  -132,   210,   -31,     4,   -64,  -647,    71,
    -647,  -647,   355,  -647,  -647,  -647,  -647,    11,  -647,  -647,
    -208,  -647,  -647,   131,  -647,  -647,   -20,   180,  -647,  -121,
    -330,    32,  -394,    53,  -239,   395,   195,    14,  -647,   918,
       8,    -3,    27,  -647,   -71,  -647,  -144,  -203,  -647,  -647,
    -647,  -647,  -647,  -182,  -647,   387,   732,  -184,  -237,   -52,
      -7,  -205,   -23,  -647,   462
};

/* YYDEFGOTO[NTERM-NUM].  */
static const yytype_int16 yydefgoto[] =
{
       0,     2,     4,     3,    44,   119,   120,    46,   577,   238,
     578,   122,   123,   124,   615,   208,    48,    49,   580,   126,
      50,    51,   406,   127,   128,   129,   581,   658,   659,   447,
     582,   364,   534,   583,   366,   546,   584,   381,   560,   382,
     561,   537,   585,    56,    57,   419,    58,   217,   135,   194,
      59,   136,   137,   138,   139,   586,    60,    61,    62,    63,
     141,   587,   283,   284,   229,   230,   695,    64,   227,    65,
     222,    66,   143,    68,    69,   602,   399,   263,   264,   265,
     200,   603,   201,   390,   202,   598,   713,   498,   203,   204,
     392,   393,    70,   368,   370,   371,    71,    72,    73,   144,
     400,   661,   464,   465,   466,   662,   663,   266,   267,   272,
     270,   145,   146,   277,   183,   179,   673,   147,   148,   149,
     285,   476,   477,   150,   356,   151,   522,   153,   154,   350,
     725,   304,   892,   305,   306,   155,   156,   157,   481,   351,
     316,   352,   329,   287,   288,   159,   289,   291,   730,   355,
     161,   489,   490,   705,   491,   286,   909,   225,   163,   164,
     165,   166,   722,   517,   838,   839,   840,   693,   412,   413,
     414,   415,   416,   694,   380,   336,   519,   592,   185,   169,
     170,   331,   594,   239,   240,   437,   241,   242,   427,   243,
     244,   436,   245,   434,   868,   435,   520,   527,   246,   376,
     442,   639,   444,   426,   794
};

/* YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
   positive, shift that token.  If negative, reduce the rule whose
   number is the opposite.  If YYTABLE_NINF, syntax error.  */
static const yytype_int16 yytable[] =
{
      77,   171,    52,   131,   152,   257,    54,   133,   160,    45,
     441,    76,    55,   134,   215,   162,    67,    74,   168,   130,
      53,   132,    47,   247,   334,   142,   317,   140,   216,   462,
     125,   172,   745,   307,   746,   357,   167,   294,   359,   407,
     313,   375,   443,   184,   440,   609,   706,   347,   205,   280,
     206,   432,   249,   193,   158,   228,   688,   675,   278,   628,
     262,   269,  -386,   271,    45,   271,   219,   349,   248,   610,
     450,   445,   326,   675,   620,  -387,   -90,    47,   422,   407,
     -86,   171,   790,   131,   152,   557,   279,   133,   160,   597,
     -56,   409,   -87,   134,  -388,   162,   888,   298,   168,   130,
     301,   132,   174,   557,   255,   142,   312,   140,   507,   859,
     125,   172,   273,   297,   374,   189,   167,   362,   557,    22,
     672,   556,   205,   563,   511,   441,   101,   723,   386,   302,
     303,   409,   724,   173,   158,   314,   672,   499,   328,   441,
     386,   501,   330,   348,   448,  -249,   249,   796,   818,   797,
     167,  -248,   505,   601,   889,   354,   626,   443,   627,   440,
     493,   327,   248,   108,   495,   110,  -250,     5,   621,   770,
     387,   388,   805,   440,   268,   827,   457,   746,   432,    79,
      80,    81,     1,   790,   859,   458,  -386,   606,   613,   391,
     221,   175,   741,   683,   328,   712,   190,   715,   330,  -387,
     -90,   717,   510,   556,   -86,   512,   328,   417,   558,   131,
     330,   248,   424,   133,   -56,   747,   -87,   327,  -388,   134,
     553,   411,   438,   439,   168,   130,   866,   132,    43,   327,
     523,   142,   418,   408,   195,   564,   125,   172,   349,   748,
     420,   867,   617,   676,   463,    93,   468,   417,   207,   131,
      95,   425,    97,   133,  -333,   566,   449,  -249,   828,   134,
     158,   411,   196,  -248,   168,   130,   783,   132,   102,   321,
     475,   142,   211,   408,   804,   629,   125,   172,  -250,   508,
     622,   771,   623,   492,   806,   478,   226,   391,    29,   620,
      30,   391,   637,   324,   262,   271,   781,   782,   250,   618,
     158,   362,   391,    32,   209,   374,   618,   210,   688,   292,
     293,   745,   197,   893,   348,   679,   198,   199,   461,   680,
     533,   309,   545,   251,   675,   675,   354,   108,   252,   110,
     249,   297,   671,   509,   108,   819,   110,   290,   707,   619,
     733,   735,   737,   178,   696,   738,   248,   432,   671,   308,
     347,   318,   182,   827,   554,   893,   784,   785,   188,   333,
      43,   541,   178,   551,   681,   752,   335,   536,   682,   547,
      45,   332,    45,   538,   358,   538,   121,   539,   540,   549,
     550,   535,   860,    47,   638,    47,   814,   672,   672,   593,
     815,   562,   870,   753,    22,   754,   624,   625,   576,   604,
     611,   441,   894,   327,   363,   327,   605,   612,   365,   675,
     861,   579,   862,   178,   178,   678,   261,   188,   261,   261,
     276,   261,   665,   666,   591,   777,   778,   607,   614,   262,
     895,   271,   896,   795,   739,   440,   828,   776,   808,    22,
     804,   235,   367,   386,   236,   407,   740,   369,   417,  -396,
     597,   298,   301,  -397,   105,   552,   121,   576,   899,  -398,
     555,   743,   674,   282,   417,   726,   191,   192,   660,   664,
     579,   322,   672,   576,   323,   675,   253,   254,   674,   372,
     273,   394,   727,   398,   492,   327,   579,   295,   391,   423,
     791,   296,   199,   728,   101,   395,   593,   409,   593,   113,
     114,   461,   593,   779,   780,   576,   874,   576,   875,   328,
     325,   576,   461,   330,   113,   114,   274,   461,   579,   275,
     579,   810,   396,   812,   579,   887,   261,   261,    91,   446,
     346,   591,   327,   591,   642,   769,   451,   591,   672,   467,
     261,   299,   469,   677,   669,   300,   199,   733,   470,   158,
     494,   158,   829,   500,   310,   158,   441,   391,   311,   199,
     669,   379,   502,   374,   792,   504,   325,   503,   506,   197,
     749,   774,   901,   389,   199,   881,   518,   750,   325,   521,
     441,   528,   405,   793,   428,   775,   496,   439,   795,   565,
     497,   199,   791,   842,   767,   911,   595,   261,   751,   671,
     671,   261,   197,   576,   616,   630,   596,   199,   410,   871,
     872,   641,   631,   417,   686,   131,   579,   488,   689,   133,
     708,    45,   405,   219,   463,   134,   463,   411,   258,   766,
     168,   130,   817,   132,    47,   718,   719,   142,     7,   408,
     379,   710,   125,   172,   258,   711,   199,   841,   410,   768,
     261,   261,   116,    34,   428,   429,   876,   474,   430,    35,
     431,   281,   113,   114,   259,   374,   158,   260,    38,   113,
     114,   391,   799,   391,   755,   803,   798,   312,   113,   114,
     259,   811,   617,   260,   671,   433,   429,   882,   197,   430,
     762,   431,   744,   199,   735,   737,   738,   807,   188,   813,
     391,   346,   391,   188,   593,   113,   114,   176,   809,   767,
     177,   816,   767,   576,   767,   417,   417,   664,   576,    22,
     821,   576,   792,   576,   576,   576,   579,   461,   830,   674,
     674,   579,  -602,   460,   579,   897,   579,   579,   579,   591,
     844,   858,   857,   438,   766,   864,   865,   766,   877,   766,
     671,   576,   524,   525,   559,   483,   526,   906,   431,   869,
     484,   880,   158,   886,   579,   158,   891,   158,   699,   700,
     701,   890,   792,   900,   835,   702,   908,   188,   656,   261,
     261,   912,   910,   315,   823,   261,   188,   261,   548,   898,
     588,   793,   883,   421,   113,   114,   485,   731,   525,   486,
     417,   526,   479,   431,   220,   762,   391,   742,   762,   576,
     762,   669,   669,   223,   674,   632,   633,   634,   488,   657,
     635,   480,   579,   383,   857,   824,   593,   703,   704,   113,
     114,   176,   224,   576,   177,   576,   391,   885,   853,   256,
     863,   692,   698,   513,   514,   515,   579,   516,   579,   113,
     114,   180,   729,   837,   181,   904,   905,   461,   461,   461,
     461,   591,   461,   835,    22,   171,   417,   131,   152,   820,
     907,   133,   160,   836,   884,   576,   697,   134,   391,   162,
     674,   325,   168,   130,   767,   132,   524,   878,   579,   142,
     879,   140,   825,   576,   125,   172,   669,   588,   640,   588,
     167,   361,     0,   588,     0,     0,   579,     0,   767,   589,
     496,   732,   734,   736,   826,   199,   670,   576,   158,   766,
     853,    75,     0,    34,   461,   461,   461,   685,   590,    35,
     579,   710,   687,     0,     0,   873,   199,     0,    38,    39,
     772,    41,     0,   766,    42,     0,     0,   452,   453,   -57,
     773,   461,   261,   261,    79,    80,    81,     0,   113,   114,
     186,     0,   669,   187,   709,     0,   714,     0,   377,     0,
     716,     0,     0,   452,   453,     0,   218,     0,     0,     0,
     762,   261,     0,   261,   454,   178,    34,   455,   405,     0,
       0,     0,    35,   456,     0,   763,   378,     0,     0,     0,
       0,    38,   113,   114,   762,   789,     0,    75,     0,     0,
     454,     0,     0,   455,   410,     0,   589,     0,   589,   456,
      93,     0,   589,     0,   690,    95,   656,    97,     0,     0,
     720,     0,     0,   721,   802,   590,   457,   590,     0,     0,
       0,   590,    34,   102,   321,   458,   377,     0,    35,   379,
       0,     0,   691,   683,    22,     0,    75,    38,   113,   114,
     459,   692,   457,     0,    34,     0,     0,   657,   324,     0,
      35,   458,     0,   -57,   378,     0,   353,     0,     0,   112,
     113,   114,     0,  -477,  -477,  -477,   459,  -477,   482,     0,
     483,     0,   513,   514,   515,   484,   516,     0,     0,     0,
       0,   261,   261,     0,     0,   588,   789,   231,   232,     0,
     763,     0,    75,   763,   764,   763,     0,     0,   843,     0,
       0,     0,     0,     0,    75,     0,     0,     0,     0,   113,
     114,   485,     0,   765,   486,     0,   261,   261,     0,     0,
       0,     0,   460,   854,     0,   487,     0,     7,     0,   261,
       0,    10,    11,   232,    12,    34,     0,   233,   234,    13,
      14,    35,    15,    16,     0,     0,     0,    19,    20,    21,
      38,   113,   114,   235,    25,     0,   236,   831,    27,     0,
     832,   237,   833,    29,     0,    30,     0,     0,     0,     0,
      31,     0,     0,   692,     0,     0,     0,     0,    32,     0,
      34,     0,   373,   234,    75,     0,    35,     0,     0,     0,
       0,     0,   261,   261,     0,    38,   113,   114,     0,     0,
       0,     0,     0,     0,   589,   854,   237,   588,     0,   764,
       0,   273,   764,     0,   764,     0,    36,    37,     0,     0,
     121,     0,     0,   590,   599,   101,     0,    75,   765,     0,
      75,   765,     0,   765,     0,    43,     0,     0,   107,   108,
     109,   110,   855,     0,     0,   902,   903,   231,   601,   116,
       0,     0,   670,   685,   687,   709,     0,   714,    10,    11,
       0,   856,    75,     0,    75,   763,    13,    14,     0,    15,
      16,     0,     0,     0,    19,    20,     0,     0,    75,     0,
      75,    25,     0,     0,     0,     0,     0,     7,     0,   763,
       0,    10,    11,    75,    75,    34,     0,   233,     0,    13,
      14,    35,    15,    16,     0,     0,     0,    19,    20,    21,
      38,   113,   114,   235,    25,     0,   236,     0,    27,   831,
     832,   716,     0,    29,   855,    30,   589,     0,     0,     0,
       0,     0,     0,     0,     0,     0,    34,     0,    32,     0,
       0,     0,    35,   856,   786,   590,   833,     0,     0,    34,
       0,    38,   113,   114,   787,    35,     0,     0,     0,   684,
      75,     0,   788,     0,    38,   113,   114,   800,     0,     0,
     801,     0,     0,     0,     0,     0,    36,    37,     0,     0,
       0,     0,   -53,    78,   764,    75,     0,    79,    80,    81,
       0,     0,     0,     0,     0,    43,     0,     7,     8,     9,
      82,    10,    11,   765,    83,     0,     0,    75,   764,    13,
      14,     0,    15,    16,   -68,    84,    18,    19,    20,    21,
      22,    23,    24,     0,    25,    85,    26,   765,    86,     0,
      87,    28,     0,    29,    88,    30,    89,     0,    90,    91,
      31,     0,     0,     0,     0,     0,    92,     0,    32,     0,
       0,     0,     0,    93,     0,    94,     0,     0,    95,    96,
      97,    98,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,    99,   100,     0,   101,   102,   103,   104,    34,
     105,     0,     0,     0,     0,    35,   106,    37,   107,   108,
     109,   110,   111,     0,   112,   113,   114,   115,     0,   116,
     117,   118,   -58,   384,     0,    43,   -53,    79,    80,    81,
       0,     0,     0,   218,     0,     0,     0,     7,     0,     0,
      82,    10,    11,     0,    12,     0,     0,     0,     0,    13,
      14,     0,    15,    16,     0,     0,     0,    19,    20,    21,
      22,     0,     0,     0,    25,   319,     0,     0,    27,     0,
      87,     0,     0,    29,   320,    30,    89,     0,    90,     0,
      31,     0,     0,     0,     0,     0,    92,     0,    32,     0,
       0,     0,     0,    93,     0,    94,     0,     0,    95,    96,
      97,   273,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,    99,   100,     0,   101,   102,   321,     0,    34,
     105,     0,     0,     0,     0,    35,    36,    37,   107,     0,
     109,     0,   111,     0,    38,   113,   114,   322,     0,   116,
     323,   324,   -53,    78,     0,    43,   -58,    79,    80,    81,
       0,     0,     0,     0,     0,     0,     0,     7,     8,     9,
      82,    10,    11,     0,    83,     0,     0,     0,     0,    13,
      14,     0,    15,    16,   -68,    84,    18,    19,    20,    21,
      22,    23,    24,     0,    25,    85,    26,     0,    86,     0,
      87,    28,     0,    29,    88,    30,    89,     0,    90,    91,
      31,     0,     0,     0,     0,     0,    92,     0,    32,     0,
       0,     0,     0,    93,     0,    94,     0,     0,    95,    96,
      97,    98,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,    99,   100,     0,   101,   102,   103,   104,    34,
     105,     0,     0,     0,     0,    35,   106,    37,   107,   108,
     109,   110,   111,     0,   112,   113,   114,   115,    78,   116,
     117,   118,    79,    80,    81,    43,     0,     0,     0,     0,
       0,     0,     7,     8,     9,    82,    10,    11,     0,    83,
       0,     0,     0,     0,    13,    14,     0,    15,    16,   -68,
      84,    18,    19,    20,    21,    22,    23,    24,     0,    25,
      85,    26,     0,    86,     0,    87,    28,     0,    29,    88,
      30,    89,     0,    90,    91,    31,     0,     0,     0,     0,
       0,    92,     0,    32,     0,     0,     0,     0,    93,     0,
      94,     0,     0,    95,    96,    97,    98,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    99,   100,     0,
     101,   102,   103,   104,    34,   105,     0,     0,     0,     0,
      35,   106,    37,   107,   108,   109,   110,   111,     0,   112,
     113,   114,   115,   384,   116,   117,   118,    79,    80,    81,
      43,   -53,     0,     0,     0,     0,     0,     7,     0,     0,
      82,    10,    11,     0,    12,     0,     0,     0,     0,    13,
      14,     0,    15,    16,     0,     0,     0,    19,    20,    21,
      22,     0,     0,     0,    25,   319,     0,     0,    27,     0,
      87,     0,     0,    29,   320,    30,    89,     0,    90,     0,
      31,     0,     0,     0,     0,     0,    92,     0,    32,     0,
       0,     0,     0,    93,     0,    94,     0,     0,    95,    96,
      97,   273,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,    99,   100,     0,   101,   102,   321,     0,    34,
     105,     0,     0,     0,     0,    35,    36,    37,   107,     0,
     109,     0,   111,   212,    38,   113,   114,   322,     0,   116,
     323,   324,     0,     0,     0,    43,   397,  -173,    22,     0,
    -173,  -173,  -173,     0,  -173,     0,     0,     0,     0,  -173,
    -173,     0,  -173,  -173,     0,     0,     0,  -173,  -173,  -173,
      22,     0,     0,     0,  -173,  -173,     0,     0,  -173,     0,
    -173,     0,     0,  -173,  -173,  -173,  -173,     0,  -173,   273,
    -173,     0,     0,     0,     0,     0,  -173,     0,  -173,     0,
       0,     0,   599,   101,     0,  -173,     0,     0,   105,  -173,
       0,     0,     0,     0,     0,     0,   107,   108,   109,   110,
       0,   600,  -173,  -173,     0,   322,   601,   116,   323,    34,
       0,     0,     0,     0,     0,    35,  -173,  -173,     0,     0,
       0,     0,  -173,     0,    38,    39,   213,    41,    -2,     6,
      42,     0,     0,     0,     0,  -173,   214,     0,     0,     0,
       0,     0,     0,     7,     8,     9,     0,    10,    11,     0,
      12,     0,     0,     0,     0,    13,    14,     0,    15,    16,
     -66,    17,    18,    19,    20,    21,    22,    23,    24,   530,
      25,     0,    26,     0,    27,     0,     0,    28,     0,    29,
       0,    30,     0,     7,     0,     9,    31,    10,    11,     0,
      12,     0,     0,     0,    32,    13,    14,     0,    15,    16,
    -135,     0,    18,    19,    20,    21,    22,    23,    24,     0,
      25,     0,    26,     0,    27,     0,    33,    28,     0,    29,
       0,    30,     0,     0,     0,    34,    31,     0,     0,     0,
       0,    35,    36,    37,    32,     0,     0,     0,     0,     0,
      38,    39,    40,    41,     0,     0,    42,     0,     0,     0,
       0,    43,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,    34,     0,     0,    22,   542,
       0,    35,    36,    37,     0,     0,     0,     0,     0,     0,
      38,    39,   531,     7,     0,     0,     0,    10,    11,     0,
      12,    43,   532,     0,     0,    13,    14,     0,    15,    16,
    -148,     0,    18,    19,    20,    21,    22,    23,    24,   273,
      25,     0,    26,     0,    27,     0,     0,    28,     0,    29,
       0,    30,   599,   101,     0,     0,    31,     0,   105,     0,
       0,     0,     0,     0,    32,     0,   107,   108,   109,   110,
       0,   608,     0,     0,     0,   322,   601,   116,   323,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,    34,     0,     0,     0,     0,
       0,    35,    36,    37,     0,     0,     0,     0,     0,   384,
      38,    39,   543,    79,    80,    81,     0,     0,     0,     0,
       0,    43,   544,     7,     0,     0,    82,    10,    11,     0,
      12,     0,     0,     0,     0,    13,    14,     0,    15,    16,
       0,     0,     0,    19,    20,    21,    22,     0,     0,     0,
      25,   319,     0,     0,    27,     0,    87,     0,     0,    29,
     320,    30,    89,     0,    90,     0,    31,     0,     0,     0,
       0,     0,    92,     0,    32,   385,     0,     0,     0,    93,
       0,    94,     0,     0,    95,    96,    97,   273,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    99,   100,
       0,   101,   102,   321,     0,    34,   105,     0,     0,     0,
       0,    35,    36,    37,   107,     0,   109,     0,   111,     0,
      38,   113,   114,   322,     0,   116,   323,   324,    79,    80,
      81,    43,     0,     0,     0,     0,     0,     0,     7,     8,
       9,    82,    10,    11,     0,    83,     0,     0,     0,     0,
      13,    14,  -306,    15,    16,   -68,     0,    18,    19,    20,
      21,    22,    23,    24,     0,    25,   319,    26,     0,    27,
       0,    87,    28,     0,    29,   320,    30,    89,     0,    90,
      91,    31,     0,     0,     0,     0,     0,    92,     0,    32,
       0,     0,     0,     0,    93,     0,    94,     0,     0,    95,
      96,    97,   845,     0,     0,     0,   566,     0,     0,     0,
       0,     0,     0,    99,   100,     0,   846,   102,   321,   569,
      34,     0,     0,     0,     0,     0,    35,    36,    37,   107,
     847,   109,   848,   111,   570,    38,   849,   572,   850,     0,
     116,   851,   852,    79,    80,    81,    43,     0,     0,     0,
       0,     0,     0,     7,     8,     9,    82,    10,    11,     0,
      83,  -298,     0,     0,     0,    13,    14,  -302,    15,    16,
       0,     0,    18,    19,    20,    21,    22,    23,    24,     0,
      25,    85,    26,     0,    86,     0,    87,    28,     0,    29,
      88,    30,    89,     0,    90,    91,    31,     0,     0,     0,
       0,     0,    92,     0,    32,     0,     0,     0,     0,    93,
       0,    94,     0,     0,    95,    96,    97,   756,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    99,   100,
       0,   757,   102,   103,   569,    34,     0,     0,     0,     0,
       0,    35,   106,    37,   107,   108,   109,   110,   111,   570,
     112,   758,   572,   759,     0,   116,   760,   761,    79,    80,
      81,    43,     0,     0,     0,     0,     0,     0,     7,     8,
       9,    82,    10,    11,     0,    83,     0,     0,     0,     0,
      13,    14,     0,    15,    16,   -68,     0,    18,    19,    20,
      21,    22,    23,    24,     0,    25,   319,    26,     0,    27,
       0,    87,    28,     0,    29,   320,    30,    89,     0,    90,
      91,    31,     0,     0,     0,     0,     0,    92,     0,    32,
       0,     0,     0,     0,    93,     0,    94,     0,     0,    95,
      96,    97,   845,     0,     0,     0,   566,     0,     0,     0,
       0,     0,     0,    99,   100,     0,   846,   102,   321,   569,
      34,     0,     0,     0,     0,     0,    35,    36,    37,   107,
     847,   109,   848,   111,   570,    38,   849,   572,   850,     0,
     116,   851,   852,    79,    80,    81,    43,     0,     0,     0,
       0,     0,     0,     7,     8,     9,    82,    10,    11,     0,
      83,  -298,     0,     0,     0,    13,    14,  -302,    15,    16,
       0,     0,    18,    19,    20,    21,    22,    23,    24,     0,
      25,   319,    26,     0,    27,     0,    87,    28,     0,    29,
     320,    30,    89,     0,    90,    91,    31,     0,     0,     0,
       0,     0,    92,     0,    32,     0,     0,     0,     0,    93,
       0,    94,     0,     0,    95,    96,    97,   756,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    99,   100,
       0,   757,   102,   321,   569,    34,     0,     0,     0,     0,
       0,    35,    36,    37,   107,   108,   109,   110,   111,   570,
      38,   758,   572,   759,     0,   116,   760,   761,    79,    80,
      81,    43,     0,     0,     0,     0,     0,     0,     7,     8,
       9,    82,    10,    11,     0,    83,     0,     0,     0,     0,
      13,    14,  -300,    15,    16,     0,     0,    18,    19,    20,
      21,    22,    23,    24,     0,    25,    85,    26,     0,    86,
       0,    87,    28,     0,    29,    88,    30,    89,     0,    90,
      91,    31,     0,     0,     0,     0,     0,    92,     0,    32,
       0,     0,     0,     0,    93,     0,    94,     0,     0,    95,
      96,    97,   567,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,    99,   100,     0,   568,   102,   103,   569,
      34,     0,     0,     0,     0,     0,    35,   106,    37,   107,
     108,   109,   110,   111,   570,   112,   571,   572,   573,     0,
     116,   574,   575,    79,    80,    81,    43,     0,     0,     0,
       0,     0,     0,     7,     8,     9,    82,    10,    11,     0,
      83,  -298,     0,     0,     0,    13,    14,     0,    15,    16,
       0,     0,    18,    19,    20,    21,    22,    23,    24,     0,
      25,   319,    26,     0,    27,     0,    87,    28,     0,    29,
     320,    30,    89,     0,    90,    91,    31,     0,     0,     0,
       0,     0,    92,     0,    32,     0,     0,     0,     0,    93,
       0,    94,     0,     0,    95,    96,    97,   756,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    99,   100,
       0,   757,   102,   321,   569,    34,     0,     0,     0,     0,
       0,    35,    36,    37,   107,   108,   109,   110,   111,   570,
      38,   758,   572,   759,     0,   116,   760,   761,    79,    80,
      81,    43,     0,     0,     0,     0,     0,     0,     7,     8,
       9,    82,    10,    11,     0,    83,     0,     0,     0,     0,
      13,    14,  -304,    15,    16,     0,     0,    18,    19,    20,
      21,    22,    23,    24,     0,    25,   319,    26,     0,    27,
       0,    87,    28,     0,    29,   320,    30,    89,     0,    90,
      91,    31,     0,     0,     0,     0,     0,    92,     0,    32,
       0,     0,     0,     0,    93,     0,    94,     0,     0,    95,
      96,    97,   567,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,    99,   100,     0,   568,   102,   321,   569,
      34,     0,     0,     0,     0,     0,    35,    36,    37,   107,
     108,   109,   110,   111,   570,    38,   571,   572,   573,     0,
     116,   574,   575,    79,    80,    81,    43,     0,     0,     0,
       0,     0,     0,     7,     8,     9,    82,    10,    11,     0,
      83,     0,     0,     0,     0,    13,    14,  -300,    15,    16,
       0,     0,    18,    19,    20,    21,    22,    23,    24,     0,
      25,   319,    26,     0,    27,     0,    87,    28,     0,    29,
     320,    30,    89,     0,    90,    91,    31,     0,     0,     0,
       0,     0,    92,     0,    32,     0,     0,     0,     0,    93,
       0,    94,     0,     0,    95,    96,    97,   567,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    99,   100,
       0,   568,   102,   321,   569,    34,     0,     0,     0,     0,
       0,    35,    36,    37,   107,   108,   109,   110,   111,   570,
      38,   571,   572,   573,     0,   116,   574,   575,    79,    80,
      81,    43,     0,     0,     0,     0,     0,     0,     7,     8,
       9,    82,    10,    11,     0,    83,     0,     0,     0,     0,
      13,    14,  -302,    15,    16,     0,     0,    18,    19,    20,
      21,    22,    23,    24,     0,    25,   319,    26,     0,    27,
       0,    87,    28,     0,    29,   320,    30,    89,     0,    90,
      91,    31,     0,     0,     0,     0,     0,    92,     0,    32,
       0,     0,     0,     0,    93,     0,    94,     0,     0,    95,
      96,    97,   756,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,    99,   100,     0,   757,   102,   321,   569,
      34,     0,     0,     0,     0,     0,    35,    36,    37,   107,
     108,   109,   110,   111,   570,    38,   758,   572,   759,     0,
     116,   760,   761,    79,    80,    81,    43,     0,     0,     0,
       0,     0,     0,     7,     8,     9,    82,    10,    11,     0,
      83,     0,     0,     0,     0,    13,    14,     0,    15,    16,
       0,     0,    18,    19,    20,    21,    22,    23,    24,     0,
      25,   319,    26,     0,    27,     0,    87,    28,     0,    29,
     320,    30,    89,     0,    90,    91,    31,     0,     0,     0,
       0,     0,    92,     0,    32,     0,     0,     0,     0,    93,
       0,    94,     0,     0,    95,    96,    97,   567,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    99,   100,
       0,   568,   102,   321,   569,    34,     0,     0,     0,     0,
       0,    35,    36,    37,   107,   108,   109,   110,   111,   570,
      38,   571,   572,   573,     0,   116,   574,   575,    79,    80,
      81,    43,     0,     0,     0,     0,     0,     0,     7,     8,
       9,    82,    10,    11,     0,    83,     0,     0,     0,     0,
      13,    14,     0,    15,    16,     0,     0,    18,    19,    20,
      21,    22,    23,    24,     0,    25,    85,    26,     0,    86,
       0,    87,    28,     0,    29,    88,    30,    89,     0,    90,
      91,    31,     0,     0,     0,     0,     0,    92,     0,    32,
       0,     0,     0,     0,    93,     0,    94,     0,     0,    95,
      96,    97,   401,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,    99,   100,     0,     0,   102,   103,   104,
      34,     0,     0,     0,     0,     0,    35,   106,    37,   107,
     108,   109,   110,   111,     0,   112,   113,   114,   402,     0,
     116,   403,   404,    79,    80,    81,    43,     0,     0,     0,
       0,     0,     0,     7,     0,     0,    82,    10,    11,     0,
      12,     0,     0,     0,     0,    13,    14,     0,    15,    16,
       0,     0,     0,    19,    20,    21,    22,     0,     0,     0,
      25,   319,     0,     0,    27,     0,    87,     0,     0,    29,
     320,    30,    89,     0,    90,     0,    31,     0,     0,     0,
       0,     0,    92,     0,    32,     0,     0,     0,     0,    93,
       0,    94,     0,     0,    95,    96,    97,   273,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    99,   100,
       0,   101,   102,   321,     0,    34,   105,     0,     0,     0,
       0,    35,    36,    37,   107,     0,   109,     0,   111,     0,
      38,   113,   114,   322,     0,   116,   323,   324,   452,   453,
       0,    43,     0,     0,     0,     0,     0,     7,     8,     9,
      82,    10,    11,     0,    83,     0,     0,     0,     0,    13,
      14,  -105,    15,    16,     0,     0,    18,    19,    20,    21,
      22,    23,    24,     0,    25,    85,    26,     0,    86,     0,
      87,    28,     0,    29,    88,    30,    89,     0,    90,    91,
      31,     0,     0,     0,     0,     0,    92,     0,    32,     0,
       0,     0,     0,     0,     0,    94,     0,     0,     0,    96,
       0,   401,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,    99,   100,     0,     0,     0,   457,   569,    34,
       0,     0,     0,     0,     0,    35,   106,    37,   107,   108,
     109,   110,   111,   570,   112,   667,   572,   402,     0,   116,
     403,   668,   452,   453,     0,    43,     0,     0,     0,     0,
       0,     7,     8,     9,    82,    10,    11,     0,    83,     0,
       0,     0,     0,    13,    14,  -106,    15,    16,     0,     0,
      18,    19,    20,    21,    22,    23,    24,     0,    25,    85,
      26,     0,    86,     0,    87,    28,     0,    29,    88,    30,
      89,     0,    90,    91,    31,     0,     0,     0,     0,     0,
      92,     0,    32,     0,     0,     0,     0,     0,     0,    94,
     231,   232,     0,    96,     0,   401,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,    99,   100,     0,     0,
       0,   457,   569,    34,     0,     0,     0,     0,     0,    35,
     106,    37,   107,   108,   109,   110,   111,   570,   112,   667,
     572,   402,     0,   116,   403,   668,   452,   453,    34,    43,
     360,   234,     0,     0,    35,     7,     0,     0,    82,    10,
      11,     0,    12,    38,   113,   114,   235,    13,    14,   236,
      15,    16,     0,     0,   237,    19,    20,    21,    22,     0,
       0,     0,    25,    85,     0,     0,    86,     0,    87,     0,
       0,    29,    88,    30,    89,     0,    90,     0,    31,     0,
      79,   337,   338,   339,    92,     0,    32,     0,     0,     0,
       0,     0,     0,    94,     0,     0,     0,    96,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
      99,   100,     0,    22,     0,   457,     0,     0,   105,     0,
       0,     0,     0,     0,   106,    37,   107,     0,   109,     0,
     111,     0,   683,     0,     0,   322,     0,   116,   323,   459,
     340,     0,     0,    43,    79,   337,   338,   339,     0,     0,
       0,     0,     0,    97,     0,   231,   232,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,   102,
     103,     0,    34,     0,     0,     0,     0,    22,    35,   106,
       0,     0,     0,     0,     0,     0,     0,   112,   113,   114,
     341,     0,     0,   342,   343,   344,     0,   345,     0,     0,
      79,    80,    81,    34,   340,   233,   234,     0,     0,    35,
       0,     0,     0,     0,   231,   232,     0,    97,    38,   113,
     114,   235,     0,     0,   236,     0,     0,     0,     0,   237,
     636,     0,     0,   102,   321,     0,    34,     0,     0,     0,
       0,     0,    35,    36,     0,     0,     0,     0,     0,     0,
       0,    38,   113,   114,   341,     0,     0,   342,   343,   344,
     471,   345,    34,     0,   529,   234,    93,     0,    35,     0,
       0,    95,     0,    97,     0,     0,     0,    38,   113,   114,
     235,     0,   643,   236,   644,     0,   472,     0,   237,   102,
     321,   645,    34,     0,   646,   647,   648,     0,    35,    36,
     473,   107,     0,   109,     0,     0,     0,    38,   113,   114,
       0,   649,   116,   650,   324,   651,     0,     0,   652,   643,
       0,   644,     0,     0,     0,   653,   654,    91,   645,     0,
       0,   646,   647,   648,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   649,     0,
     650,     0,   651,     0,     0,   652,     0,     0,     0,     0,
       0,     0,   653,   654,     0,     0,     0,    34,     0,     0,
       0,     0,     0,    35,     0,     0,     0,     0,     0,     0,
       0,     0,    38,   113,   114,   176,     0,     0,   177,     0,
       0,     0,     0,     0,   655,     0,     0,     0,     0,     0,
       0,     0,     0,     0,    34,     0,     0,     0,     0,     0,
      35,     0,     0,     0,     0,     0,     0,     0,     0,    38,
     113,   114,   176,     0,     0,   177,     0,     0,     0,     0,
       0,   822,     7,     8,     9,    82,    10,    11,     0,    83,
       0,     0,     0,     0,    13,    14,     0,    15,    16,     0,
       0,    18,    19,    20,    21,    22,    23,    24,     0,    25,
     319,    26,     0,    27,     0,    87,    28,     0,    29,   320,
      30,    89,     0,    90,    91,    31,     0,     0,     0,     0,
       0,    92,     0,    32,     0,     0,     0,     0,     0,     0,
      94,     0,     0,     0,    96,     0,   401,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    99,   100,     0,
       0,     0,     0,   569,    34,     0,     0,     0,     0,     0,
      35,    36,    37,   107,   108,   109,   110,   111,   570,    38,
     667,   572,   402,     0,   116,   403,   834,     0,     7,     0,
      43,    82,    10,    11,     0,    12,     0,     0,     0,     0,
      13,    14,     0,    15,    16,  -159,     0,    18,    19,    20,
      21,    22,     0,    24,     0,    25,   319,    26,     0,    27,
       0,    87,     0,     0,    29,   320,    30,    89,     0,    90,
       0,    31,     0,     0,     0,     0,     0,    92,     0,    32,
       0,     0,     0,     0,     0,     0,    94,     0,     0,     0,
      96,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,    99,   100,     0,     0,     0,     0,     0,
      34,   105,     0,     0,     0,     0,    35,    36,    37,   107,
       0,   109,     0,   111,     0,    38,   113,   114,   322,     0,
     116,   323,     0,     0,     7,     0,    43,    82,    10,    11,
       0,    12,     0,     0,     0,     0,    13,    14,     0,    15,
      16,     0,     0,     0,    19,    20,    21,    22,     0,     0,
       0,    25,   319,     0,     0,    27,     0,    87,     0,     0,
      29,   320,    30,    89,     0,    90,     0,    31,     0,     0,
       0,     0,     0,    92,     0,    32,     0,     0,     0,     0,
       0,     0,    94,     0,     0,     0,    96,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,    99,
     100,     0,     0,     0,     0,     0,     0,   105,     0,     0,
       0,     0,     0,    36,    37,   107,     0,   109,     7,   111,
       0,    82,    10,    11,   322,    83,   116,   323,     0,     0,
      13,    14,    43,    15,    16,     0,     0,     0,    19,    20,
      21,     0,     0,     0,     0,    25,   319,     0,     0,    27,
       0,    87,     0,     0,    29,   320,    30,    89,     0,    90,
       0,    31,     0,     0,     0,     0,     0,    92,     0,    32,
       0,     0,     0,     0,     0,     0,    94,     0,     0,     0,
      96,     0,     0,     0,     7,     0,     0,     0,    10,    11,
       0,    12,     0,    99,   100,     0,    13,    14,     0,    15,
      16,     0,     0,     0,    19,    20,    21,    36,    37,     0,
       0,    25,     0,   111,     0,    27,     0,     0,     0,     0,
      29,     0,    30,    89,     0,     0,    43,    31,     0,     0,
       0,     0,     0,    92,     0,    32,     0,     0,     0,     0,
       0,     0,    94,     0,     0,     0,    96,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,    99,
     100,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,    36,    37,   107,     0,   109,     0,   111,
       0,     0,     0,     0,     0,     0,   116,     0,     0,     0,
       0,     0,    43
};

static const yytype_int16 yycheck[] =
{
       3,     4,     3,     4,     4,    84,     3,     4,     4,     3,
     247,     3,     3,     4,    58,     4,     3,     3,     4,     4,
       3,     4,     3,    75,   156,     4,   126,     4,    58,   268,
       4,     4,   565,   109,   565,   163,     4,   101,   166,   210,
     116,   185,   247,    21,   247,   396,   496,   158,    38,    90,
      43,   233,    75,    31,     4,    72,   467,   451,    89,   422,
      85,    87,     0,    88,    58,    90,    58,   158,    75,   396,
     266,   250,   138,   467,   404,     0,     0,    58,   226,   250,
       0,    84,   633,    84,    84,     1,    89,    84,    84,   394,
       0,   210,     0,    84,     0,    84,    62,   103,    84,    84,
     106,    84,   123,     1,    82,    84,   112,    84,   319,   755,
      84,    84,    79,   103,   185,    27,    84,   169,     1,    38,
     451,   380,   112,   382,   332,   362,    93,   125,   194,     9,
      10,   250,   130,     7,    84,    79,   467,   295,   138,   376,
     206,   299,   138,   158,    10,    10,   169,   650,   111,   652,
     118,    10,   310,   116,   120,   158,   126,   362,   128,   362,
     288,   138,   169,   107,   292,   109,    10,     0,    10,    10,
     195,   196,    10,   376,    86,   708,    95,   708,   360,     5,
       6,     7,    87,   734,   830,   104,   124,   395,   396,   197,
      64,   123,   555,   112,   194,   500,   123,   502,   194,   124,
     124,   506,   330,   462,   124,   333,   206,   210,   124,   210,
     206,   218,   229,   210,   124,   566,   124,   194,   124,   210,
     368,   210,   101,   102,   210,   210,   124,   210,   123,   206,
     358,   210,   217,   210,   122,   383,   210,   210,   329,   566,
     217,   124,   107,   454,   270,    71,   277,   250,    32,   250,
      76,   230,    78,   250,    82,    83,   122,   122,   708,   250,
     210,   250,   122,   122,   250,   250,   629,   250,    94,    95,
     285,   250,   123,   250,   668,   423,   250,   250,   122,   320,
     122,   122,   410,   286,   122,   285,    13,   295,    51,   619,
      53,   299,   436,   119,   319,   320,   626,   627,   125,    79,
     250,   353,   310,    66,   122,   376,    79,   125,   719,    99,
     100,   844,    81,   844,   329,    81,    85,    86,   268,    85,
     364,   111,   366,    79,   718,   719,   329,   107,    79,   109,
     353,   321,   451,   326,   107,   686,   109,   107,   496,   119,
     524,   525,   526,    12,   472,   529,   353,   529,   467,   107,
     461,    29,    21,   886,   371,   886,   630,   631,    27,   149,
     123,   364,    31,   366,    81,    79,   156,   364,    85,   366,
     364,    82,   366,   364,   164,   366,     4,   364,   364,   366,
     366,   364,    79,   364,   436,   366,    81,   718,   719,   392,
      85,   382,   803,   107,    38,   109,   119,   120,   392,   395,
     396,   638,    79,   380,    88,   382,   395,   396,    88,   803,
     107,   392,   109,    82,    83,   456,    85,    86,    87,    88,
      89,    90,   448,   449,   392,   621,   622,   395,   396,   454,
     107,   456,   109,   638,   534,   638,   886,   616,   677,    38,
     834,   115,   125,   509,   118,   616,   546,   105,   451,   125,
     755,   457,   458,   125,    98,   367,    84,   451,   869,   125,
     372,   561,   451,    91,   467,    79,    29,    30,   447,   447,
     451,   115,   803,   467,   118,   869,    80,    81,   467,   125,
      79,    81,    96,   123,   487,   462,   467,    81,   496,    13,
     634,    85,    86,   107,    93,    83,   499,   616,   501,   113,
     114,   451,   505,   624,   625,   499,   811,   501,   813,   509,
     138,   505,   462,   509,   113,   114,   115,   467,   499,   118,
     501,   679,    83,   681,   505,   830,   195,   196,    57,   125,
     158,   499,   509,   501,   446,   599,   125,   505,   869,   125,
     209,    81,   125,   455,   451,    85,    86,   731,   123,   499,
     122,   501,   710,    81,    81,   505,   793,   565,    85,    86,
     467,   189,    81,   634,   635,    10,   194,     9,    81,    81,
     566,   615,   877,    85,    86,    10,     8,   566,   206,   126,
     817,    29,   210,   635,    80,   615,    81,   102,   793,    81,
      85,    86,   736,   725,   597,   900,    82,   266,   566,   718,
     719,   270,    81,   597,   125,   123,    85,    86,   210,   805,
     806,    82,   123,   616,    82,   616,   597,   286,   125,   616,
      81,   615,   250,   615,   650,   616,   652,   616,    79,   597,
     616,   616,   684,   616,   615,   125,   125,   616,    15,   616,
     268,    81,   616,   616,    79,    85,    86,   723,   250,    23,
     319,   320,   117,    97,    80,    81,   814,   285,    84,   103,
      86,   105,   113,   114,   115,   736,   616,   118,   112,   113,
     114,   679,   659,   681,    81,   125,   654,   683,   113,   114,
     115,    81,   107,   118,   803,    80,    81,   122,    81,    84,
     597,    86,    85,    86,   878,   879,   880,   125,   367,    81,
     708,   329,   710,   372,   707,   113,   114,   115,   125,   712,
     118,    99,   715,   707,   717,   718,   719,   695,   712,    38,
     119,   715,   793,   717,   718,   719,   707,   677,    81,   718,
     719,   712,    29,   268,   715,   863,   717,   718,   719,   707,
      81,    23,   745,   101,   712,   770,   771,   715,    81,   717,
     869,   745,    80,    81,   382,    74,    84,   885,    86,   125,
      79,    99,   712,    81,   745,   715,   842,   717,    68,    69,
      70,   122,   843,    81,    79,    75,    47,   446,   447,   448,
     449,   909,   124,   118,   695,   454,   455,   456,   366,   868,
     392,   843,   818,   222,   113,   114,   115,    80,    81,   118,
     803,    84,   285,    86,    64,   712,   814,   556,   715,   803,
     717,   718,   719,    64,   803,   428,   429,   430,   487,   447,
     433,   285,   803,   193,   827,   695,   829,   127,   128,   113,
     114,   115,    64,   827,   118,   829,   844,   825,   745,   123,
     769,   469,   487,    59,    60,    61,   827,    63,   829,   113,
     114,   115,   521,   722,   118,   881,   882,   807,   808,   809,
     810,   829,   812,    79,    38,   868,   869,   868,   868,   689,
     890,   868,   868,    89,   821,   869,   481,   868,   886,   868,
     869,   509,   868,   868,   887,   868,    80,    81,   869,   868,
      84,   868,   697,   887,   868,   868,   803,   499,   436,   501,
     868,   169,    -1,   505,    -1,    -1,   887,    -1,   911,   392,
      81,   524,   525,   526,    85,    86,   451,   911,   868,   887,
     827,     3,    -1,    97,   874,   875,   876,   462,   392,   103,
     911,    81,   467,    -1,    -1,    85,    86,    -1,   112,   113,
     114,   115,    -1,   911,   118,    -1,    -1,     6,     7,     0,
     124,   901,   621,   622,     5,     6,     7,    -1,   113,   114,
     115,    -1,   869,   118,   499,    -1,   501,    -1,    79,    -1,
     505,    -1,    -1,     6,     7,    -1,    58,    -1,    -1,    -1,
     887,   650,    -1,   652,    43,   654,    97,    46,   616,    -1,
      -1,    -1,   103,    52,    -1,   597,   107,    -1,    -1,    -1,
      -1,   112,   113,   114,   911,   633,    -1,    89,    -1,    -1,
      43,    -1,    -1,    46,   616,    -1,   499,    -1,   501,    52,
      71,    -1,   505,    -1,    79,    76,   695,    78,    -1,    -1,
     513,    -1,    -1,   516,   662,   499,    95,   501,    -1,    -1,
      -1,   505,    97,    94,    95,   104,    79,    -1,   103,   677,
      -1,    -1,   107,   112,    38,    -1,   138,   112,   113,   114,
     119,   689,    95,    -1,    97,    -1,    -1,   695,   119,    -1,
     103,   104,    -1,   124,   107,    -1,   158,    -1,    -1,   112,
     113,   114,    -1,    59,    60,    61,   119,    63,    72,    -1,
      74,    -1,    59,    60,    61,    79,    63,    -1,    -1,    -1,
      -1,   770,   771,    -1,    -1,   707,   734,    49,    50,    -1,
     712,    -1,   194,   715,   597,   717,    -1,    -1,   731,    -1,
      -1,    -1,    -1,    -1,   206,    -1,    -1,    -1,    -1,   113,
     114,   115,    -1,   597,   118,    -1,   805,   806,    -1,    -1,
      -1,    -1,   677,   745,    -1,   129,    -1,    15,    -1,   818,
      -1,    19,    20,    50,    22,    97,    -1,    99,   100,    27,
      28,   103,    30,    31,    -1,    -1,    -1,    35,    36,    37,
     112,   113,   114,   115,    42,    -1,   118,   712,    46,    -1,
     715,   123,   717,    51,    -1,    53,    -1,    -1,    -1,    -1,
      58,    -1,    -1,   821,    -1,    -1,    -1,    -1,    66,    -1,
      97,    -1,    99,   100,   286,    -1,   103,    -1,    -1,    -1,
      -1,    -1,   881,   882,    -1,   112,   113,   114,    -1,    -1,
      -1,    -1,    -1,    -1,   707,   827,   123,   829,    -1,   712,
      -1,    79,   715,    -1,   717,    -1,   104,   105,    -1,    -1,
     868,    -1,    -1,   707,    92,    93,    -1,   329,   712,    -1,
     332,   715,    -1,   717,    -1,   123,    -1,    -1,   106,   107,
     108,   109,   745,    -1,    -1,   878,   879,    49,   116,   117,
      -1,    -1,   807,   808,   809,   810,    -1,   812,    19,    20,
      -1,   745,   364,    -1,   366,   887,    27,    28,    -1,    30,
      31,    -1,    -1,    -1,    35,    36,    -1,    -1,   380,    -1,
     382,    42,    -1,    -1,    -1,    -1,    -1,    15,    -1,   911,
      -1,    19,    20,   395,   396,    97,    -1,    99,    -1,    27,
      28,   103,    30,    31,    -1,    -1,    -1,    35,    36,    37,
     112,   113,   114,   115,    42,    -1,   118,    -1,    46,   874,
     875,   876,    -1,    51,   827,    53,   829,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    97,    -1,    66,    -1,
      -1,    -1,   103,   827,   105,   829,   901,    -1,    -1,    97,
      -1,   112,   113,   114,   115,   103,    -1,    -1,    -1,   461,
     462,    -1,   123,    -1,   112,   113,   114,   115,    -1,    -1,
     118,    -1,    -1,    -1,    -1,    -1,   104,   105,    -1,    -1,
      -1,    -1,     0,     1,   887,   487,    -1,     5,     6,     7,
      -1,    -1,    -1,    -1,    -1,   123,    -1,    15,    16,    17,
      18,    19,    20,   887,    22,    -1,    -1,   509,   911,    27,
      28,    -1,    30,    31,    32,    33,    34,    35,    36,    37,
      38,    39,    40,    -1,    42,    43,    44,   911,    46,    -1,
      48,    49,    -1,    51,    52,    53,    54,    -1,    56,    57,
      58,    -1,    -1,    -1,    -1,    -1,    64,    -1,    66,    -1,
      -1,    -1,    -1,    71,    -1,    73,    -1,    -1,    76,    77,
      78,    79,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    90,    91,    -1,    93,    94,    95,    96,    97,
      98,    -1,    -1,    -1,    -1,   103,   104,   105,   106,   107,
     108,   109,   110,    -1,   112,   113,   114,   115,    -1,   117,
     118,   119,     0,     1,    -1,   123,   124,     5,     6,     7,
      -1,    -1,    -1,   615,    -1,    -1,    -1,    15,    -1,    -1,
      18,    19,    20,    -1,    22,    -1,    -1,    -1,    -1,    27,
      28,    -1,    30,    31,    -1,    -1,    -1,    35,    36,    37,
      38,    -1,    -1,    -1,    42,    43,    -1,    -1,    46,    -1,
      48,    -1,    -1,    51,    52,    53,    54,    -1,    56,    -1,
      58,    -1,    -1,    -1,    -1,    -1,    64,    -1,    66,    -1,
      -1,    -1,    -1,    71,    -1,    73,    -1,    -1,    76,    77,
      78,    79,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    90,    91,    -1,    93,    94,    95,    -1,    97,
      98,    -1,    -1,    -1,    -1,   103,   104,   105,   106,    -1,
     108,    -1,   110,    -1,   112,   113,   114,   115,    -1,   117,
     118,   119,     0,     1,    -1,   123,   124,     5,     6,     7,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    15,    16,    17,
      18,    19,    20,    -1,    22,    -1,    -1,    -1,    -1,    27,
      28,    -1,    30,    31,    32,    33,    34,    35,    36,    37,
      38,    39,    40,    -1,    42,    43,    44,    -1,    46,    -1,
      48,    49,    -1,    51,    52,    53,    54,    -1,    56,    57,
      58,    -1,    -1,    -1,    -1,    -1,    64,    -1,    66,    -1,
      -1,    -1,    -1,    71,    -1,    73,    -1,    -1,    76,    77,
      78,    79,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    90,    91,    -1,    93,    94,    95,    96,    97,
      98,    -1,    -1,    -1,    -1,   103,   104,   105,   106,   107,
     108,   109,   110,    -1,   112,   113,   114,   115,     1,   117,
     118,   119,     5,     6,     7,   123,    -1,    -1,    -1,    -1,
      -1,    -1,    15,    16,    17,    18,    19,    20,    -1,    22,
      -1,    -1,    -1,    -1,    27,    28,    -1,    30,    31,    32,
      33,    34,    35,    36,    37,    38,    39,    40,    -1,    42,
      43,    44,    -1,    46,    -1,    48,    49,    -1,    51,    52,
      53,    54,    -1,    56,    57,    58,    -1,    -1,    -1,    -1,
      -1,    64,    -1,    66,    -1,    -1,    -1,    -1,    71,    -1,
      73,    -1,    -1,    76,    77,    78,    79,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    90,    91,    -1,
      93,    94,    95,    96,    97,    98,    -1,    -1,    -1,    -1,
     103,   104,   105,   106,   107,   108,   109,   110,    -1,   112,
     113,   114,   115,     1,   117,   118,   119,     5,     6,     7,
     123,   124,    -1,    -1,    -1,    -1,    -1,    15,    -1,    -1,
      18,    19,    20,    -1,    22,    -1,    -1,    -1,    -1,    27,
      28,    -1,    30,    31,    -1,    -1,    -1,    35,    36,    37,
      38,    -1,    -1,    -1,    42,    43,    -1,    -1,    46,    -1,
      48,    -1,    -1,    51,    52,    53,    54,    -1,    56,    -1,
      58,    -1,    -1,    -1,    -1,    -1,    64,    -1,    66,    -1,
      -1,    -1,    -1,    71,    -1,    73,    -1,    -1,    76,    77,
      78,    79,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    90,    91,    -1,    93,    94,    95,    -1,    97,
      98,    -1,    -1,    -1,    -1,   103,   104,   105,   106,    -1,
     108,    -1,   110,     1,   112,   113,   114,   115,    -1,   117,
     118,   119,    -1,    -1,    -1,   123,   124,    15,    38,    -1,
      18,    19,    20,    -1,    22,    -1,    -1,    -1,    -1,    27,
      28,    -1,    30,    31,    -1,    -1,    -1,    35,    36,    37,
      38,    -1,    -1,    -1,    42,    43,    -1,    -1,    46,    -1,
      48,    -1,    -1,    51,    52,    53,    54,    -1,    56,    79,
      58,    -1,    -1,    -1,    -1,    -1,    64,    -1,    66,    -1,
      -1,    -1,    92,    93,    -1,    73,    -1,    -1,    98,    77,
      -1,    -1,    -1,    -1,    -1,    -1,   106,   107,   108,   109,
      -1,   111,    90,    91,    -1,   115,   116,   117,   118,    97,
      -1,    -1,    -1,    -1,    -1,   103,   104,   105,    -1,    -1,
      -1,    -1,   110,    -1,   112,   113,   114,   115,     0,     1,
     118,    -1,    -1,    -1,    -1,   123,   124,    -1,    -1,    -1,
      -1,    -1,    -1,    15,    16,    17,    -1,    19,    20,    -1,
      22,    -1,    -1,    -1,    -1,    27,    28,    -1,    30,    31,
      32,    33,    34,    35,    36,    37,    38,    39,    40,     1,
      42,    -1,    44,    -1,    46,    -1,    -1,    49,    -1,    51,
      -1,    53,    -1,    15,    -1,    17,    58,    19,    20,    -1,
      22,    -1,    -1,    -1,    66,    27,    28,    -1,    30,    31,
      32,    -1,    34,    35,    36,    37,    38,    39,    40,    -1,
      42,    -1,    44,    -1,    46,    -1,    88,    49,    -1,    51,
      -1,    53,    -1,    -1,    -1,    97,    58,    -1,    -1,    -1,
      -1,   103,   104,   105,    66,    -1,    -1,    -1,    -1,    -1,
     112,   113,   114,   115,    -1,    -1,   118,    -1,    -1,    -1,
      -1,   123,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    97,    -1,    -1,    38,     1,
      -1,   103,   104,   105,    -1,    -1,    -1,    -1,    -1,    -1,
     112,   113,   114,    15,    -1,    -1,    -1,    19,    20,    -1,
      22,   123,   124,    -1,    -1,    27,    28,    -1,    30,    31,
      32,    -1,    34,    35,    36,    37,    38,    39,    40,    79,
      42,    -1,    44,    -1,    46,    -1,    -1,    49,    -1,    51,
      -1,    53,    92,    93,    -1,    -1,    58,    -1,    98,    -1,
      -1,    -1,    -1,    -1,    66,    -1,   106,   107,   108,   109,
      -1,   111,    -1,    -1,    -1,   115,   116,   117,   118,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    97,    -1,    -1,    -1,    -1,
      -1,   103,   104,   105,    -1,    -1,    -1,    -1,    -1,     1,
     112,   113,   114,     5,     6,     7,    -1,    -1,    -1,    -1,
      -1,   123,   124,    15,    -1,    -1,    18,    19,    20,    -1,
      22,    -1,    -1,    -1,    -1,    27,    28,    -1,    30,    31,
      -1,    -1,    -1,    35,    36,    37,    38,    -1,    -1,    -1,
      42,    43,    -1,    -1,    46,    -1,    48,    -1,    -1,    51,
      52,    53,    54,    -1,    56,    -1,    58,    -1,    -1,    -1,
      -1,    -1,    64,    -1,    66,    67,    -1,    -1,    -1,    71,
      -1,    73,    -1,    -1,    76,    77,    78,    79,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    90,    91,
      -1,    93,    94,    95,    -1,    97,    98,    -1,    -1,    -1,
      -1,   103,   104,   105,   106,    -1,   108,    -1,   110,    -1,
     112,   113,   114,   115,    -1,   117,   118,   119,     5,     6,
       7,   123,    -1,    -1,    -1,    -1,    -1,    -1,    15,    16,
      17,    18,    19,    20,    -1,    22,    -1,    -1,    -1,    -1,
      27,    28,    29,    30,    31,    32,    -1,    34,    35,    36,
      37,    38,    39,    40,    -1,    42,    43,    44,    -1,    46,
      -1,    48,    49,    -1,    51,    52,    53,    54,    -1,    56,
      57,    58,    -1,    -1,    -1,    -1,    -1,    64,    -1,    66,
      -1,    -1,    -1,    -1,    71,    -1,    73,    -1,    -1,    76,
      77,    78,    79,    -1,    -1,    -1,    83,    -1,    -1,    -1,
      -1,    -1,    -1,    90,    91,    -1,    93,    94,    95,    96,
      97,    -1,    -1,    -1,    -1,    -1,   103,   104,   105,   106,
     107,   108,   109,   110,   111,   112,   113,   114,   115,    -1,
     117,   118,   119,     5,     6,     7,   123,    -1,    -1,    -1,
      -1,    -1,    -1,    15,    16,    17,    18,    19,    20,    -1,
      22,    23,    -1,    -1,    -1,    27,    28,    29,    30,    31,
      -1,    -1,    34,    35,    36,    37,    38,    39,    40,    -1,
      42,    43,    44,    -1,    46,    -1,    48,    49,    -1,    51,
      52,    53,    54,    -1,    56,    57,    58,    -1,    -1,    -1,
      -1,    -1,    64,    -1,    66,    -1,    -1,    -1,    -1,    71,
      -1,    73,    -1,    -1,    76,    77,    78,    79,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    90,    91,
      -1,    93,    94,    95,    96,    97,    -1,    -1,    -1,    -1,
      -1,   103,   104,   105,   106,   107,   108,   109,   110,   111,
     112,   113,   114,   115,    -1,   117,   118,   119,     5,     6,
       7,   123,    -1,    -1,    -1,    -1,    -1,    -1,    15,    16,
      17,    18,    19,    20,    -1,    22,    -1,    -1,    -1,    -1,
      27,    28,    -1,    30,    31,    32,    -1,    34,    35,    36,
      37,    38,    39,    40,    -1,    42,    43,    44,    -1,    46,
      -1,    48,    49,    -1,    51,    52,    53,    54,    -1,    56,
      57,    58,    -1,    -1,    -1,    -1,    -1,    64,    -1,    66,
      -1,    -1,    -1,    -1,    71,    -1,    73,    -1,    -1,    76,
      77,    78,    79,    -1,    -1,    -1,    83,    -1,    -1,    -1,
      -1,    -1,    -1,    90,    91,    -1,    93,    94,    95,    96,
      97,    -1,    -1,    -1,    -1,    -1,   103,   104,   105,   106,
     107,   108,   109,   110,   111,   112,   113,   114,   115,    -1,
     117,   118,   119,     5,     6,     7,   123,    -1,    -1,    -1,
      -1,    -1,    -1,    15,    16,    17,    18,    19,    20,    -1,
      22,    23,    -1,    -1,    -1,    27,    28,    29,    30,    31,
      -1,    -1,    34,    35,    36,    37,    38,    39,    40,    -1,
      42,    43,    44,    -1,    46,    -1,    48,    49,    -1,    51,
      52,    53,    54,    -1,    56,    57,    58,    -1,    -1,    -1,
      -1,    -1,    64,    -1,    66,    -1,    -1,    -1,    -1,    71,
      -1,    73,    -1,    -1,    76,    77,    78,    79,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    90,    91,
      -1,    93,    94,    95,    96,    97,    -1,    -1,    -1,    -1,
      -1,   103,   104,   105,   106,   107,   108,   109,   110,   111,
     112,   113,   114,   115,    -1,   117,   118,   119,     5,     6,
       7,   123,    -1,    -1,    -1,    -1,    -1,    -1,    15,    16,
      17,    18,    19,    20,    -1,    22,    -1,    -1,    -1,    -1,
      27,    28,    29,    30,    31,    -1,    -1,    34,    35,    36,
      37,    38,    39,    40,    -1,    42,    43,    44,    -1,    46,
      -1,    48,    49,    -1,    51,    52,    53,    54,    -1,    56,
      57,    58,    -1,    -1,    -1,    -1,    -1,    64,    -1,    66,
      -1,    -1,    -1,    -1,    71,    -1,    73,    -1,    -1,    76,
      77,    78,    79,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    90,    91,    -1,    93,    94,    95,    96,
      97,    -1,    -1,    -1,    -1,    -1,   103,   104,   105,   106,
     107,   108,   109,   110,   111,   112,   113,   114,   115,    -1,
     117,   118,   119,     5,     6,     7,   123,    -1,    -1,    -1,
      -1,    -1,    -1,    15,    16,    17,    18,    19,    20,    -1,
      22,    23,    -1,    -1,    -1,    27,    28,    -1,    30,    31,
      -1,    -1,    34,    35,    36,    37,    38,    39,    40,    -1,
      42,    43,    44,    -1,    46,    -1,    48,    49,    -1,    51,
      52,    53,    54,    -1,    56,    57,    58,    -1,    -1,    -1,
      -1,    -1,    64,    -1,    66,    -1,    -1,    -1,    -1,    71,
      -1,    73,    -1,    -1,    76,    77,    78,    79,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    90,    91,
      -1,    93,    94,    95,    96,    97,    -1,    -1,    -1,    -1,
      -1,   103,   104,   105,   106,   107,   108,   109,   110,   111,
     112,   113,   114,   115,    -1,   117,   118,   119,     5,     6,
       7,   123,    -1,    -1,    -1,    -1,    -1,    -1,    15,    16,
      17,    18,    19,    20,    -1,    22,    -1,    -1,    -1,    -1,
      27,    28,    29,    30,    31,    -1,    -1,    34,    35,    36,
      37,    38,    39,    40,    -1,    42,    43,    44,    -1,    46,
      -1,    48,    49,    -1,    51,    52,    53,    54,    -1,    56,
      57,    58,    -1,    -1,    -1,    -1,    -1,    64,    -1,    66,
      -1,    -1,    -1,    -1,    71,    -1,    73,    -1,    -1,    76,
      77,    78,    79,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    90,    91,    -1,    93,    94,    95,    96,
      97,    -1,    -1,    -1,    -1,    -1,   103,   104,   105,   106,
     107,   108,   109,   110,   111,   112,   113,   114,   115,    -1,
     117,   118,   119,     5,     6,     7,   123,    -1,    -1,    -1,
      -1,    -1,    -1,    15,    16,    17,    18,    19,    20,    -1,
      22,    -1,    -1,    -1,    -1,    27,    28,    29,    30,    31,
      -1,    -1,    34,    35,    36,    37,    38,    39,    40,    -1,
      42,    43,    44,    -1,    46,    -1,    48,    49,    -1,    51,
      52,    53,    54,    -1,    56,    57,    58,    -1,    -1,    -1,
      -1,    -1,    64,    -1,    66,    -1,    -1,    -1,    -1,    71,
      -1,    73,    -1,    -1,    76,    77,    78,    79,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    90,    91,
      -1,    93,    94,    95,    96,    97,    -1,    -1,    -1,    -1,
      -1,   103,   104,   105,   106,   107,   108,   109,   110,   111,
     112,   113,   114,   115,    -1,   117,   118,   119,     5,     6,
       7,   123,    -1,    -1,    -1,    -1,    -1,    -1,    15,    16,
      17,    18,    19,    20,    -1,    22,    -1,    -1,    -1,    -1,
      27,    28,    29,    30,    31,    -1,    -1,    34,    35,    36,
      37,    38,    39,    40,    -1,    42,    43,    44,    -1,    46,
      -1,    48,    49,    -1,    51,    52,    53,    54,    -1,    56,
      57,    58,    -1,    -1,    -1,    -1,    -1,    64,    -1,    66,
      -1,    -1,    -1,    -1,    71,    -1,    73,    -1,    -1,    76,
      77,    78,    79,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    90,    91,    -1,    93,    94,    95,    96,
      97,    -1,    -1,    -1,    -1,    -1,   103,   104,   105,   106,
     107,   108,   109,   110,   111,   112,   113,   114,   115,    -1,
     117,   118,   119,     5,     6,     7,   123,    -1,    -1,    -1,
      -1,    -1,    -1,    15,    16,    17,    18,    19,    20,    -1,
      22,    -1,    -1,    -1,    -1,    27,    28,    -1,    30,    31,
      -1,    -1,    34,    35,    36,    37,    38,    39,    40,    -1,
      42,    43,    44,    -1,    46,    -1,    48,    49,    -1,    51,
      52,    53,    54,    -1,    56,    57,    58,    -1,    -1,    -1,
      -1,    -1,    64,    -1,    66,    -1,    -1,    -1,    -1,    71,
      -1,    73,    -1,    -1,    76,    77,    78,    79,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    90,    91,
      -1,    93,    94,    95,    96,    97,    -1,    -1,    -1,    -1,
      -1,   103,   104,   105,   106,   107,   108,   109,   110,   111,
     112,   113,   114,   115,    -1,   117,   118,   119,     5,     6,
       7,   123,    -1,    -1,    -1,    -1,    -1,    -1,    15,    16,
      17,    18,    19,    20,    -1,    22,    -1,    -1,    -1,    -1,
      27,    28,    -1,    30,    31,    -1,    -1,    34,    35,    36,
      37,    38,    39,    40,    -1,    42,    43,    44,    -1,    46,
      -1,    48,    49,    -1,    51,    52,    53,    54,    -1,    56,
      57,    58,    -1,    -1,    -1,    -1,    -1,    64,    -1,    66,
      -1,    -1,    -1,    -1,    71,    -1,    73,    -1,    -1,    76,
      77,    78,    79,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    90,    91,    -1,    -1,    94,    95,    96,
      97,    -1,    -1,    -1,    -1,    -1,   103,   104,   105,   106,
     107,   108,   109,   110,    -1,   112,   113,   114,   115,    -1,
     117,   118,   119,     5,     6,     7,   123,    -1,    -1,    -1,
      -1,    -1,    -1,    15,    -1,    -1,    18,    19,    20,    -1,
      22,    -1,    -1,    -1,    -1,    27,    28,    -1,    30,    31,
      -1,    -1,    -1,    35,    36,    37,    38,    -1,    -1,    -1,
      42,    43,    -1,    -1,    46,    -1,    48,    -1,    -1,    51,
      52,    53,    54,    -1,    56,    -1,    58,    -1,    -1,    -1,
      -1,    -1,    64,    -1,    66,    -1,    -1,    -1,    -1,    71,
      -1,    73,    -1,    -1,    76,    77,    78,    79,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    90,    91,
      -1,    93,    94,    95,    -1,    97,    98,    -1,    -1,    -1,
      -1,   103,   104,   105,   106,    -1,   108,    -1,   110,    -1,
     112,   113,   114,   115,    -1,   117,   118,   119,     6,     7,
      -1,   123,    -1,    -1,    -1,    -1,    -1,    15,    16,    17,
      18,    19,    20,    -1,    22,    -1,    -1,    -1,    -1,    27,
      28,    29,    30,    31,    -1,    -1,    34,    35,    36,    37,
      38,    39,    40,    -1,    42,    43,    44,    -1,    46,    -1,
      48,    49,    -1,    51,    52,    53,    54,    -1,    56,    57,
      58,    -1,    -1,    -1,    -1,    -1,    64,    -1,    66,    -1,
      -1,    -1,    -1,    -1,    -1,    73,    -1,    -1,    -1,    77,
      -1,    79,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    90,    91,    -1,    -1,    -1,    95,    96,    97,
      -1,    -1,    -1,    -1,    -1,   103,   104,   105,   106,   107,
     108,   109,   110,   111,   112,   113,   114,   115,    -1,   117,
     118,   119,     6,     7,    -1,   123,    -1,    -1,    -1,    -1,
      -1,    15,    16,    17,    18,    19,    20,    -1,    22,    -1,
      -1,    -1,    -1,    27,    28,    29,    30,    31,    -1,    -1,
      34,    35,    36,    37,    38,    39,    40,    -1,    42,    43,
      44,    -1,    46,    -1,    48,    49,    -1,    51,    52,    53,
      54,    -1,    56,    57,    58,    -1,    -1,    -1,    -1,    -1,
      64,    -1,    66,    -1,    -1,    -1,    -1,    -1,    -1,    73,
      49,    50,    -1,    77,    -1,    79,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    90,    91,    -1,    -1,
      -1,    95,    96,    97,    -1,    -1,    -1,    -1,    -1,   103,
     104,   105,   106,   107,   108,   109,   110,   111,   112,   113,
     114,   115,    -1,   117,   118,   119,     6,     7,    97,   123,
      99,   100,    -1,    -1,   103,    15,    -1,    -1,    18,    19,
      20,    -1,    22,   112,   113,   114,   115,    27,    28,   118,
      30,    31,    -1,    -1,   123,    35,    36,    37,    38,    -1,
      -1,    -1,    42,    43,    -1,    -1,    46,    -1,    48,    -1,
      -1,    51,    52,    53,    54,    -1,    56,    -1,    58,    -1,
       5,     6,     7,     8,    64,    -1,    66,    -1,    -1,    -1,
      -1,    -1,    -1,    73,    -1,    -1,    -1,    77,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      90,    91,    -1,    38,    -1,    95,    -1,    -1,    98,    -1,
      -1,    -1,    -1,    -1,   104,   105,   106,    -1,   108,    -1,
     110,    -1,   112,    -1,    -1,   115,    -1,   117,   118,   119,
      65,    -1,    -1,   123,     5,     6,     7,     8,    -1,    -1,
      -1,    -1,    -1,    78,    -1,    49,    50,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    94,
      95,    -1,    97,    -1,    -1,    -1,    -1,    38,   103,   104,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,   112,   113,   114,
     115,    -1,    -1,   118,   119,   120,    -1,   122,    -1,    -1,
       5,     6,     7,    97,    65,    99,   100,    -1,    -1,   103,
      -1,    -1,    -1,    -1,    49,    50,    -1,    78,   112,   113,
     114,   115,    -1,    -1,   118,    -1,    -1,    -1,    -1,   123,
     124,    -1,    -1,    94,    95,    -1,    97,    -1,    -1,    -1,
      -1,    -1,   103,   104,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,   112,   113,   114,   115,    -1,    -1,   118,   119,   120,
      65,   122,    97,    -1,    99,   100,    71,    -1,   103,    -1,
      -1,    76,    -1,    78,    -1,    -1,    -1,   112,   113,   114,
     115,    -1,    12,   118,    14,    -1,    91,    -1,   123,    94,
      95,    21,    97,    -1,    24,    25,    26,    -1,   103,   104,
     105,   106,    -1,   108,    -1,    -1,    -1,   112,   113,   114,
      -1,    41,   117,    43,   119,    45,    -1,    -1,    48,    12,
      -1,    14,    -1,    -1,    -1,    55,    56,    57,    21,    -1,
      -1,    24,    25,    26,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    41,    -1,
      43,    -1,    45,    -1,    -1,    48,    -1,    -1,    -1,    -1,
      -1,    -1,    55,    56,    -1,    -1,    -1,    97,    -1,    -1,
      -1,    -1,    -1,   103,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,   112,   113,   114,   115,    -1,    -1,   118,    -1,
      -1,    -1,    -1,    -1,   124,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    97,    -1,    -1,    -1,    -1,    -1,
     103,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   112,
     113,   114,   115,    -1,    -1,   118,    -1,    -1,    -1,    -1,
      -1,   124,    15,    16,    17,    18,    19,    20,    -1,    22,
      -1,    -1,    -1,    -1,    27,    28,    -1,    30,    31,    -1,
      -1,    34,    35,    36,    37,    38,    39,    40,    -1,    42,
      43,    44,    -1,    46,    -1,    48,    49,    -1,    51,    52,
      53,    54,    -1,    56,    57,    58,    -1,    -1,    -1,    -1,
      -1,    64,    -1,    66,    -1,    -1,    -1,    -1,    -1,    -1,
      73,    -1,    -1,    -1,    77,    -1,    79,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    90,    91,    -1,
      -1,    -1,    -1,    96,    97,    -1,    -1,    -1,    -1,    -1,
     103,   104,   105,   106,   107,   108,   109,   110,   111,   112,
     113,   114,   115,    -1,   117,   118,   119,    -1,    15,    -1,
     123,    18,    19,    20,    -1,    22,    -1,    -1,    -1,    -1,
      27,    28,    -1,    30,    31,    32,    -1,    34,    35,    36,
      37,    38,    -1,    40,    -1,    42,    43,    44,    -1,    46,
      -1,    48,    -1,    -1,    51,    52,    53,    54,    -1,    56,
      -1,    58,    -1,    -1,    -1,    -1,    -1,    64,    -1,    66,
      -1,    -1,    -1,    -1,    -1,    -1,    73,    -1,    -1,    -1,
      77,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    90,    91,    -1,    -1,    -1,    -1,    -1,
      97,    98,    -1,    -1,    -1,    -1,   103,   104,   105,   106,
      -1,   108,    -1,   110,    -1,   112,   113,   114,   115,    -1,
     117,   118,    -1,    -1,    15,    -1,   123,    18,    19,    20,
      -1,    22,    -1,    -1,    -1,    -1,    27,    28,    -1,    30,
      31,    -1,    -1,    -1,    35,    36,    37,    38,    -1,    -1,
      -1,    42,    43,    -1,    -1,    46,    -1,    48,    -1,    -1,
      51,    52,    53,    54,    -1,    56,    -1,    58,    -1,    -1,
      -1,    -1,    -1,    64,    -1,    66,    -1,    -1,    -1,    -1,
      -1,    -1,    73,    -1,    -1,    -1,    77,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    90,
      91,    -1,    -1,    -1,    -1,    -1,    -1,    98,    -1,    -1,
      -1,    -1,    -1,   104,   105,   106,    -1,   108,    15,   110,
      -1,    18,    19,    20,   115,    22,   117,   118,    -1,    -1,
      27,    28,   123,    30,    31,    -1,    -1,    -1,    35,    36,
      37,    -1,    -1,    -1,    -1,    42,    43,    -1,    -1,    46,
      -1,    48,    -1,    -1,    51,    52,    53,    54,    -1,    56,
      -1,    58,    -1,    -1,    -1,    -1,    -1,    64,    -1,    66,
      -1,    -1,    -1,    -1,    -1,    -1,    73,    -1,    -1,    -1,
      77,    -1,    -1,    -1,    15,    -1,    -1,    -1,    19,    20,
      -1,    22,    -1,    90,    91,    -1,    27,    28,    -1,    30,
      31,    -1,    -1,    -1,    35,    36,    37,   104,   105,    -1,
      -1,    42,    -1,   110,    -1,    46,    -1,    -1,    -1,    -1,
      51,    -1,    53,    54,    -1,    -1,   123,    58,    -1,    -1,
      -1,    -1,    -1,    64,    -1,    66,    -1,    -1,    -1,    -1,
      -1,    -1,    73,    -1,    -1,    -1,    77,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    90,
      91,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,   104,   105,   106,    -1,   108,    -1,   110,
      -1,    -1,    -1,    -1,    -1,    -1,   117,    -1,    -1,    -1,
      -1,    -1,   123
};

/* YYSTOS[STATE-NUM] -- The symbol kind of the accessing symbol of
   state STATE-NUM.  */
static const yytype_int16 yystos[] =
{
       0,    87,   132,   134,   133,     0,     1,    15,    16,    17,
      19,    20,    22,    27,    28,    30,    31,    33,    34,    35,
      36,    37,    38,    39,    40,    42,    44,    46,    49,    51,
      53,    58,    66,    88,    97,   103,   104,   105,   112,   113,
     114,   115,   118,   123,   135,   136,   138,   142,   147,   148,
     151,   152,   161,   164,   167,   173,   174,   175,   177,   181,
     187,   188,   189,   190,   198,   200,   202,   203,   204,   205,
     223,   227,   228,   229,   308,   310,   311,   312,     1,     5,
       6,     7,    18,    22,    33,    43,    46,    48,    52,    54,
      56,    57,    64,    71,    73,    76,    77,    78,    79,    90,
      91,    93,    94,    95,    96,    98,   104,   106,   107,   108,
     109,   110,   112,   113,   114,   115,   117,   118,   119,   136,
     137,   140,   142,   143,   144,   149,   150,   154,   155,   156,
     157,   161,   164,   167,   173,   179,   182,   183,   184,   185,
     186,   191,   192,   203,   230,   242,   243,   248,   249,   250,
     254,   256,   257,   258,   259,   266,   267,   268,   273,   276,
     277,   281,   288,   289,   290,   291,   292,   302,   308,   310,
     311,   312,   313,   200,   123,   123,   115,   118,   137,   246,
     115,   118,   137,   245,   246,   309,   115,   118,   137,   245,
     123,   181,   181,   246,   180,   122,   122,    81,    85,    86,
     211,   213,   215,   219,   220,   215,   180,    32,   146,   122,
     125,   123,     1,   114,   124,   138,   152,   178,   310,   311,
     183,   200,   201,   202,   248,   288,    13,   199,   205,   195,
     196,    49,    50,    99,   100,   115,   118,   123,   140,   314,
     315,   317,   318,   320,   321,   323,   329,   330,   331,   333,
     125,    79,    79,   271,   271,   246,   123,   144,    79,   115,
     118,   137,   207,   208,   209,   210,   238,   239,   245,   208,
     241,   207,   240,    79,   115,   118,   137,   244,   276,   312,
     240,   105,   140,   193,   194,   251,   286,   274,   275,   277,
     107,   278,   275,   275,   278,    81,    85,   215,   217,    81,
      85,   217,     9,    10,   262,   264,   265,   262,   107,   275,
      81,    85,   217,   262,    79,   143,   271,   146,    29,    43,
      52,    95,   115,   118,   119,   140,   184,   186,   257,   273,
     277,   312,    82,   275,   274,   275,   306,     6,     7,     8,
      65,   115,   118,   119,   120,   122,   140,   155,   223,   259,
     260,   270,   272,   310,   312,   280,   255,   255,   275,   255,
      99,   327,   330,    88,   162,    88,   165,   125,   224,   105,
     225,   226,   125,    99,   315,   317,   330,    79,   107,   140,
     305,   168,   170,   224,     1,    67,   184,   207,   207,    85,
     214,   219,   221,   222,    81,    83,    83,   124,   123,   207,
     231,    79,   115,   118,   119,   140,   153,   154,   186,   210,
     256,   288,   299,   300,   301,   302,   303,   312,   157,   176,
     186,   199,   195,    13,   205,   192,   334,   319,    80,    81,
      84,    86,   324,    80,   324,   326,   322,   316,   101,   102,
     318,   329,   331,   332,   333,   153,   125,   160,    10,   122,
     231,   125,     6,     7,    43,    46,    52,    95,   104,   119,
     156,   273,   305,   208,   233,   234,   235,   125,   276,   125,
     123,    65,    91,   105,   140,   223,   252,   253,   257,   267,
     268,   269,    72,    74,    79,   115,   118,   129,   137,   282,
     283,   285,   312,   255,   122,   255,    81,    85,   218,   221,
      81,   221,    81,     9,    10,   221,    81,   239,   240,   180,
     255,   291,   255,    59,    60,    61,    63,   294,     8,   307,
     327,   126,   257,   255,    80,    81,    84,   328,    29,    99,
       1,   114,   124,   138,   163,   164,   167,   172,   173,   203,
     308,   312,     1,   114,   124,   138,   166,   167,   172,   203,
     308,   312,   245,   195,   205,   245,   182,     1,   124,   140,
     169,   171,   173,   182,   195,    81,    83,    79,    93,    96,
     111,   113,   114,   115,   118,   119,   136,   139,   141,   142,
     149,   157,   161,   164,   167,   173,   186,   192,   256,   267,
     268,   302,   308,   312,   313,    82,    85,   211,   216,    92,
     111,   116,   206,   212,   277,   288,   291,   302,   111,   206,
     212,   277,   288,   291,   302,   145,   125,   107,    79,   119,
     301,    10,   122,   255,   119,   120,   126,   128,   225,   195,
     123,   123,   326,   326,   326,   326,   124,   317,   330,   332,
     335,    82,   245,    12,    14,    21,    24,    25,    26,    41,
      43,    45,    48,    55,    56,   124,   137,   140,   158,   159,
     192,   232,   236,   237,   246,   208,   208,   113,   119,   141,
     156,   210,   243,   247,   288,   303,   239,   245,   240,    81,
      85,    81,    85,   112,   310,   156,    82,   156,   247,   125,
      79,   107,   140,   298,   304,   197,   255,   306,   283,    68,
      69,    70,    75,   127,   128,   284,   218,   221,    81,   156,
      81,    85,   211,   217,   156,   211,   156,   211,   125,   125,
     267,   267,   293,   125,   130,   261,    79,    96,   107,   137,
     279,    80,   326,   328,   326,   328,   326,   328,   328,   146,
     146,   225,   188,   146,    85,   214,   222,   206,   212,   277,
     288,   302,    79,   107,   109,    81,    79,    93,   113,   115,
     118,   119,   141,   256,   267,   268,   302,   312,    23,   278,
      10,   122,   114,   124,   138,   152,   153,   231,   231,   300,
     300,   301,   301,   225,   168,   168,   105,   115,   123,   140,
     227,   317,   315,   330,   335,   332,   233,   233,   246,   203,
     115,   118,   140,   125,   303,    10,   122,   125,   305,   125,
     221,    81,   221,    81,    81,    85,    99,   330,   111,   206,
     298,   119,   124,   158,   237,   307,    85,   214,   218,   221,
      81,   156,   156,   156,   119,    79,    89,   294,   295,   296,
     297,   262,   274,   326,    81,    79,    93,   107,   109,   113,
     115,   118,   119,   141,   256,   267,   268,   312,    23,   216,
      79,   107,   109,   280,   207,   207,   124,   124,   325,   125,
     247,   231,   231,    85,   211,   211,   221,    81,    81,    84,
      99,    10,   122,   208,   304,   261,    81,   211,    62,   120,
     122,   262,   263,   222,    79,   107,   109,   255,   144,   247,
      81,   211,   326,   326,   208,   208,   255,   297,    47,   287,
     124,   211,   255
};

/* YYR1[RULE-NUM] -- Symbol kind of the left-hand side of rule RULE-NUM.  */
static const yytype_int16 yyr1[] =
{
       0,   131,   132,   133,   132,   134,   134,   134,   134,   134,
     135,   135,   135,   135,   135,   135,   135,   135,   135,   135,
     135,   136,   136,   136,   136,   137,   137,   138,   138,   138,
     139,   139,   139,   139,   139,   139,   139,   139,   139,   139,
     139,   139,   139,   140,   140,   140,   141,   141,   141,   142,
     143,   143,   143,   144,   144,   144,   144,   144,   144,   144,
     144,   145,   145,   145,   145,   146,   148,   147,   150,   149,
     151,   151,   152,   152,   152,   153,   153,   153,   153,   153,
     154,   154,   154,   154,   154,   154,   154,   154,   154,   154,
     154,   154,   154,   154,   154,   155,   155,   155,   155,   155,
     155,   155,   155,   155,   156,   156,   156,   156,   156,   156,
     156,   156,   156,   156,   156,   156,   157,   158,   159,   158,
     160,   160,   160,   160,   161,   162,   162,   162,   162,   162,
     162,   162,   162,   162,   162,   163,   162,   162,   164,   165,
     165,   165,   165,   165,   165,   165,   165,   165,   166,   165,
     165,   167,   168,   168,   169,   169,   169,   170,   170,   171,
     170,   172,   173,   174,   174,   174,   175,   176,   176,   177,
     177,   177,   177,   178,   177,   177,   179,   179,   179,   180,
     180,   180,   181,   182,   182,   182,   183,   184,   184,   184,
     184,   184,   185,   186,   186,   187,   187,   188,   189,   189,
     190,   190,   191,   191,   191,   193,   192,   192,   194,   194,
     195,   196,   196,   197,   197,   197,   198,   198,   199,   199,
     200,   200,   200,   200,   200,   200,   201,   201,   201,   202,
     202,   202,   203,   203,   203,   204,   204,   204,   205,   205,
     206,   206,   206,   207,   207,   207,   208,   208,   209,   209,
     210,   210,   211,   211,   211,   211,   211,   211,   211,   211,
     211,   211,   211,   211,   211,   211,   212,   213,   213,   213,
     213,   213,   213,   213,   213,   214,   214,   214,   214,   214,
     214,   214,   214,   214,   214,   214,   214,   214,   214,   214,
     214,   214,   214,   214,   214,   214,   215,   215,   216,   216,
     217,   217,   217,   217,   218,   218,   218,   218,   219,   219,
     219,   219,   219,   219,   219,   219,   219,   219,   219,   219,
     220,   220,   220,   220,   220,   220,   220,   220,   220,   220,
     221,   221,   221,   222,   222,   223,   224,   224,   226,   225,
     225,   227,   227,   228,   228,   228,   228,   228,   229,   229,
     229,   229,   230,   231,   232,   232,   232,   232,   233,   234,
     235,   235,   235,   235,   236,   236,   236,   236,   236,   236,
     236,   236,   236,   237,   237,   237,   237,   238,   239,   239,
     240,   241,   242,   242,   242,   242,   243,   243,   243,   244,
     244,   244,   244,   245,   245,   245,   246,   246,   246,   247,
     247,   247,   247,   247,   247,   247,   247,   248,   248,   248,
     248,   248,   249,   250,   250,   251,   251,   252,   252,   252,
     252,   253,   253,   253,   254,   255,   255,   256,   256,   256,
     256,   256,   256,   256,   256,   257,   257,   258,   259,   259,
     259,   259,   260,   260,   261,   261,   262,   262,   262,   263,
     264,   264,   265,   265,   266,   267,   268,   268,   269,   269,
     270,   270,   270,   270,   271,   272,   272,   272,   272,   272,
     272,   272,   273,   273,   273,   274,   274,   275,   276,   276,
     277,   278,   278,   279,   279,   280,   280,   280,   280,   281,
     281,   282,   282,   282,   282,   282,   283,   283,   283,   283,
     283,   284,   284,   284,   284,   285,   285,   286,   286,   287,
     287,   288,   288,   289,   289,   290,   290,   290,   290,   291,
     292,   292,   293,   293,   294,   294,   294,   294,   295,   295,
     295,   296,   296,   297,   297,   297,   298,   298,   299,   299,
     299,   300,   300,   300,   301,   301,   302,   302,   302,   303,
     303,   303,   304,   304,   304,   305,   305,   305,   306,   306,
     307,   307,   309,   308,   310,   311,   312,   312,   313,   314,
     314,   314,   316,   315,   317,   317,   318,   318,   318,   319,
     318,   320,   321,   322,   322,   322,   323,   324,   324,   325,
     324,   324,   324,   324,   324,   326,   326,   327,   327,   328,
     328,   328,   328,   328,   328,   329,   330,   330,   331,   331,
     332,   332,   334,   333,   333,   333,   335,   335
};

/* YYR2[RULE-NUM] -- Number of symbols on the right-hand side of rule RULE-NUM.  */
static const yytype_int8 yyr2[] =
{
       0,     2,     1,     0,     3,     0,     2,     2,     2,     2,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     3,     1,     3,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     2,
       2,     1,     2,     0,     1,     1,     1,     2,     3,     1,
       2,     0,     2,     2,     2,     4,     0,     2,     0,     2,
       1,     1,     3,     5,     3,     1,     1,     3,     3,     2,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     2,     2,     2,     2,     4,     4,
       4,     5,     5,     5,     1,     3,     3,     4,     4,     3,
       4,     2,     3,     3,     2,     1,     4,     1,     0,     2,
       0,     2,     2,     2,     4,     0,     1,     2,     2,     2,
       2,     2,     2,     2,     2,     0,     3,     2,     4,     0,
       1,     2,     2,     2,     2,     2,     2,     2,     0,     3,
       2,     4,     1,     2,     1,     1,     1,     0,     2,     0,
       3,     1,     2,     1,     1,     1,     1,     1,     1,     2,
       2,     2,     2,     0,     3,     2,     4,     5,     2,     0,
       2,     2,     3,     1,     1,     1,     1,     1,     1,     1,
       3,     2,     1,     1,     1,     4,     5,     2,     2,     1,
       2,     3,     1,     1,     1,     0,     5,     2,     1,     1,
       1,     0,     2,     0,     2,     2,     4,     4,     3,     4,
       1,     1,     1,     1,     3,     5,     1,     1,     1,     3,
       2,     2,     1,     1,     2,     1,     1,     1,     1,     1,
       1,     3,     3,     1,     3,     3,     1,     1,     1,     1,
       1,     1,     1,     4,     5,     5,     5,     4,     4,     3,
       3,     3,     3,     3,     3,     3,     4,     4,     4,     4,
       4,     4,     4,     4,     4,     1,     4,     4,     4,     4,
       4,     4,     4,     4,     5,     5,     5,     4,     4,     4,
       4,     3,     3,     3,     3,     3,     1,     4,     1,     3,
       2,     2,     3,     3,     2,     2,     3,     3,     1,     3,
       3,     3,     4,     3,     3,     3,     3,     3,     3,     3,
       3,     3,     3,     3,     3,     3,     3,     3,     4,     4,
       1,     4,     2,     1,     3,     2,     0,     2,     0,     2,
       1,     2,     3,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     4,     1,     3,     2,     4,     2,     3,     1,
       1,     5,     5,     4,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     2,     2,     2,     1,     1,     2,
       1,     1,     4,     3,     4,     2,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     2,     1,     3,     3,     2,     3,     3,
       1,     1,     3,     3,     3,     0,     2,     5,     2,     1,
       1,     1,     1,     1,     2,     0,     2,     1,     2,     1,
       1,     2,     2,     2,     2,     1,     2,     1,     1,     1,
       1,     1,     1,     1,     0,     2,     0,     1,     1,     1,
       1,     2,     1,     2,     2,     2,     1,     2,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     0,     1,     1,     2,     2,
       2,     0,     2,     1,     1,     0,     3,     3,     3,     1,
       2,     1,     1,     1,     1,     1,     1,     1,     2,     2,
       2,     1,     1,     1,     1,     1,     2,     0,     2,     0,
       1,     8,     2,     2,     2,     1,     1,     1,     1,     3,
       2,     4,     0,     2,     1,     1,     2,     2,     1,     1,
       1,     1,     3,     1,     2,     2,     1,     3,     3,     3,
       1,     1,     3,     3,     2,     1,     1,     1,     2,     1,
       1,     2,     1,     1,     1,     1,     1,     1,     0,     2,
       0,     2,     0,     3,     1,     2,     2,     1,     3,     1,
       2,     1,     0,     3,     1,     1,     1,     1,     2,     0,
       5,     2,     3,     0,     2,     2,     2,     3,     3,     0,
       6,     3,     3,     3,     1,     1,     3,     2,     3,     2,
       2,     2,     2,     2,     2,     3,     1,     2,     1,     1,
       1,     1,     0,     5,     2,     2,     2,     1
};


enum { YYENOMEM = -2 };

#define yyerrok         (yyerrstatus = 0)
#define yyclearin       (yychar = YYEMPTY)

#define YYACCEPT        goto yyacceptlab
#define YYABORT         goto yyabortlab
#define YYERROR         goto yyerrorlab
#define YYNOMEM         goto yyexhaustedlab


#define YYRECOVERING()  (!!yyerrstatus)

#define YYBACKUP(Token, Value)                                    \
  do                                                              \
    if (yychar == YYEMPTY)                                        \
      {                                                           \
        yychar = (Token);                                         \
        yylval = (Value);                                         \
        YYPOPSTACK (yylen);                                       \
        yystate = *yyssp;                                         \
        goto yybackup;                                            \
      }                                                           \
    else                                                          \
      {                                                           \
        yyerror (&yylloc, parser, retval, YY_("syntax error: cannot back up")); \
        YYERROR;                                                  \
      }                                                           \
  while (0)

/* Backward compatibility with an undocumented macro.
   Use YYerror or YYUNDEF. */
#define YYERRCODE YYUNDEF

/* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
   If N is 0, then set CURRENT to the empty location which ends
   the previous symbol: RHS[0] (always defined).  */

#ifndef YYLLOC_DEFAULT
# define YYLLOC_DEFAULT(Current, Rhs, N)                                \
    do                                                                  \
      if (N)                                                            \
        {                                                               \
          (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;        \
          (Current).first_column = YYRHSLOC (Rhs, 1).first_column;      \
          (Current).last_line    = YYRHSLOC (Rhs, N).last_line;         \
          (Current).last_column  = YYRHSLOC (Rhs, N).last_column;       \
        }                                                               \
      else                                                              \
        {                                                               \
          (Current).first_line   = (Current).last_line   =              \
            YYRHSLOC (Rhs, 0).last_line;                                \
          (Current).first_column = (Current).last_column =              \
            YYRHSLOC (Rhs, 0).last_column;                              \
        }                                                               \
    while (0)
#endif

#define YYRHSLOC(Rhs, K) ((Rhs)[K])


/* Enable debugging if requested.  */
#if YYDEBUG

# ifndef YYFPRINTF
#  include <stdio.h> /* INFRINGES ON USER NAME SPACE */
#  define YYFPRINTF fprintf
# endif

# define YYDPRINTF(Args)                        \
do {                                            \
  if (yydebug)                                  \
    YYFPRINTF Args;                             \
} while (0)


/* YYLOCATION_PRINT -- Print the location on the stream.
   This macro was not mandated originally: define only if we know
   we won't break user code: when these are the locations we know.  */

# ifndef YYLOCATION_PRINT

#  if defined YY_LOCATION_PRINT

   /* Temporary convenience wrapper in case some people defined the
      undocumented and private YY_LOCATION_PRINT macros.  */
#   define YYLOCATION_PRINT(File, Loc)  YY_LOCATION_PRINT(File, *(Loc))

#  elif defined YYLTYPE_IS_TRIVIAL && YYLTYPE_IS_TRIVIAL

/* Print *YYLOCP on YYO.  Private, do not rely on its existence. */

YY_ATTRIBUTE_UNUSED
static int
yy_location_print_ (FILE *yyo, YYLTYPE const * const yylocp)
{
  int res = 0;
  int end_col = 0 != yylocp->last_column ? yylocp->last_column - 1 : 0;
  if (0 <= yylocp->first_line)
    {
      res += YYFPRINTF (yyo, "%d", yylocp->first_line);
      if (0 <= yylocp->first_column)
        res += YYFPRINTF (yyo, ".%d", yylocp->first_column);
    }
  if (0 <= yylocp->last_line)
    {
      if (yylocp->first_line < yylocp->last_line)
        {
          res += YYFPRINTF (yyo, "-%d", yylocp->last_line);
          if (0 <= end_col)
            res += YYFPRINTF (yyo, ".%d", end_col);
        }
      else if (0 <= end_col && yylocp->first_column < end_col)
        res += YYFPRINTF (yyo, "-%d", end_col);
    }
  return res;
}

#   define YYLOCATION_PRINT  yy_location_print_

    /* Temporary convenience wrapper in case some people defined the
       undocumented and private YY_LOCATION_PRINT macros.  */
#   define YY_LOCATION_PRINT(File, Loc)  YYLOCATION_PRINT(File, &(Loc))

#  else

#   define YYLOCATION_PRINT(File, Loc) ((void) 0)
    /* Temporary convenience wrapper in case some people defined the
       undocumented and private YY_LOCATION_PRINT macros.  */
#   define YY_LOCATION_PRINT  YYLOCATION_PRINT

#  endif
# endif /* !defined YYLOCATION_PRINT */


# define YY_SYMBOL_PRINT(Title, Kind, Value, Location)                    \
do {                                                                      \
  if (yydebug)                                                            \
    {                                                                     \
      YYFPRINTF (stderr, "%s ", Title);                                   \
      yy_symbol_print (stderr,                                            \
                  Kind, Value, Location, parser, retval); \
      YYFPRINTF (stderr, "\n");                                           \
    }                                                                     \
} while (0)


/*-----------------------------------.
| Print this symbol's value on YYO.  |
`-----------------------------------*/

static void
yy_symbol_value_print (FILE *yyo,
                       yysymbol_kind_t yykind, YYSTYPE const * const yyvaluep, YYLTYPE const * const yylocationp, Lily_parser *parser, SCM *retval)
{
  FILE *yyoutput = yyo;
  YY_USE (yyoutput);
  YY_USE (yylocationp);
  YY_USE (parser);
  YY_USE (retval);
  if (!yyvaluep)
    return;
  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  YY_USE (yykind);
  YY_IGNORE_MAYBE_UNINITIALIZED_END
}


/*---------------------------.
| Print this symbol on YYO.  |
`---------------------------*/

static void
yy_symbol_print (FILE *yyo,
                 yysymbol_kind_t yykind, YYSTYPE const * const yyvaluep, YYLTYPE const * const yylocationp, Lily_parser *parser, SCM *retval)
{
  YYFPRINTF (yyo, "%s %s (",
             yykind < YYNTOKENS ? "token" : "nterm", yysymbol_name (yykind));

  YYLOCATION_PRINT (yyo, yylocationp);
  YYFPRINTF (yyo, ": ");
  yy_symbol_value_print (yyo, yykind, yyvaluep, yylocationp, parser, retval);
  YYFPRINTF (yyo, ")");
}

/*------------------------------------------------------------------.
| yy_stack_print -- Print the state stack from its BOTTOM up to its |
| TOP (included).                                                   |
`------------------------------------------------------------------*/

static void
yy_stack_print (yy_state_t *yybottom, yy_state_t *yytop)
{
  YYFPRINTF (stderr, "Stack now");
  for (; yybottom <= yytop; yybottom++)
    {
      int yybot = *yybottom;
      YYFPRINTF (stderr, " %d", yybot);
    }
  YYFPRINTF (stderr, "\n");
}

# define YY_STACK_PRINT(Bottom, Top)                            \
do {                                                            \
  if (yydebug)                                                  \
    yy_stack_print ((Bottom), (Top));                           \
} while (0)


/*------------------------------------------------.
| Report that the YYRULE is going to be reduced.  |
`------------------------------------------------*/

static void
yy_reduce_print (yy_state_t *yyssp, YYSTYPE *yyvsp, YYLTYPE *yylsp,
                 int yyrule, Lily_parser *parser, SCM *retval)
{
  int yylno = yyrline[yyrule];
  int yynrhs = yyr2[yyrule];
  int yyi;
  YYFPRINTF (stderr, "Reducing stack by rule %d (line %d):\n",
             yyrule - 1, yylno);
  /* The symbols being reduced.  */
  for (yyi = 0; yyi < yynrhs; yyi++)
    {
      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
      yy_symbol_print (stderr,
                       YY_ACCESSING_SYMBOL (+yyssp[yyi + 1 - yynrhs]),
                       &yyvsp[(yyi + 1) - (yynrhs)],
                       &(yylsp[(yyi + 1) - (yynrhs)]), parser, retval);
      YYFPRINTF (stderr, "\n");
    }
}

# define YY_REDUCE_PRINT(Rule)          \
do {                                    \
  if (yydebug)                          \
    yy_reduce_print (yyssp, yyvsp, yylsp, Rule, parser, retval); \
} while (0)

/* Nonzero means print parse trace.  It is left uninitialized so that
   multiple parsers can coexist.  */
int yydebug;
#else /* !YYDEBUG */
# define YYDPRINTF(Args) ((void) 0)
# define YY_SYMBOL_PRINT(Title, Kind, Value, Location)
# define YY_STACK_PRINT(Bottom, Top)
# define YY_REDUCE_PRINT(Rule)
#endif /* !YYDEBUG */


/* YYINITDEPTH -- initial size of the parser's stacks.  */
#ifndef YYINITDEPTH
# define YYINITDEPTH 200
#endif

/* YYMAXDEPTH -- maximum size the stacks can grow to (effective only
   if the built-in stack extension method is used).

   Do not make this value too large; the results are undefined if
   YYSTACK_ALLOC_MAXIMUM < YYSTACK_BYTES (YYMAXDEPTH)
   evaluated with infinite-precision integer arithmetic.  */

#ifndef YYMAXDEPTH
# define YYMAXDEPTH 10000
#endif


/* Context of a parse error.  */
typedef struct
{
  yy_state_t *yyssp;
  yysymbol_kind_t yytoken;
  YYLTYPE *yylloc;
} yypcontext_t;

/* Put in YYARG at most YYARGN of the expected tokens given the
   current YYCTX, and return the number of tokens stored in YYARG.  If
   YYARG is null, return the number of expected tokens (guaranteed to
   be less than YYNTOKENS).  Return YYENOMEM on memory exhaustion.
   Return 0 if there are more than YYARGN expected tokens, yet fill
   YYARG up to YYARGN. */
static int
yypcontext_expected_tokens (const yypcontext_t *yyctx,
                            yysymbol_kind_t yyarg[], int yyargn)
{
  /* Actual size of YYARG. */
  int yycount = 0;
  int yyn = yypact[+*yyctx->yyssp];
  if (!yypact_value_is_default (yyn))
    {
      /* Start YYX at -YYN if negative to avoid negative indexes in
         YYCHECK.  In other words, skip the first -YYN actions for
         this state because they are default actions.  */
      int yyxbegin = yyn < 0 ? -yyn : 0;
      /* Stay within bounds of both yycheck and yytname.  */
      int yychecklim = YYLAST - yyn + 1;
      int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
      int yyx;
      for (yyx = yyxbegin; yyx < yyxend; ++yyx)
        if (yycheck[yyx + yyn] == yyx && yyx != YYSYMBOL_YYerror
            && !yytable_value_is_error (yytable[yyx + yyn]))
          {
            if (!yyarg)
              ++yycount;
            else if (yycount == yyargn)
              return 0;
            else
              yyarg[yycount++] = YY_CAST (yysymbol_kind_t, yyx);
          }
    }
  if (yyarg && yycount == 0 && 0 < yyargn)
    yyarg[0] = YYSYMBOL_YYEMPTY;
  return yycount;
}




#ifndef yystrlen
# if defined __GLIBC__ && defined _STRING_H
#  define yystrlen(S) (YY_CAST (YYPTRDIFF_T, strlen (S)))
# else
/* Return the length of YYSTR.  */
static YYPTRDIFF_T
yystrlen (const char *yystr)
{
  YYPTRDIFF_T yylen;
  for (yylen = 0; yystr[yylen]; yylen++)
    continue;
  return yylen;
}
# endif
#endif

#ifndef yystpcpy
# if defined __GLIBC__ && defined _STRING_H && defined _GNU_SOURCE
#  define yystpcpy stpcpy
# else
/* Copy YYSRC to YYDEST, returning the address of the terminating '\0' in
   YYDEST.  */
static char *
yystpcpy (char *yydest, const char *yysrc)
{
  char *yyd = yydest;
  const char *yys = yysrc;

  while ((*yyd++ = *yys++) != '\0')
    continue;

  return yyd - 1;
}
# endif
#endif

#ifndef yytnamerr
/* Copy to YYRES the contents of YYSTR after stripping away unnecessary
   quotes and backslashes, so that it's suitable for yyerror.  The
   heuristic is that double-quoting is unnecessary unless the string
   contains an apostrophe, a comma, or backslash (other than
   backslash-backslash).  YYSTR is taken from yytname.  If YYRES is
   null, do not copy; instead, return the length of what the result
   would have been.  */
static YYPTRDIFF_T
yytnamerr (char *yyres, const char *yystr)
{
  if (*yystr == '"')
    {
      YYPTRDIFF_T yyn = 0;
      char const *yyp = yystr;
      for (;;)
        switch (*++yyp)
          {
          case '\'':
          case ',':
            goto do_not_strip_quotes;

          case '\\':
            if (*++yyp != '\\')
              goto do_not_strip_quotes;
            else
              goto append;

          append:
          default:
            if (yyres)
              yyres[yyn] = *yyp;
            yyn++;
            break;

          case '"':
            if (yyres)
              yyres[yyn] = '\0';
            return yyn;
          }
    do_not_strip_quotes: ;
    }

  if (yyres)
    return yystpcpy (yyres, yystr) - yyres;
  else
    return yystrlen (yystr);
}
#endif


static int
yy_syntax_error_arguments (const yypcontext_t *yyctx,
                           yysymbol_kind_t yyarg[], int yyargn)
{
  /* Actual size of YYARG. */
  int yycount = 0;
  /* There are many possibilities here to consider:
     - If this state is a consistent state with a default action, then
       the only way this function was invoked is if the default action
       is an error action.  In that case, don't check for expected
       tokens because there are none.
     - The only way there can be no lookahead present (in yychar) is if
       this state is a consistent state with a default action.  Thus,
       detecting the absence of a lookahead is sufficient to determine
       that there is no unexpected or expected token to report.  In that
       case, just report a simple "syntax error".
     - Don't assume there isn't a lookahead just because this state is a
       consistent state with a default action.  There might have been a
       previous inconsistent state, consistent state with a non-default
       action, or user semantic action that manipulated yychar.
     - Of course, the expected token list depends on states to have
       correct lookahead information, and it depends on the parser not
       to perform extra reductions after fetching a lookahead from the
       scanner and before detecting a syntax error.  Thus, state merging
       (from LALR or IELR) and default reductions corrupt the expected
       token list.  However, the list is correct for canonical LR with
       one exception: it will still contain any token that will not be
       accepted due to an error action in a later state.
  */
  if (yyctx->yytoken != YYSYMBOL_YYEMPTY)
    {
      int yyn;
      if (yyarg)
        yyarg[yycount] = yyctx->yytoken;
      ++yycount;
      yyn = yypcontext_expected_tokens (yyctx,
                                        yyarg ? yyarg + 1 : yyarg, yyargn - 1);
      if (yyn == YYENOMEM)
        return YYENOMEM;
      else
        yycount += yyn;
    }
  return yycount;
}

/* Copy into *YYMSG, which is of size *YYMSG_ALLOC, an error message
   about the unexpected token YYTOKEN for the state stack whose top is
   YYSSP.

   Return 0 if *YYMSG was successfully written.  Return -1 if *YYMSG is
   not large enough to hold the message.  In that case, also set
   *YYMSG_ALLOC to the required number of bytes.  Return YYENOMEM if the
   required number of bytes is too large to store.  */
static int
yysyntax_error (YYPTRDIFF_T *yymsg_alloc, char **yymsg,
                const yypcontext_t *yyctx)
{
  enum { YYARGS_MAX = 5 };
  /* Internationalized format string. */
  const char *yyformat = YY_NULLPTR;
  /* Arguments of yyformat: reported tokens (one for the "unexpected",
     one per "expected"). */
  yysymbol_kind_t yyarg[YYARGS_MAX];
  /* Cumulated lengths of YYARG.  */
  YYPTRDIFF_T yysize = 0;

  /* Actual size of YYARG. */
  int yycount = yy_syntax_error_arguments (yyctx, yyarg, YYARGS_MAX);
  if (yycount == YYENOMEM)
    return YYENOMEM;

  switch (yycount)
    {
#define YYCASE_(N, S)                       \
      case N:                               \
        yyformat = S;                       \
        break
    default: /* Avoid compiler warnings. */
      YYCASE_(0, YY_("syntax error"));
      YYCASE_(1, YY_("syntax error, unexpected %s"));
      YYCASE_(2, YY_("syntax error, unexpected %s, expecting %s"));
      YYCASE_(3, YY_("syntax error, unexpected %s, expecting %s or %s"));
      YYCASE_(4, YY_("syntax error, unexpected %s, expecting %s or %s or %s"));
      YYCASE_(5, YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s"));
#undef YYCASE_
    }

  /* Compute error message size.  Don't count the "%s"s, but reserve
     room for the terminator.  */
  yysize = yystrlen (yyformat) - 2 * yycount + 1;
  {
    int yyi;
    for (yyi = 0; yyi < yycount; ++yyi)
      {
        YYPTRDIFF_T yysize1
          = yysize + yytnamerr (YY_NULLPTR, yytname[yyarg[yyi]]);
        if (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM)
          yysize = yysize1;
        else
          return YYENOMEM;
      }
  }

  if (*yymsg_alloc < yysize)
    {
      *yymsg_alloc = 2 * yysize;
      if (! (yysize <= *yymsg_alloc
             && *yymsg_alloc <= YYSTACK_ALLOC_MAXIMUM))
        *yymsg_alloc = YYSTACK_ALLOC_MAXIMUM;
      return -1;
    }

  /* Avoid sprintf, as that infringes on the user's name space.
     Don't have undefined behavior even if the translation
     produced a string with the wrong number of "%s"s.  */
  {
    char *yyp = *yymsg;
    int yyi = 0;
    while ((*yyp = *yyformat) != '\0')
      if (*yyp == '%' && yyformat[1] == 's' && yyi < yycount)
        {
          yyp += yytnamerr (yyp, yytname[yyarg[yyi++]]);
          yyformat += 2;
        }
      else
        {
          ++yyp;
          ++yyformat;
        }
  }
  return 0;
}


/*-----------------------------------------------.
| Release the memory associated to this symbol.  |
`-----------------------------------------------*/

static void
yydestruct (const char *yymsg,
            yysymbol_kind_t yykind, YYSTYPE *yyvaluep, YYLTYPE *yylocationp, Lily_parser *parser, SCM *retval)
{
  YY_USE (yyvaluep);
  YY_USE (yylocationp);
  YY_USE (parser);
  YY_USE (retval);
  if (!yymsg)
    yymsg = "Deleting";
  YY_SYMBOL_PRINT (yymsg, yykind, yyvaluep, yylocationp);

  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  YY_USE (yykind);
  YY_IGNORE_MAYBE_UNINITIALIZED_END
}






/*----------.
| yyparse.  |
`----------*/

int
yyparse (Lily_parser *parser, SCM *retval)
{
/* Lookahead token kind.  */
int yychar;


/* The semantic value of the lookahead symbol.  */
/* Default value used for initialization, for pacifying older GCCs
   or non-GCC compilers.  */
YY_INITIAL_VALUE (static YYSTYPE yyval_default;)
YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);

/* Location data for the lookahead symbol.  */
static YYLTYPE yyloc_default
# if defined YYLTYPE_IS_TRIVIAL && YYLTYPE_IS_TRIVIAL
  = { 1, 1, 1, 1 }
# endif
;
YYLTYPE yylloc = yyloc_default;

    /* Number of syntax errors so far.  */
    int yynerrs = 0;

    yy_state_fast_t yystate = 0;
    /* Number of tokens to shift before error messages enabled.  */
    int yyerrstatus = 0;

    /* Refer to the stacks through separate pointers, to allow yyoverflow
       to reallocate them elsewhere.  */

    /* Their size.  */
    YYPTRDIFF_T yystacksize = YYINITDEPTH;

    /* The state stack: array, bottom, top.  */
    yy_state_t yyssa[YYINITDEPTH];
    yy_state_t *yyss = yyssa;
    yy_state_t *yyssp = yyss;

    /* The semantic value stack: array, bottom, top.  */
    YYSTYPE yyvsa[YYINITDEPTH];
    YYSTYPE *yyvs = yyvsa;
    YYSTYPE *yyvsp = yyvs;

    /* The location stack: array, bottom, top.  */
    YYLTYPE yylsa[YYINITDEPTH];
    YYLTYPE *yyls = yylsa;
    YYLTYPE *yylsp = yyls;

  int yyn;
  /* The return value of yyparse.  */
  int yyresult;
  /* Lookahead symbol kind.  */
  yysymbol_kind_t yytoken = YYSYMBOL_YYEMPTY;
  /* The variables used to return semantic value and location from the
     action routines.  */
  YYSTYPE yyval;
  YYLTYPE yyloc;

  /* The locations where the error started and ended.  */
  YYLTYPE yyerror_range[3];

  /* Buffer for error messages, and its allocated size.  */
  char yymsgbuf[128];
  char *yymsg = yymsgbuf;
  YYPTRDIFF_T yymsg_alloc = sizeof yymsgbuf;

#define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N), yylsp -= (N))

  /* The number of symbols on the RHS of the reduced rule.
     Keep to zero when no symbol should be popped.  */
  int yylen = 0;

  YYDPRINTF ((stderr, "Starting parse\n"));

  yychar = YYEMPTY; /* Cause a token to be read.  */

  yylsp[0] = yylloc;
  goto yysetstate;


/*------------------------------------------------------------.
| yynewstate -- push a new state, which is found in yystate.  |
`------------------------------------------------------------*/
yynewstate:
  /* In all cases, when you get here, the value and location stacks
     have just been pushed.  So pushing a state here evens the stacks.  */
  yyssp++;


/*--------------------------------------------------------------------.
| yysetstate -- set current state (the top of the stack) to yystate.  |
`--------------------------------------------------------------------*/
yysetstate:
  YYDPRINTF ((stderr, "Entering state %d\n", yystate));
  YY_ASSERT (0 <= yystate && yystate < YYNSTATES);
  YY_IGNORE_USELESS_CAST_BEGIN
  *yyssp = YY_CAST (yy_state_t, yystate);
  YY_IGNORE_USELESS_CAST_END
  YY_STACK_PRINT (yyss, yyssp);

  if (yyss + yystacksize - 1 <= yyssp)
#if !defined yyoverflow && !defined YYSTACK_RELOCATE
    YYNOMEM;
#else
    {
      /* Get the current used size of the three stacks, in elements.  */
      YYPTRDIFF_T yysize = yyssp - yyss + 1;

# if defined yyoverflow
      {
        /* Give user a chance to reallocate the stack.  Use copies of
           these so that the &'s don't force the real ones into
           memory.  */
        yy_state_t *yyss1 = yyss;
        YYSTYPE *yyvs1 = yyvs;
        YYLTYPE *yyls1 = yyls;

        /* Each stack pointer address is followed by the size of the
           data in use in that stack, in bytes.  This used to be a
           conditional around just the two extra args, but that might
           be undefined if yyoverflow is a macro.  */
        yyoverflow (YY_("memory exhausted"),
                    &yyss1, yysize * YYSIZEOF (*yyssp),
                    &yyvs1, yysize * YYSIZEOF (*yyvsp),
                    &yyls1, yysize * YYSIZEOF (*yylsp),
                    &yystacksize);
        yyss = yyss1;
        yyvs = yyvs1;
        yyls = yyls1;
      }
# else /* defined YYSTACK_RELOCATE */
      /* Extend the stack our own way.  */
      if (YYMAXDEPTH <= yystacksize)
        YYNOMEM;
      yystacksize *= 2;
      if (YYMAXDEPTH < yystacksize)
        yystacksize = YYMAXDEPTH;

      {
        yy_state_t *yyss1 = yyss;
        union yyalloc *yyptr =
          YY_CAST (union yyalloc *,
                   YYSTACK_ALLOC (YY_CAST (YYSIZE_T, YYSTACK_BYTES (yystacksize))));
        if (! yyptr)
          YYNOMEM;
        YYSTACK_RELOCATE (yyss_alloc, yyss);
        YYSTACK_RELOCATE (yyvs_alloc, yyvs);
        YYSTACK_RELOCATE (yyls_alloc, yyls);
#  undef YYSTACK_RELOCATE
        if (yyss1 != yyssa)
          YYSTACK_FREE (yyss1);
      }
# endif

      yyssp = yyss + yysize - 1;
      yyvsp = yyvs + yysize - 1;
      yylsp = yyls + yysize - 1;

      YY_IGNORE_USELESS_CAST_BEGIN
      YYDPRINTF ((stderr, "Stack size increased to %ld\n",
                  YY_CAST (long, yystacksize)));
      YY_IGNORE_USELESS_CAST_END

      if (yyss + yystacksize - 1 <= yyssp)
        YYABORT;
    }
#endif /* !defined yyoverflow && !defined YYSTACK_RELOCATE */


  if (yystate == YYFINAL)
    YYACCEPT;

  goto yybackup;


/*-----------.
| yybackup.  |
`-----------*/
yybackup:
  /* Do appropriate processing given the current state.  Read a
     lookahead token if we need one and don't already have one.  */

  /* First try to decide what to do without reference to lookahead token.  */
  yyn = yypact[yystate];
  if (yypact_value_is_default (yyn))
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* YYCHAR is either empty, or end-of-input, or a valid lookahead.  */
  if (yychar == YYEMPTY)
    {
      YYDPRINTF ((stderr, "Reading a token\n"));
      yychar = yylex (&yylval, &yylloc, parser);
    }

  if (yychar <= END_OF_FILE)
    {
      yychar = END_OF_FILE;
      yytoken = YYSYMBOL_YYEOF;
      YYDPRINTF ((stderr, "Now at end of input.\n"));
    }
  else if (yychar == YYerror)
    {
      /* The scanner already issued an error message, process directly
         to error recovery.  But do not keep the error token as
         lookahead, it is too special and may lead us to an endless
         loop in error recovery. */
      yychar = YYUNDEF;
      yytoken = YYSYMBOL_YYerror;
      yyerror_range[1] = yylloc;
      goto yyerrlab1;
    }
  else
    {
      yytoken = YYTRANSLATE (yychar);
      YY_SYMBOL_PRINT ("Next token is", yytoken, &yylval, &yylloc);
    }

  /* If the proper action on seeing token YYTOKEN is to reduce or to
     detect an error, take that action.  */
  yyn += yytoken;
  if (yyn < 0 || YYLAST < yyn || yycheck[yyn] != yytoken)
    goto yydefault;
  yyn = yytable[yyn];
  if (yyn <= 0)
    {
      if (yytable_value_is_error (yyn))
        goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }

  /* Count tokens shifted since error; after three, turn off error
     status.  */
  if (yyerrstatus)
    yyerrstatus--;

  /* Shift the lookahead token.  */
  YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
  yystate = yyn;
  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  *++yyvsp = yylval;
  YY_IGNORE_MAYBE_UNINITIALIZED_END
  *++yylsp = yylloc;

  /* Discard the shifted token.  */
  yychar = YYEMPTY;
  goto yynewstate;


/*-----------------------------------------------------------.
| yydefault -- do the default action for the current state.  |
`-----------------------------------------------------------*/
yydefault:
  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;
  goto yyreduce;


/*-----------------------------.
| yyreduce -- do a reduction.  |
`-----------------------------*/
yyreduce:
  /* yyn is the number of a rule to reduce with.  */
  yylen = yyr2[yyn];

  /* If YYLEN is nonzero, implement the default value of the action:
     '$$ = $1'.

     Otherwise, the following line sets YYVAL to garbage.
     This behavior is undocumented and Bison
     users should not rely upon it.  Assigning to YYVAL
     unconditionally makes the parser a bit smaller, and it avoids a
     GCC warning that YYVAL may be used uninitialized.  */
  yyval = yyvsp[1-yylen];

  /* Default location. */
  YYLLOC_DEFAULT (yyloc, (yylsp - yylen), yylen);
  yyerror_range[1] = yyloc;
  YY_REDUCE_PRINT (yyn);
  switch (yyn)
    {
  case 3: /* $@1: %empty  */
#line 410 "parser.yy"
                        {
		parser->lexer_->push_note_state ();
	}
#line 3553 "./out/parser.cc"
    break;

  case 4: /* start_symbol: "#{" $@1 embedded_lilypond  */
#line 412 "parser.yy"
                            {
		parser->lexer_->pop_state ();
                *retval = yyvsp[0];
 	}
#line 3562 "./out/parser.cc"
    break;

  case 5: /* lilypond: %empty  */
#line 418 "parser.yy"
                            { yyval = SCM_UNSPECIFIED; }
#line 3568 "./out/parser.cc"
    break;

  case 6: /* lilypond: lilypond toplevel_expression  */
#line 419 "parser.yy"
                                       {
	}
#line 3575 "./out/parser.cc"
    break;

  case 7: /* lilypond: lilypond assignment  */
#line 421 "parser.yy"
                              {
	}
#line 3582 "./out/parser.cc"
    break;

  case 8: /* lilypond: lilypond error  */
#line 423 "parser.yy"
                         {
		parser->error_level_ = 1;
	}
#line 3590 "./out/parser.cc"
    break;

  case 9: /* lilypond: lilypond "\\version-error"  */
#line 426 "parser.yy"
                                {
		parser->error_level_ = 1;
	}
#line 3598 "./out/parser.cc"
    break;

  case 10: /* toplevel_expression: header_block  */
#line 433 "parser.yy"
                     {
		parser->lexer_->set_identifier (ly_symbol2scm ("$defaultheader"), yyvsp[0]);
	}
#line 3606 "./out/parser.cc"
    break;

  case 11: /* toplevel_expression: book_block  */
#line 436 "parser.yy"
                     {
		SCM proc = parser->lexer_->lookup_identifier ("toplevel-book-handler");
		ly_call (proc, yyvsp[0]);
	}
#line 3615 "./out/parser.cc"
    break;

  case 12: /* toplevel_expression: bookpart_block  */
#line 440 "parser.yy"
                         {
		SCM proc = parser->lexer_->lookup_identifier ("toplevel-bookpart-handler");
		ly_call (proc, yyvsp[0]);
	}
#line 3624 "./out/parser.cc"
    break;

  case 13: /* toplevel_expression: BOOK_IDENTIFIER  */
#line 444 "parser.yy"
                          {
		SCM sym = unsmob<Book>(yyvsp[0])->paper_
			? ly_symbol2scm ("toplevel-book-handler")
			: ly_symbol2scm ("toplevel-bookpart-handler");

		SCM proc = parser->lexer_->lookup_identifier_symbol (sym);
		ly_call (proc, yyvsp[0]);
	}
#line 3637 "./out/parser.cc"
    break;

  case 14: /* toplevel_expression: score_block  */
#line 452 "parser.yy"
                      {
		SCM proc = parser->lexer_->lookup_identifier ("toplevel-score-handler");
		ly_call (proc, yyvsp[0]);
	}
#line 3646 "./out/parser.cc"
    break;

  case 15: /* toplevel_expression: composite_music  */
#line 456 "parser.yy"
                          {
		SCM proc = parser->lexer_->lookup_identifier ("toplevel-music-handler");
		ly_call (proc, yyvsp[0]);
	}
#line 3655 "./out/parser.cc"
    break;

  case 16: /* toplevel_expression: full_markup  */
#line 460 "parser.yy"
                      {
		SCM proc = parser->lexer_->lookup_identifier ("toplevel-text-handler");
		ly_call (proc, ly_list (yyvsp[0]));
	}
#line 3664 "./out/parser.cc"
    break;

  case 17: /* toplevel_expression: full_markup_list  */
#line 464 "parser.yy"
                           {
		SCM proc = parser->lexer_->lookup_identifier ("toplevel-text-handler");
		ly_call (proc, yyvsp[0]);
	}
#line 3673 "./out/parser.cc"
    break;

  case 18: /* toplevel_expression: SCM_TOKEN  */
#line 468 "parser.yy"
                    {
		// Evaluate and ignore #xxx, as opposed to \xxx
		parser->lexer_->eval_scm_token (yyvsp[0], (yylsp[0]));
	}
#line 3682 "./out/parser.cc"
    break;

  case 19: /* toplevel_expression: embedded_scm_active  */
#line 473 "parser.yy"
        {
		SCM out = SCM_UNDEFINED;
		if (Text_interface::is_markup (yyvsp[0]))
			out = ly_list (yyvsp[0]);
		else if (Text_interface::is_markup_list (yyvsp[0]))
			out = yyvsp[0];
		if (scm_is_pair (out))
		{
			SCM proc = parser->lexer_->lookup_identifier ("toplevel-text-handler");
			ly_call (proc, out);
		} else if (unsmob<Score> (yyvsp[0]))
		{
			SCM proc = parser->lexer_->lookup_identifier ("toplevel-score-handler");
			ly_call (proc, yyvsp[0]);
		} else if (Output_def * od = unsmob<Output_def> (yyvsp[0])) {
			SCM id = SCM_EOL;
			SCM kind = od->c_variable ("output-def-kind");
			if (scm_is_eq (kind, ly_symbol2scm ("paper")))
				id = ly_symbol2scm ("$defaultpaper");
			else if (scm_is_eq (kind, ly_symbol2scm ("midi")))
				id = ly_symbol2scm ("$defaultmidi");
			else if (scm_is_eq (kind, ly_symbol2scm ("layout")))
				id = ly_symbol2scm ("$defaultlayout");

			parser->lexer_->set_identifier (id, yyvsp[0]);
		} else if (ly_is_module (yyvsp[0]))
		{
			SCM module = get_header (parser);
			ly_module_copy (module, yyvsp[0]);
			parser->lexer_->set_identifier
				(ly_symbol2scm ("$defaultheader"), module);
		} else if (!scm_is_eq (yyvsp[0], SCM_UNSPECIFIED))
			parser->parser_error ((yylsp[0]), _("bad expression type"));
	}
#line 3721 "./out/parser.cc"
    break;

  case 20: /* toplevel_expression: output_def  */
#line 507 "parser.yy"
                     {
		SCM id = SCM_EOL;
		Output_def * od = unsmob<Output_def> (yyvsp[0]);
		SCM kind = od->c_variable ("output-def-kind");
		if (scm_is_eq (kind, ly_symbol2scm ("paper")))
			id = ly_symbol2scm ("$defaultpaper");
		else if (scm_is_eq (kind, ly_symbol2scm ("midi")))
			id = ly_symbol2scm ("$defaultmidi");
		else if (scm_is_eq (kind, ly_symbol2scm ("layout")))
			id = ly_symbol2scm ("$defaultlayout");

		parser->lexer_->set_identifier (id, yyvsp[0]);
	}
#line 3739 "./out/parser.cc"
    break;

  case 22: /* lookup: LOOKUP_IDENTIFIER '.' symbol_list_rev  */
#line 525 "parser.yy"
        {
		yyval = loc_on_copy (parser, (yyloc),
				  nested_property (yyvsp[-2], scm_reverse_x (yyvsp[0], SCM_EOL)));
	}
#line 3748 "./out/parser.cc"
    break;

  case 24: /* lookup: MODULE_IDENTIFIER '.' symbol_list_rev  */
#line 531 "parser.yy"
        {
		SCM val = yyvsp[-2];
		SCM path = scm_reverse_x (yyvsp[0], SCM_EOL);
		// Look up modules until we find a non-module or reach the end
		// of the path.
		while (true)
		{
			SCM var = scm_module_variable (val, scm_car (path));
			if (scm_is_false (var))
			{
				parser->parser_error ((yyloc), _("not found"));
				val = SCM_UNDEFINED;
				break;
			}
			val = scm_variable_ref (var);
			path = scm_cdr (path);
			if (scm_is_null (path))
			{
				// Return the found value, whatever it is.
				break;
			}
			else if (scm_is_false (Guile_user::module_p (val)))
			{
				// We have reached the end of the modules but
				// not the end of the path.  Drop into the alist
				// implementation.
				val = nested_property (val, path);
				break;
			}
		}
		yyval = loc_on_copy (parser, (yyloc), val);
	}
#line 3785 "./out/parser.cc"
    break;

  case 25: /* embedded_scm_bare: SCM_TOKEN  */
#line 567 "parser.yy"
        {
		yyval = parser->lexer_->eval_scm_token (yyvsp[0], (yylsp[0]));
	}
#line 3793 "./out/parser.cc"
    break;

  case 31: /* embedded_scm_bare_arg: SCM_TOKEN  */
#line 582 "parser.yy"
        {
		yyval = parser->lexer_->eval_scm_token (yyvsp[0], (yylsp[0]));
	}
#line 3801 "./out/parser.cc"
    break;

  case 49: /* scm_function_call: SCM_FUNCTION function_arglist  */
#line 613 "parser.yy"
                                      {
		yyval = MAKE_SYNTAX (music_function, (yyloc),
				  yyvsp[-1], yyvsp[0]);
	}
#line 3810 "./out/parser.cc"
    break;

  case 50: /* embedded_lilypond_number: '-' embedded_lilypond_number  */
#line 621 "parser.yy"
        {
		yyval = scm_difference (yyvsp[0], SCM_UNDEFINED);
	}
#line 3818 "./out/parser.cc"
    break;

  case 52: /* embedded_lilypond_number: UNSIGNED NUMBER_IDENTIFIER  */
#line 626 "parser.yy"
        {
		yyval = scm_product (yyvsp[-1], yyvsp[0]);
	}
#line 3826 "./out/parser.cc"
    break;

  case 53: /* embedded_lilypond: %empty  */
#line 633 "parser.yy"
        {
		// FIXME: @$ does not contain a useful source location
		// for empty rules, and the only token in the whole
		// production, EMBEDDED_LILY, is synthetic and also
		// contains no source location.
		yyval = MAKE_SYNTAX (unspecified_music, (yyloc));
	}
#line 3838 "./out/parser.cc"
    break;

  case 56: /* embedded_lilypond: post_event  */
#line 643 "parser.yy"
        {
		if (!unsmob<Music> (yyvsp[0]))
			yyval = MY_MAKE_MUSIC ("PostEvents", (yyloc))->unprotect ();
	}
#line 3847 "./out/parser.cc"
    break;

  case 57: /* embedded_lilypond: duration post_events  */
#line 648 "parser.yy"
        {
		if (scm_is_pair (yyvsp[0])) {
			Music *n = MY_MAKE_MUSIC ("NoteEvent", (yyloc));

			parser->default_duration_ = *unsmob<Duration> (yyvsp[-1]);
			set_property (n, "duration", yyvsp[-1]);
			set_property (n, "articulations",
					 scm_reverse_x (yyvsp[0], SCM_EOL));
			yyval = n->unprotect ();
		}
	}
#line 3863 "./out/parser.cc"
    break;

  case 58: /* embedded_lilypond: music_embedded music_embedded music_list  */
#line 659 "parser.yy"
                                                   {
		SCM tail = SCM_EOL;
		if (unsmob<Music> (yyvsp[-2]))
			tail = scm_cons (yyvsp[-2], tail);
		if (unsmob<Music> (yyvsp[-1]))
			tail = scm_cons (yyvsp[-1], tail);
		yyval = reverse_music_list (parser, (yyloc),
					 scm_append_x (ly_list (yyvsp[0], tail)),
					 true, true);
		if (scm_is_pair (yyval)) // unpackaged list
			if (scm_is_null (scm_cdr (yyval)))
				yyval = scm_car (yyval); // single expression
			else
				yyval = MAKE_SYNTAX (sequential_music, (yyloc), yyval);
		else if (scm_is_null (yyval))
			yyval = MAKE_SYNTAX (unspecified_music, (yyloc));
		// else already packaged post-event
	}
#line 3886 "./out/parser.cc"
    break;

  case 59: /* embedded_lilypond: error  */
#line 677 "parser.yy"
                {
		parser->error_level_ = 1;
                yyval = SCM_UNSPECIFIED;
	}
#line 3895 "./out/parser.cc"
    break;

  case 60: /* embedded_lilypond: "\\version-error" embedded_lilypond  */
#line 681 "parser.yy"
                                    {
		parser->error_level_ = 1;
                yyval = yyvsp[0];
	}
#line 3904 "./out/parser.cc"
    break;

  case 61: /* lilypond_header_body: %empty  */
#line 689 "parser.yy"
                    { yyval = SCM_UNSPECIFIED; }
#line 3910 "./out/parser.cc"
    break;

  case 62: /* lilypond_header_body: lilypond_header_body assignment  */
#line 690 "parser.yy"
                                           {

	}
#line 3918 "./out/parser.cc"
    break;

  case 63: /* lilypond_header_body: lilypond_header_body SCM_TOKEN  */
#line 693 "parser.yy"
                                         {
		// Evaluate and ignore #xxx, as opposed to \xxx
		parser->lexer_->eval_scm_token (yyvsp[0], (yylsp[0]));
	}
#line 3927 "./out/parser.cc"
    break;

  case 64: /* lilypond_header_body: lilypond_header_body embedded_scm_active  */
#line 697 "parser.yy"
                                                   {
		if (ly_is_module (yyvsp[0]))
			ly_module_copy (scm_current_module (), yyvsp[0]);
		else if (!scm_is_eq (yyvsp[0], SCM_UNSPECIFIED))
			parser->parser_error ((yylsp[0]), _("bad expression type"));
	}
#line 3938 "./out/parser.cc"
    break;

  case 65: /* lilypond_header: "\\header" '{' lilypond_header_body '}'  */
#line 706 "parser.yy"
                                                {
		yyval = parser->lexer_->remove_scope ();
	}
#line 3946 "./out/parser.cc"
    break;

  case 66: /* $@2: %empty  */
#line 714 "parser.yy"
        {
		parser->lexer_->add_scope (get_header (parser));
	}
#line 3954 "./out/parser.cc"
    break;

  case 67: /* header_block: $@2 lilypond_header  */
#line 716 "parser.yy"
                          {
		yyval = yyvsp[0];
	}
#line 3962 "./out/parser.cc"
    break;

  case 68: /* $@3: %empty  */
#line 724 "parser.yy"
        {
		parser->lexer_->add_scope (ly_make_module ());
	}
#line 3970 "./out/parser.cc"
    break;

  case 69: /* header_modification: $@3 lilypond_header  */
#line 726 "parser.yy"
                          {
		yyval = yyvsp[0];
	}
#line 3978 "./out/parser.cc"
    break;

  case 70: /* assignment_id: STRING  */
#line 736 "parser.yy"
        {
		yyval = scm_string_to_symbol (yyvsp[0]);
	}
#line 3986 "./out/parser.cc"
    break;

  case 71: /* assignment_id: SYMBOL  */
#line 740 "parser.yy"
        {
		yyval = scm_string_to_symbol (yyvsp[0]);
	}
#line 3994 "./out/parser.cc"
    break;

  case 72: /* assignment: assignment_id '=' identifier_init  */
#line 746 "parser.yy"
                                           {
	        parser->lexer_->set_identifier (yyvsp[-2], yyvsp[0]);
                yyval = SCM_UNSPECIFIED;
	}
#line 4003 "./out/parser.cc"
    break;

  case 73: /* assignment: assignment_id '.' property_path '=' identifier_init  */
#line 750 "parser.yy"
                                                              {
		SCM path = scm_cons (yyvsp[-4], yyvsp[-2]);
		parser->lexer_->set_identifier (path, yyvsp[0]);
                yyval = SCM_UNSPECIFIED;
	}
#line 4013 "./out/parser.cc"
    break;

  case 74: /* assignment: markup_mode_word '=' identifier_init  */
#line 756 "parser.yy"
        {
		if (scm_is_false (Lily::markup_function_p (yyvsp[0])))
		{
			parser->parser_error ((yylsp[0]), _ ("Not a markup function"));
		} else {
			Lily::define_markup_command_internal
 				(scm_string_to_symbol (yyvsp[-2]), yyvsp[0], SCM_BOOL_F);
		}
		yyval = SCM_UNSPECIFIED;
	}
#line 4028 "./out/parser.cc"
    break;

  case 77: /* identifier_init: symbol_list_part_bare '.' property_path  */
#line 773 "parser.yy"
        {
		yyval = scm_reverse_x (yyvsp[-2], yyvsp[0]);
	}
#line 4036 "./out/parser.cc"
    break;

  case 78: /* identifier_init: symbol_list_part_bare ',' property_path  */
#line 777 "parser.yy"
        {
		yyval = scm_reverse_x (yyvsp[-2], yyvsp[0]);
	}
#line 4044 "./out/parser.cc"
    break;

  case 79: /* identifier_init: post_event_nofinger post_events  */
#line 781 "parser.yy"
        {
		yyval = post_event_cons (yyvsp[-1], scm_reverse_x (yyvsp[0], SCM_EOL));
		if (scm_is_pair (yyval)
		    && scm_is_null (scm_cdr (yyval)))
			yyval = scm_car (yyval);
		else
		{
			Music * m = MY_MAKE_MUSIC ("PostEvents", (yyloc));
			set_property (m, "elements", yyval);
			yyval = m->unprotect ();
		}
	}
#line 4061 "./out/parser.cc"
    break;

  case 94: /* identifier_init_nonumber: partial_function "\\etc"  */
#line 811 "parser.yy"
        {
		yyval = MAKE_SYNTAX (partial_music_function, (yyloc),
				  scm_reverse_x (yyvsp[-1], SCM_EOL));
	}
#line 4070 "./out/parser.cc"
    break;

  case 95: /* partial_function_scriptable: MUSIC_FUNCTION function_arglist_partial  */
#line 821 "parser.yy"
        {
		yyval = scm_acons (yyvsp[-1], yyvsp[0], SCM_EOL);
	}
#line 4078 "./out/parser.cc"
    break;

  case 96: /* partial_function_scriptable: EVENT_FUNCTION function_arglist_partial  */
#line 825 "parser.yy"
        {
		yyval = scm_acons (yyvsp[-1], yyvsp[0], SCM_EOL);
	}
#line 4086 "./out/parser.cc"
    break;

  case 97: /* partial_function_scriptable: SCM_FUNCTION function_arglist_partial  */
#line 829 "parser.yy"
        {
		yyval = scm_acons (yyvsp[-1], yyvsp[0], SCM_EOL);
	}
#line 4094 "./out/parser.cc"
    break;

  case 98: /* partial_function_scriptable: MUSIC_FUNCTION "scheme?" function_arglist_optional partial_function  */
#line 833 "parser.yy"
        {
		yyval = scm_acons (yyvsp[-3], yyvsp[-1], yyvsp[0]);
	}
#line 4102 "./out/parser.cc"
    break;

  case 99: /* partial_function_scriptable: EVENT_FUNCTION "scheme?" function_arglist_optional partial_function  */
#line 837 "parser.yy"
        {
		yyval = scm_acons (yyvsp[-3], yyvsp[-1], yyvsp[0]);
	}
#line 4110 "./out/parser.cc"
    break;

  case 100: /* partial_function_scriptable: SCM_FUNCTION "scheme?" function_arglist_optional partial_function  */
#line 841 "parser.yy"
        {
		yyval = scm_acons (yyvsp[-3], yyvsp[-1], yyvsp[0]);
	}
#line 4118 "./out/parser.cc"
    break;

  case 101: /* partial_function_scriptable: MUSIC_FUNCTION "optional?" "scheme?" function_arglist_nonbackup partial_function  */
#line 845 "parser.yy"
        {
		yyval = scm_acons (yyvsp[-4], yyvsp[-1], yyvsp[0]);
	}
#line 4126 "./out/parser.cc"
    break;

  case 102: /* partial_function_scriptable: EVENT_FUNCTION "optional?" "scheme?" function_arglist_nonbackup partial_function  */
#line 849 "parser.yy"
        {
		yyval = scm_acons (yyvsp[-4], yyvsp[-1], yyvsp[0]);
	}
#line 4134 "./out/parser.cc"
    break;

  case 103: /* partial_function_scriptable: SCM_FUNCTION "optional?" "scheme?" function_arglist_nonbackup partial_function  */
#line 853 "parser.yy"
        {
		yyval = scm_acons (yyvsp[-4], yyvsp[-1], yyvsp[0]);
	}
#line 4142 "./out/parser.cc"
    break;

  case 105: /* partial_function: "\\override" grob_prop_path '='  */
#line 861 "parser.yy"
        {
		if (SCM_UNBNDP (yyvsp[-1]))
			yyval = ly_list (SCM_BOOL_F);
		else
			yyval = scm_cons
				(ly_list (Syntax::property_override,
			                  scm_cdr (yyvsp[-1]), scm_car (yyvsp[-1])),
				 SCM_EOL);
	}
#line 4156 "./out/parser.cc"
    break;

  case 106: /* partial_function: "\\set" context_prop_spec '='  */
#line 871 "parser.yy"
        {
		if (SCM_UNBNDP (yyvsp[-1]))
			yyval = ly_list (SCM_BOOL_F);
		else
			yyval = scm_cons
				(ly_list (Syntax::property_set,
				          scm_cadr (yyvsp[-1]), scm_car (yyvsp[-1])),
				 SCM_EOL);
	}
#line 4170 "./out/parser.cc"
    break;

  case 107: /* partial_function: "\\override" grob_prop_path '=' partial_function  */
#line 881 "parser.yy"
        {
		if (SCM_UNBNDP (yyvsp[-2]))
			yyval = ly_list (SCM_BOOL_F);
		else
			yyval = scm_cons
				(ly_list (Syntax::property_override,
			       	          scm_cdr (yyvsp[-2]), scm_car (yyvsp[-2])),
				 yyvsp[0]);
	}
#line 4184 "./out/parser.cc"
    break;

  case 108: /* partial_function: "\\set" context_prop_spec '=' partial_function  */
#line 891 "parser.yy"
        {
		if (SCM_UNBNDP (yyvsp[-2]))
			yyval = ly_list (SCM_BOOL_F);
		else
			yyval = scm_cons
				(ly_list (Syntax::property_set,
					  scm_cadr (yyvsp[-2]), scm_car (yyvsp[-2])),
				 yyvsp[0]);
	}
#line 4198 "./out/parser.cc"
    break;

  case 109: /* partial_function: "\\repeat" simple_string unsigned_integer  */
#line 901 "parser.yy"
        {
		yyval = scm_cons (ly_list (Syntax::repeat, yyvsp[0], yyvsp[-1]), SCM_EOL);
	}
#line 4206 "./out/parser.cc"
    break;

  case 110: /* partial_function: "\\repeat" simple_string unsigned_integer partial_function  */
#line 905 "parser.yy"
        {
		yyval = scm_cons (ly_list (Syntax::repeat, yyvsp[-1], yyvsp[-2]), yyvsp[0]);
	}
#line 4214 "./out/parser.cc"
    break;

  case 111: /* partial_function: "\\repeat" simple_string  */
#line 909 "parser.yy"
        {
		yyval = scm_cons (ly_list (Syntax::repeat, yyvsp[0]), SCM_EOL);
	}
#line 4222 "./out/parser.cc"
    break;

  case 112: /* partial_function: "\\repeat" simple_string partial_function  */
#line 913 "parser.yy"
        {
		yyval = scm_cons (ly_list (Syntax::repeat, yyvsp[-1]), yyvsp[0]);
	}
#line 4230 "./out/parser.cc"
    break;

  case 113: /* partial_function: script_dir markup_mode markup_partial_function  */
#line 918 "parser.yy"
        {
		if (SCM_UNBNDP (yyvsp[-2]))
			yyvsp[-2] = SCM_INUM0;
		yyvsp[0] = MAKE_SYNTAX (partial_markup, (yylsp[0]), yyvsp[0]);
		parser->lexer_->pop_state ();
// This relies on partial_function always being followed by ETC
		yyval = ly_list (ly_list (MAKE_SYNTAX (partial_text_script, (yyloc), yyvsp[0]),
				       yyvsp[0], yyvsp[-2]));
	}
#line 4244 "./out/parser.cc"
    break;

  case 114: /* partial_function: script_dir partial_function_scriptable  */
#line 928 "parser.yy"
        {
		if (SCM_UNBNDP (yyvsp[-1]))
			yyvsp[-1] = SCM_INUM0;
		yyval = scm_acons (Syntax::create_script_function, ly_list (yyvsp[-1]), yyvsp[0]);
	}
#line 4254 "./out/parser.cc"
    break;

  case 115: /* partial_function: script_dir  */
#line 934 "parser.yy"
        {
		if (SCM_UNBNDP (yyvsp[0]))
			yyvsp[0] = SCM_INUM0;
		yyval = scm_acons (Syntax::create_script_function, ly_list (yyvsp[0]), SCM_EOL);
	}
#line 4264 "./out/parser.cc"
    break;

  case 116: /* context_def_spec_block: "\\context" '{' context_def_spec_body '}'  */
#line 943 "parser.yy"
        {
		yyval = yyvsp[-1];
		Context_def *td = unsmob<Context_def> (yyval);
		if (!td) {
			yyval = Context_def::make_scm ();
			td = unsmob<Context_def> (yyval);
		}
		td->origin ()->set_spot ((yyloc));
	}
#line 4278 "./out/parser.cc"
    break;

  case 118: /* $@4: %empty  */
#line 957 "parser.yy"
        {
		parser->lexer_->push_note_state ();
	}
#line 4286 "./out/parser.cc"
    break;

  case 119: /* context_mod_arg: $@4 composite_music  */
#line 961 "parser.yy"
        {
		parser->lexer_->pop_state ();
		yyval = yyvsp[0];
	}
#line 4295 "./out/parser.cc"
    break;

  case 120: /* context_def_spec_body: %empty  */
#line 969 "parser.yy"
             {
		yyval = SCM_UNSPECIFIED;
	}
#line 4303 "./out/parser.cc"
    break;

  case 121: /* context_def_spec_body: context_def_spec_body context_mod  */
#line 972 "parser.yy"
                                            {
		if (!SCM_UNBNDP (yyvsp[0])) {
			Context_def *td = unsmob<Context_def> (yyval);
			if (!td) {
				yyval = Context_def::make_scm ();
				td = unsmob<Context_def> (yyval);
			}
			unsmob<Context_def> (yyval)->add_context_mod (yyvsp[0]);
		}
	}
#line 4318 "./out/parser.cc"
    break;

  case 122: /* context_def_spec_body: context_def_spec_body context_modification  */
#line 982 "parser.yy"
                                                     {
                Context_def *td = unsmob<Context_def> (yyval);
		if (!td) {
			yyval = Context_def::make_scm ();
			td = unsmob<Context_def> (yyval);
		}
                SCM new_mods = unsmob<Context_mod> (yyvsp[0])->get_mods ();
                for (SCM m = new_mods; scm_is_pair (m); m = scm_cdr (m)) {
                    td->add_context_mod (scm_car (m));
                }
	}
#line 4334 "./out/parser.cc"
    break;

  case 123: /* context_def_spec_body: context_def_spec_body context_mod_arg  */
#line 993 "parser.yy"
                                                {
		Context_def *td = unsmob<Context_def> (yyvsp[-1]);
		if (scm_is_eq (yyvsp[0], SCM_UNSPECIFIED))
			;
		else if (!td && unsmob<Context_def> (yyvsp[0]))
			yyval = yyvsp[0];
		else {
			if (!td) {
				yyval = Context_def::make_scm ();
				td = unsmob<Context_def> (yyval);
			}
			if (unsmob<Music> (yyvsp[0])) {
				SCM proc = parser->lexer_->lookup_identifier ("context-mod-music-handler");
				yyvsp[0] = ly_call (proc, yyvsp[0]);
			}
			if (Context_mod *cm = unsmob<Context_mod> (yyvsp[0])) {
				for (SCM m = cm->get_mods (); scm_is_pair (m); m = scm_cdr (m)) {
					td->add_context_mod (scm_car (m));
				}
			} else
				parser->parser_error ((yylsp[0]), _ ("not a context mod"));
		}
	}
#line 4362 "./out/parser.cc"
    break;

  case 124: /* book_block: "\\book" '{' book_body '}'  */
#line 1021 "parser.yy"
                                {
		yyval = yyvsp[-1];
		unsmob<Book> (yyval)->origin ()->set_spot ((yyloc));
		pop_paper (parser);
		parser->lexer_->set_identifier (ly_symbol2scm ("$current-book"), SCM_BOOL_F);
	}
#line 4373 "./out/parser.cc"
    break;

  case 125: /* book_body: %empty  */
#line 1033 "parser.yy"
        {
		Book *book = new Book;
		init_papers (parser);
		book->paper_ = unsmob<Output_def> (parser->lexer_->lookup_identifier ("$defaultpaper"))->clone ();
		book->paper_->unprotect ();
		push_paper (parser, book->paper_);
		book->header_ = get_header (parser);
                yyval = book->unprotect ();
		parser->lexer_->set_identifier (ly_symbol2scm ("$current-book"), yyval);
	}
#line 4388 "./out/parser.cc"
    break;

  case 126: /* book_body: BOOK_IDENTIFIER  */
#line 1043 "parser.yy"
                          {
		parser->lexer_->set_identifier (ly_symbol2scm ("$current-book"), yyvsp[0]);
	}
#line 4396 "./out/parser.cc"
    break;

  case 127: /* book_body: book_body paper_block  */
#line 1046 "parser.yy"
                                {
		unsmob<Book> (yyvsp[-1])->paper_ = unsmob<Output_def> (yyvsp[0]);
		set_paper (parser, unsmob<Output_def> (yyvsp[0]));
	}
#line 4405 "./out/parser.cc"
    break;

  case 128: /* book_body: book_body bookpart_block  */
#line 1050 "parser.yy"
                                   {
		SCM proc = parser->lexer_->lookup_identifier ("book-bookpart-handler");
		ly_call (proc, yyvsp[-1], yyvsp[0]);
	}
#line 4414 "./out/parser.cc"
    break;

  case 129: /* book_body: book_body score_block  */
#line 1054 "parser.yy"
                                {
		SCM proc = parser->lexer_->lookup_identifier ("book-score-handler");
		ly_call (proc, yyvsp[-1], yyvsp[0]);
	}
#line 4423 "./out/parser.cc"
    break;

  case 130: /* book_body: book_body composite_music  */
#line 1058 "parser.yy"
                                    {
		SCM proc = parser->lexer_->lookup_identifier ("book-music-handler");
		ly_call (proc, yyvsp[-1], yyvsp[0]);
	}
#line 4432 "./out/parser.cc"
    break;

  case 131: /* book_body: book_body full_markup  */
#line 1062 "parser.yy"
                                {
		SCM proc = parser->lexer_->lookup_identifier ("book-text-handler");
		ly_call (proc, yyvsp[-1], ly_list (yyvsp[0]));
	}
#line 4441 "./out/parser.cc"
    break;

  case 132: /* book_body: book_body full_markup_list  */
#line 1066 "parser.yy"
                                     {
		SCM proc = parser->lexer_->lookup_identifier ("book-text-handler");
		ly_call (proc, yyvsp[-1], yyvsp[0]);
	}
#line 4450 "./out/parser.cc"
    break;

  case 133: /* book_body: book_body SCM_TOKEN  */
#line 1070 "parser.yy"
                              {
		// Evaluate and ignore #xxx, as opposed to \xxx
		parser->lexer_->eval_scm_token (yyvsp[0], (yylsp[0]));
	}
#line 4459 "./out/parser.cc"
    break;

  case 134: /* book_body: book_body embedded_scm_active  */
#line 1075 "parser.yy"
        {
		SCM out = SCM_UNDEFINED;
		if (Text_interface::is_markup (yyvsp[0]))
			out = ly_list (yyvsp[0]);
		else if (Text_interface::is_markup_list (yyvsp[0]))
			out = yyvsp[0];
		if (scm_is_pair (out))
		{
			SCM proc = parser->lexer_->lookup_identifier ("book-text-handler");
			ly_call (proc, yyvsp[-1], out);
		} else if (unsmob<Score> (yyvsp[0]))
		{
			SCM proc = parser->lexer_->lookup_identifier ("book-score-handler");
			ly_call (proc, yyvsp[-1], yyvsp[0]);
		} else if (Output_def *od = unsmob<Output_def> (yyvsp[0])) {
			if (scm_is_eq (od->lookup_variable (ly_symbol2scm ("output-def-kind")),
				       ly_symbol2scm ("paper"))) {
				unsmob<Book> (yyvsp[-1])->paper_ = od;
				set_paper (parser, od);
			} else {
				parser->parser_error ((yylsp[0]), _ ("need \\paper for paper block"));
			}
		} else if (ly_is_module (yyvsp[0]))
		{
			ly_module_copy (unsmob<Book> (yyvsp[-1])->header_, yyvsp[0]);
		} else if (!scm_is_eq (yyvsp[0], SCM_UNSPECIFIED))
			parser->parser_error ((yylsp[0]), _("bad expression type"));
	}
#line 4492 "./out/parser.cc"
    break;

  case 135: /* $@5: %empty  */
#line 1104 "parser.yy"
        {
		parser->lexer_->add_scope (unsmob<Book> (yyvsp[0])->header_);
	}
#line 4500 "./out/parser.cc"
    break;

  case 137: /* book_body: book_body error  */
#line 1107 "parser.yy"
                          {
                Book *book = unsmob<Book> (yyvsp[-1]);
		book->paper_ = 0;
		book->scores_ = SCM_EOL;
		book->bookparts_ = SCM_EOL;
	}
#line 4511 "./out/parser.cc"
    break;

  case 138: /* bookpart_block: "\\bookpart" '{' bookpart_body '}'  */
#line 1116 "parser.yy"
                                       {
		yyval = yyvsp[-1];
		unsmob<Book> (yyval)->origin ()->set_spot ((yyloc));
		parser->lexer_->set_identifier (ly_symbol2scm ("$current-bookpart"), SCM_BOOL_F);
	}
#line 4521 "./out/parser.cc"
    break;

  case 139: /* bookpart_body: %empty  */
#line 1124 "parser.yy"
        {
		Book *book = new Book;
                yyval = book->unprotect ();
		parser->lexer_->set_identifier (ly_symbol2scm ("$current-bookpart"), yyval);
	}
#line 4531 "./out/parser.cc"
    break;

  case 140: /* bookpart_body: BOOK_IDENTIFIER  */
#line 1129 "parser.yy"
                          {
		parser->lexer_->set_identifier (ly_symbol2scm ("$current-bookpart"), yyvsp[0]);
	}
#line 4539 "./out/parser.cc"
    break;

  case 141: /* bookpart_body: bookpart_body paper_block  */
#line 1132 "parser.yy"
                                    {
		unsmob<Book> (yyval)->paper_ = unsmob<Output_def> (yyvsp[0]);
	}
#line 4547 "./out/parser.cc"
    break;

  case 142: /* bookpart_body: bookpart_body score_block  */
#line 1135 "parser.yy"
                                    {
		SCM proc = parser->lexer_->lookup_identifier ("bookpart-score-handler");
		ly_call (proc, yyvsp[-1], yyvsp[0]);
	}
#line 4556 "./out/parser.cc"
    break;

  case 143: /* bookpart_body: bookpart_body composite_music  */
#line 1139 "parser.yy"
                                        {
		SCM proc = parser->lexer_->lookup_identifier ("bookpart-music-handler");
		ly_call (proc, yyvsp[-1], yyvsp[0]);
	}
#line 4565 "./out/parser.cc"
    break;

  case 144: /* bookpart_body: bookpart_body full_markup  */
#line 1143 "parser.yy"
                                    {
		SCM proc = parser->lexer_->lookup_identifier ("bookpart-text-handler");
		ly_call (proc, yyvsp[-1], ly_list (yyvsp[0]));
	}
#line 4574 "./out/parser.cc"
    break;

  case 145: /* bookpart_body: bookpart_body full_markup_list  */
#line 1147 "parser.yy"
                                         {
		SCM proc = parser->lexer_->lookup_identifier ("bookpart-text-handler");
		ly_call (proc, yyvsp[-1], yyvsp[0]);
	}
#line 4583 "./out/parser.cc"
    break;

  case 146: /* bookpart_body: bookpart_body SCM_TOKEN  */
#line 1151 "parser.yy"
                                  {
		// Evaluate and ignore #xxx, as opposed to \xxx
		parser->lexer_->eval_scm_token (yyvsp[0], (yylsp[0]));
	}
#line 4592 "./out/parser.cc"
    break;

  case 147: /* bookpart_body: bookpart_body embedded_scm_active  */
#line 1156 "parser.yy"
        {
		SCM out = SCM_UNDEFINED;
		if (Text_interface::is_markup (yyvsp[0]))
			out = ly_list (yyvsp[0]);
		else if (Text_interface::is_markup_list (yyvsp[0]))
			out = yyvsp[0];
		if (scm_is_pair (out))
		{
			SCM proc = parser->lexer_->lookup_identifier ("bookpart-text-handler");
			ly_call (proc, yyvsp[-1], out);
		} else if (unsmob<Score> (yyvsp[0]))
		{
			SCM proc = parser->lexer_->lookup_identifier ("bookpart-score-handler");
			ly_call (proc, yyvsp[-1], yyvsp[0]);
		} else if (Output_def *od = unsmob<Output_def> (yyvsp[0])) {
			if (scm_is_eq (od->lookup_variable (ly_symbol2scm ("output-def-kind")),
				       ly_symbol2scm ("paper"))) {
				unsmob<Book> (yyvsp[-1])->paper_ = od;
			} else {
				parser->parser_error ((yylsp[0]), _ ("need \\paper for paper block"));
			}
		} else if (ly_is_module (yyvsp[0])) {
			Book *book = unsmob<Book> (yyvsp[-1]);
			if (!ly_is_module (book->header_))
				book->header_ = ly_make_module ();
			ly_module_copy (book->header_, yyvsp[0]);
		} else if (!scm_is_eq (yyvsp[0], SCM_UNSPECIFIED))
			parser->parser_error ((yylsp[0]), _("bad expression type"));
	}
#line 4626 "./out/parser.cc"
    break;

  case 148: /* $@6: %empty  */
#line 1186 "parser.yy"
        {
                Book *book = unsmob<Book> (yyvsp[0]);
		if (!ly_is_module (book->header_))
			book->header_ = ly_make_module ();
		parser->lexer_->add_scope (book->header_);
	}
#line 4637 "./out/parser.cc"
    break;

  case 150: /* bookpart_body: bookpart_body error  */
#line 1192 "parser.yy"
                              {
                Book *book = unsmob<Book> (yyvsp[-1]);
		book->paper_ = 0;
		book->scores_ = SCM_EOL;
	}
#line 4647 "./out/parser.cc"
    break;

  case 151: /* score_block: "\\score" '{' score_body '}'  */
#line 1200 "parser.yy"
                                        {
		unsmob<Score> (yyvsp[-1])->origin ()->set_spot ((yyloc));
		yyval = yyvsp[-1];
	}
#line 4656 "./out/parser.cc"
    break;

  case 152: /* score_body: score_items  */
#line 1207 "parser.yy"
                    {
		if (!unsmob<Score> (yyvsp[0])) {
			parser->parser_error ((yylsp[0]), _("Missing music in \\score"));
			yyval = (new Score)->unprotect ();
			if (scm_is_pair (yyvsp[0]) && ly_is_module (scm_car (yyvsp[0])))
			{
				unsmob<Score> (yyval)->set_header (scm_car (yyvsp[0]));
				yyvsp[0] = scm_cdr (yyvsp[0]);
			}
			for (SCM p = scm_reverse_x (yyvsp[0], SCM_EOL);
			     scm_is_pair (p); p = scm_cdr (p))
			{
				unsmob<Score> (yyval)->
					add_output_def (unsmob<Output_def> (scm_car (p)));
			}
		}
	}
#line 4678 "./out/parser.cc"
    break;

  case 153: /* score_body: score_body error  */
#line 1224 "parser.yy"
                           {
		unsmob<Score> (yyval)->error_found_ = true;
	}
#line 4686 "./out/parser.cc"
    break;

  case 157: /* score_items: %empty  */
#line 1237 "parser.yy"
        {
		yyval = SCM_EOL;
	}
#line 4694 "./out/parser.cc"
    break;

  case 158: /* score_items: score_items score_item  */
#line 1241 "parser.yy"
        {
		Output_def *od = unsmob<Output_def> (yyvsp[0]);
		if (od) {
			if (scm_is_eq (od->lookup_variable (ly_symbol2scm ("output-def-kind")),
				       ly_symbol2scm ("paper")))
			{
				parser->parser_error ((yylsp[0]), _("\\paper cannot be used in \\score, use \\layout instead"));
				od = 0;
				yyvsp[0] = SCM_UNSPECIFIED;
			}
		} else if (!unsmob<Score> (yyval)) {
			if (unsmob<Music> (yyvsp[0])) {
				yyvsp[0] = Lily::scorify_music (yyvsp[0]);
			}
			if (unsmob<Score> (yyvsp[0]))
			{
				yyval = yyvsp[0];
				yyvsp[0] = SCM_UNSPECIFIED;
			}
		}
		Score *score = unsmob<Score> (yyval);
		if (score && scm_is_pair (yyvsp[-1])) {
			if (ly_is_module (scm_car (yyvsp[-1])))
			{
				score->set_header (scm_car (yyvsp[-1]));
				yyvsp[-1] = scm_cdr (yyvsp[-1]);
			}
			for (SCM p = scm_reverse_x (yyvsp[-1], SCM_EOL);
			     scm_is_pair (p); p = scm_cdr (p))
			{
				score->add_output_def (unsmob<Output_def> (scm_car (p)));
			}
		}
		if (od) {
			if (score)
				score->add_output_def (od);
			else if (scm_is_pair (yyval) && ly_is_module (scm_car (yyval)))
				scm_set_cdr_x (yyval, scm_cons (yyvsp[0], scm_cdr (yyval)));
			else
				yyval = scm_cons (yyvsp[0], yyval);
		} else if (ly_is_module (yyvsp[0])) {
			SCM module = SCM_UNSPECIFIED;
			if (score) {
				module = score->get_header ();
				if (!ly_is_module (module))
				{
					module = ly_make_module ();
					score->set_header (module);
				}
			} else if (scm_is_pair (yyval) && ly_is_module (scm_car (yyval)))
				module = scm_car (yyval);
			else {
				module = ly_make_module ();
				yyval = scm_cons (module, yyval);
			}
			ly_module_copy (module, yyvsp[0]);
		} else if (!scm_is_eq (yyvsp[0], SCM_UNSPECIFIED))
			parser->parser_error ((yylsp[0]), _("Spurious expression in \\score"));
	}
#line 4758 "./out/parser.cc"
    break;

  case 159: /* $@7: %empty  */
#line 1301 "parser.yy"
        {
		if (Score *score = unsmob<Score> (yyvsp[0])) {
			if (!ly_is_module (score->get_header ()))
				score->set_header (ly_make_module ());
			parser->lexer_->add_scope (score->get_header ());
		} else {
			if (!scm_is_pair (yyvsp[0]) || !ly_is_module (scm_car (yyvsp[0])))
				yyvsp[0] = scm_cons (ly_make_module (), yyvsp[0]);
			parser->lexer_->add_scope (scm_car (yyvsp[0]));
		}
	}
#line 4774 "./out/parser.cc"
    break;

  case 160: /* score_items: score_items $@7 lilypond_header  */
#line 1312 "parser.yy"
        {
		yyval = yyvsp[-2];
	}
#line 4782 "./out/parser.cc"
    break;

  case 161: /* paper_block: output_def  */
#line 1323 "parser.yy"
                   {
                Output_def *od = unsmob<Output_def> (yyvsp[0]);

		if (!scm_is_eq (od->lookup_variable (ly_symbol2scm ("output-def-kind")),
				ly_symbol2scm ("paper")))
		{
			parser->parser_error ((yylsp[0]), _ ("need \\paper for paper block"));
			yyval = get_paper (parser)->unprotect ();
		}
	}
#line 4797 "./out/parser.cc"
    break;

  case 162: /* output_def: output_def_body '}'  */
#line 1337 "parser.yy"
                            {
		if (scm_is_pair (yyvsp[-1]))
			yyval = scm_car (yyvsp[-1]);

		parser->lexer_->remove_scope ();
		parser->lexer_->pop_state ();
	}
#line 4809 "./out/parser.cc"
    break;

  case 163: /* output_def_head: "\\paper"  */
#line 1347 "parser.yy"
              {
                Output_def *p = get_paper (parser);
		p->input_origin_ = (yyloc);
		parser->lexer_->add_scope (p->scope_);
                yyval = p->unprotect ();
	}
#line 4820 "./out/parser.cc"
    break;

  case 164: /* output_def_head: "\\midi"  */
#line 1353 "parser.yy"
                  {
		Output_def *p = get_midi (parser);
		yyval = p->unprotect ();
		parser->lexer_->add_scope (p->scope_);
	}
#line 4830 "./out/parser.cc"
    break;

  case 165: /* output_def_head: "\\layout"  */
#line 1358 "parser.yy"
                        {
		Output_def *p = get_layout (parser);

		parser->lexer_->add_scope (p->scope_);
		yyval = p->unprotect ();
	}
#line 4841 "./out/parser.cc"
    break;

  case 166: /* output_def_head_with_mode_switch: output_def_head  */
#line 1367 "parser.yy"
                        {
		parser->lexer_->push_initial_state ();
		yyval = yyvsp[0];
	}
#line 4850 "./out/parser.cc"
    break;

  case 169: /* output_def_body: output_def_head_with_mode_switch '{'  */
#line 1384 "parser.yy"
                                             {
		unsmob<Output_def> (yyvsp[-1])->input_origin_.set_spot ((yyloc));
		// This is a stupid trick to mark the beginning of the
		// body for deciding whether to allow
		// embedded_scm_active to have an output definition
		yyval = ly_list (yyvsp[-1]);
	}
#line 4862 "./out/parser.cc"
    break;

  case 170: /* output_def_body: output_def_body assignment  */
#line 1391 "parser.yy"
                                      {
		if (scm_is_pair (yyvsp[-1]))
			yyval = scm_car (yyvsp[-1]);
	}
#line 4871 "./out/parser.cc"
    break;

  case 171: /* output_def_body: output_def_body embedded_scm_active  */
#line 1396 "parser.yy"
        {
		// We don't switch into note mode for Scheme functions
		// here.  Does not seem warranted/required in output
		// definitions.
		if (scm_is_pair (yyvsp[-1]))
		{
			Output_def *o = unsmob<Output_def> (yyvsp[0]);
			if (o) {
				o->input_origin_.set_spot ((yyloc));
				yyvsp[-1] = o->self_scm ();
				parser->lexer_->remove_scope ();
				parser->lexer_->add_scope (o->scope_);
				yyvsp[0] = SCM_UNSPECIFIED;
			} else
				yyvsp[-1] = scm_car (yyvsp[-1]);
		}
		if (unsmob<Context_def> (yyvsp[0]))
			assign_context_def (unsmob<Output_def> (yyvsp[-1]), yyvsp[0]);
		// Seems unlikely, but let's be complete:
		else if (unsmob<Music> (yyvsp[0]))
		{
			SCM proc = parser->lexer_->lookup_identifier ("output-def-music-handler");
			ly_call (proc, yyvsp[-1], yyvsp[0]);
		} else if (!scm_is_eq (yyvsp[0], SCM_UNSPECIFIED))
			parser->parser_error ((yylsp[0]), _("bad expression type"));
		yyval = yyvsp[-1];
	}
#line 4903 "./out/parser.cc"
    break;

  case 172: /* output_def_body: output_def_body SCM_TOKEN  */
#line 1423 "parser.yy"
                                    {
		if (scm_is_pair (yyvsp[-1]))
			yyval = scm_car (yyvsp[-1]);
		// Evaluate and ignore #xxx, as opposed to \xxx
		parser->lexer_->eval_scm_token (yyvsp[0], (yylsp[0]));
	}
#line 4914 "./out/parser.cc"
    break;

  case 173: /* $@8: %empty  */
#line 1430 "parser.yy"
        {
		if (scm_is_pair (yyvsp[0]))
			yyvsp[0] = scm_car (yyvsp[0]);
		parser->lexer_->push_note_state ();
	}
#line 4924 "./out/parser.cc"
    break;

  case 174: /* output_def_body: output_def_body $@8 music_or_context_def  */
#line 1435 "parser.yy"
        {
		parser->lexer_->pop_state ();
		if (unsmob<Context_def> (yyvsp[0]))
			assign_context_def (unsmob<Output_def> (yyvsp[-2]), yyvsp[0]);
		else {

			SCM proc = parser->lexer_->lookup_identifier ("output-def-music-handler");
			ly_call (proc, yyvsp[-2], yyvsp[0]);
		}
		yyval = yyvsp[-2];
	}
#line 4940 "./out/parser.cc"
    break;

  case 175: /* output_def_body: output_def_body error  */
#line 1446 "parser.yy"
                                {

	}
#line 4948 "./out/parser.cc"
    break;

  case 176: /* tempo_event: "\\tempo" steno_duration '=' tempo_range  */
#line 1452 "parser.yy"
                                                {
		yyval = MAKE_SYNTAX (tempo, (yyloc), SCM_EOL, yyvsp[-2], yyvsp[0]);
	}
#line 4956 "./out/parser.cc"
    break;

  case 177: /* tempo_event: "\\tempo" text steno_duration '=' tempo_range  */
#line 1455 "parser.yy"
                                                        {
		yyval = MAKE_SYNTAX (tempo, (yyloc), yyvsp[-3], yyvsp[-2], yyvsp[0]);
	}
#line 4964 "./out/parser.cc"
    break;

  case 178: /* tempo_event: "\\tempo" text  */
#line 1458 "parser.yy"
                     {
		yyval = MAKE_SYNTAX (tempo, (yyloc), yyvsp[0]);
	}
#line 4972 "./out/parser.cc"
    break;

  case 179: /* music_list: %empty  */
#line 1467 "parser.yy"
                    {
		yyval = SCM_EOL;
	}
#line 4980 "./out/parser.cc"
    break;

  case 180: /* music_list: music_list music_embedded  */
#line 1470 "parser.yy"
                                    {
		if (unsmob<Music> (yyvsp[0]))
			yyval = scm_cons (yyvsp[0], yyvsp[-1]);
	}
#line 4989 "./out/parser.cc"
    break;

  case 181: /* music_list: music_list error  */
#line 1474 "parser.yy"
                           {
		Music *m = MY_MAKE_MUSIC("Music", (yyloc));
		// ugh. code dup
		set_property (m, "error-found", SCM_BOOL_T);
		yyval = scm_cons (m->self_scm (), yyvsp[-1]);
		m->unprotect (); /* UGH */
	}
#line 5001 "./out/parser.cc"
    break;

  case 182: /* braced_music_list: '{' music_list '}'  */
#line 1485 "parser.yy"
        {
		yyval = reverse_music_list (parser, (yyloc), yyvsp[-1], true, false);
	}
#line 5009 "./out/parser.cc"
    break;

  case 186: /* pitch_as_music: pitch_or_music  */
#line 1497 "parser.yy"
        {
	        yyval = make_music_from_simple (parser, (yylsp[0]), yyvsp[0]);
                if (!unsmob<Music> (yyval))
		{
                        parser->parser_error ((yylsp[0]), _ ("music expected"));
			yyval = MAKE_SYNTAX (unspecified_music, (yyloc));
		}
	}
#line 5022 "./out/parser.cc"
    break;

  case 189: /* music_embedded: music_embedded_backup  */
#line 1511 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 5030 "./out/parser.cc"
    break;

  case 190: /* music_embedded: music_embedded_backup "(backed-up?)" lyric_element_music  */
#line 1515 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 5038 "./out/parser.cc"
    break;

  case 191: /* music_embedded: duration post_events  */
#line 1519 "parser.yy"
        {
		Music *n = MY_MAKE_MUSIC ("NoteEvent", (yyloc));

		parser->default_duration_ = *unsmob<Duration> (yyvsp[-1]);
		set_property (n, "duration", yyvsp[-1]);

		if (scm_is_pair (yyvsp[0]))
			set_property (n, "articulations",
					 scm_reverse_x (yyvsp[0], SCM_EOL));
		yyval = n->unprotect ();
	}
#line 5054 "./out/parser.cc"
    break;

  case 192: /* music_embedded_backup: embedded_scm  */
#line 1534 "parser.yy"
        {
		if (scm_is_eq (yyvsp[0], SCM_UNSPECIFIED)
		    || unsmob<Music> (yyvsp[0]))
			yyval = yyvsp[0];
		else if (parser->lexer_->is_lyric_state ()
			   && Text_interface::is_markup (yyvsp[0]))
			MYBACKUP (LYRIC_ELEMENT, yyvsp[0], (yylsp[0]));
		else {
			(yyloc).warning (_ ("Ignoring non-music expression"));
			yyval = SCM_UNSPECIFIED;
		}
	}
#line 5071 "./out/parser.cc"
    break;

  case 195: /* repeated_music: "\\repeat" simple_string unsigned_integer music  */
#line 1557 "parser.yy"
        {
		yyval = MAKE_SYNTAX (repeat, (yyloc), yyvsp[-2], yyvsp[-1], yyvsp[0]);
	}
#line 5079 "./out/parser.cc"
    break;

  case 196: /* repeated_music: "\\repeat" simple_string unsigned_integer music alternative_music  */
#line 1561 "parser.yy"
        {
		yyval = MAKE_SYNTAX (repeat_alt, (yyloc), yyvsp[-3], yyvsp[-2], yyvsp[-1], yyvsp[0]);
	}
#line 5087 "./out/parser.cc"
    break;

  case 197: /* alternative_music: "\\alternative" basic_music  */
#line 1567 "parser.yy"
                                {
		yyval = MAKE_SYNTAX (alternative, (yyloc), yyvsp[0]);
	}
#line 5095 "./out/parser.cc"
    break;

  case 198: /* sequential_music: "\\sequential" braced_music_list  */
#line 1573 "parser.yy"
                                     {
		yyval = MAKE_SYNTAX (sequential_music, (yyloc), yyvsp[0]);
	}
#line 5103 "./out/parser.cc"
    break;

  case 199: /* sequential_music: braced_music_list  */
#line 1576 "parser.yy"
                            {
		yyval = MAKE_SYNTAX (sequential_music, (yyloc), yyvsp[0]);
	}
#line 5111 "./out/parser.cc"
    break;

  case 200: /* simultaneous_music: "\\simultaneous" braced_music_list  */
#line 1582 "parser.yy"
                                       {
		yyval = MAKE_SYNTAX (simultaneous_music, (yyloc), yyvsp[0]);
	}
#line 5119 "./out/parser.cc"
    break;

  case 201: /* simultaneous_music: "<<" music_list ">>"  */
#line 1585 "parser.yy"
                                                                {
		yyval = MAKE_SYNTAX (simultaneous_music, (yyloc),
				  reverse_music_list (parser, (yyloc), yyvsp[-1],
						      true, false));
	}
#line 5129 "./out/parser.cc"
    break;

  case 205: /* $@9: %empty  */
#line 1600 "parser.yy"
        {
		parser->lexer_->push_note_state ();
	}
#line 5137 "./out/parser.cc"
    break;

  case 206: /* context_modification: "\\with" $@9 '{' context_mod_list '}'  */
#line 1603 "parser.yy"
        {
                parser->lexer_->pop_state ();
                yyval = yyvsp[-1];
        }
#line 5146 "./out/parser.cc"
    break;

  case 207: /* context_modification: "\\with" context_modification_arg  */
#line 1608 "parser.yy"
        {
		if (unsmob<Music> (yyvsp[0])) {
			SCM proc = parser->lexer_->lookup_identifier ("context-mod-music-handler");
			yyvsp[0] = ly_call (proc, yyvsp[0]);
		}
		if (unsmob<Context_mod> (yyvsp[0]))
			yyval = yyvsp[0];
		else {
			// let's permit \with #*unspecified* to go for
			// an empty context mod
			if (!scm_is_eq (yyvsp[0], SCM_UNSPECIFIED))
				parser->parser_error ((yylsp[0]), _ ("not a context mod"));
			yyval = Context_mod ().smobbed_copy ();
		}
	}
#line 5166 "./out/parser.cc"
    break;

  case 210: /* optional_context_mods: context_modification_mods_list  */
#line 1636 "parser.yy"
        {
		if (scm_is_pair (yyvsp[0]))
			yyval = scm_append_x (scm_reverse_x (yyvsp[0], SCM_EOL));
        }
#line 5175 "./out/parser.cc"
    break;

  case 211: /* context_modification_mods_list: %empty  */
#line 1649 "parser.yy"
             {
		yyval = SCM_EOL;
	}
#line 5183 "./out/parser.cc"
    break;

  case 212: /* context_modification_mods_list: context_modification_mods_list context_modification  */
#line 1653 "parser.yy"
        {
		if (Context_mod *m = unsmob<Context_mod> (yyvsp[0]))
			yyval = scm_cons (m->get_mods (), yyvsp[-1]);
	}
#line 5192 "./out/parser.cc"
    break;

  case 213: /* context_mod_list: %empty  */
#line 1665 "parser.yy"
             {
            yyval = Context_mod ().smobbed_copy ();
        }
#line 5200 "./out/parser.cc"
    break;

  case 214: /* context_mod_list: context_mod_list context_mod  */
#line 1668 "parser.yy"
                                        {
		if (!SCM_UNBNDP (yyvsp[0]))
			unsmob<Context_mod> (yyvsp[-1])->add_context_mod (yyvsp[0]);
        }
#line 5209 "./out/parser.cc"
    break;

  case 215: /* context_mod_list: context_mod_list context_mod_arg  */
#line 1672 "parser.yy"
                                           {
		if (unsmob<Music> (yyvsp[0])) {
			SCM proc = parser->lexer_->lookup_identifier ("context-mod-music-handler");
			yyvsp[0] = ly_call (proc, yyvsp[0]);
		}
		if (unsmob<Context_mod> (yyvsp[0]))
			unsmob<Context_mod> (yyval)->add_context_mods
				(unsmob<Context_mod> (yyvsp[0])->get_mods ());
		else if (!scm_is_eq (yyvsp[0], SCM_UNSPECIFIED))
			parser->parser_error ((yylsp[0]), _ ("not a context mod"));
        }
#line 5225 "./out/parser.cc"
    break;

  case 216: /* context_prefix: "\\context" symbol optional_id optional_context_mods  */
#line 1686 "parser.yy"
                                                         {
		yyval = START_MAKE_SYNTAX (context_find_or_create, yyvsp[-2], yyvsp[-1], yyvsp[0]);
	}
#line 5233 "./out/parser.cc"
    break;

  case 217: /* context_prefix: "\\new" symbol optional_id optional_context_mods  */
#line 1689 "parser.yy"
                                                              {
		yyval = START_MAKE_SYNTAX (context_create, yyvsp[-2], yyvsp[-1], yyvsp[0]);
	}
#line 5241 "./out/parser.cc"
    break;

  case 218: /* new_lyrics: "\\addlyrics" optional_context_mods lyric_mode_music  */
#line 1695 "parser.yy"
                                                         {
		yyval = scm_acons (yyvsp[0], yyvsp[-1], SCM_EOL);
	}
#line 5249 "./out/parser.cc"
    break;

  case 219: /* new_lyrics: new_lyrics "\\addlyrics" optional_context_mods lyric_mode_music  */
#line 1698 "parser.yy"
                                                                      {
		yyval = scm_acons (yyvsp[0], yyvsp[-1], yyvsp[-3]);
	}
#line 5257 "./out/parser.cc"
    break;

  case 224: /* basic_music: "\\lyricsto" simple_string lyric_mode_music  */
#line 1724 "parser.yy"
                                                  {
		yyval = MAKE_SYNTAX (lyric_combine, (yyloc), yyvsp[-1], SCM_EOL, yyvsp[0]);
	}
#line 5265 "./out/parser.cc"
    break;

  case 225: /* basic_music: "\\lyricsto" symbol '=' simple_string lyric_mode_music  */
#line 1728 "parser.yy"
        {
		yyval = MAKE_SYNTAX (lyric_combine, (yyloc), yyvsp[-1], yyvsp[-3], yyvsp[0]);
	}
#line 5273 "./out/parser.cc"
    break;

  case 229: /* contexted_basic_music: context_prefix contextable_music new_lyrics  */
#line 1741 "parser.yy"
        {
		Input i;
		i.set_location ((yylsp[-2]), (yylsp[-1]));
		yyval = FINISH_MAKE_SYNTAX (yyvsp[-2], i, yyvsp[-1]);
		yyval = MAKE_SYNTAX (add_lyrics, (yyloc), yyval, scm_reverse_x (yyvsp[0], SCM_EOL));
	}
#line 5284 "./out/parser.cc"
    break;

  case 230: /* contexted_basic_music: context_prefix contextable_music  */
#line 1748 "parser.yy"
        {
		yyval = FINISH_MAKE_SYNTAX (yyvsp[-1], (yyloc), yyvsp[0]);
	}
#line 5292 "./out/parser.cc"
    break;

  case 231: /* contexted_basic_music: context_prefix contexted_basic_music  */
#line 1752 "parser.yy"
        {
		yyval = FINISH_MAKE_SYNTAX (yyvsp[-1], (yyloc), yyvsp[0]);
	}
#line 5300 "./out/parser.cc"
    break;

  case 234: /* composite_music: basic_music new_lyrics  */
#line 1761 "parser.yy"
        {
		yyval = MAKE_SYNTAX (add_lyrics, (yyloc), yyvsp[-1], scm_reverse_x (yyvsp[0], SCM_EOL));
	}
#line 5308 "./out/parser.cc"
    break;

  case 238: /* grouped_music_list: simultaneous_music  */
#line 1773 "parser.yy"
                                        { yyval = yyvsp[0]; }
#line 5314 "./out/parser.cc"
    break;

  case 239: /* grouped_music_list: sequential_music  */
#line 1774 "parser.yy"
                                        { yyval = yyvsp[0]; }
#line 5320 "./out/parser.cc"
    break;

  case 241: /* symbol_list_arg: SYMBOL_LIST '.' symbol_list_rev  */
#line 1833 "parser.yy"
        {
		yyval = ly_append (yyvsp[-2], scm_reverse_x (yyvsp[0], SCM_EOL));
	}
#line 5328 "./out/parser.cc"
    break;

  case 242: /* symbol_list_arg: SYMBOL_LIST ',' symbol_list_rev  */
#line 1837 "parser.yy"
        {
		yyval = ly_append (yyvsp[-2], scm_reverse_x (yyvsp[0], SCM_EOL));
	}
#line 5336 "./out/parser.cc"
    break;

  case 244: /* symbol_list_rev: symbol_list_rev '.' symbol_list_part  */
#line 1845 "parser.yy"
        {
		yyval = scm_append_x (ly_list (yyvsp[0], yyvsp[-2]));
	}
#line 5344 "./out/parser.cc"
    break;

  case 245: /* symbol_list_rev: symbol_list_rev ',' symbol_list_part  */
#line 1849 "parser.yy"
        {
		yyval = scm_append_x (ly_list (yyvsp[0], yyvsp[-2]));
	}
#line 5352 "./out/parser.cc"
    break;

  case 247: /* symbol_list_part: embedded_scm_bare  */
#line 1859 "parser.yy"
        {
		yyval = make_reverse_key_list (yyvsp[0]);
		if (SCM_UNBNDP (yyval)) {
			parser->parser_error ((yylsp[0]), _("not a key"));
			yyval = SCM_EOL;
		}
	}
#line 5364 "./out/parser.cc"
    break;

  case 248: /* symbol_list_element: STRING  */
#line 1871 "parser.yy"
        {
		yyval = scm_string_to_symbol (yyvsp[0]);
	}
#line 5372 "./out/parser.cc"
    break;

  case 250: /* symbol_list_part_bare: SYMBOL  */
#line 1880 "parser.yy"
        {
		yyval = try_word_variants (Lily::key_list_p, yyvsp[0]);
		if (SCM_UNBNDP (yyval)) {
			parser->parser_error ((yylsp[0]), _("not a key"));
			yyval = SCM_EOL;
		} else
			yyval = scm_reverse (yyval);
	}
#line 5385 "./out/parser.cc"
    break;

  case 251: /* symbol_list_part_bare: symbol_list_element  */
#line 1889 "parser.yy"
        {
		yyval = ly_list (yyvsp[0]);
	}
#line 5393 "./out/parser.cc"
    break;

  case 253: /* function_arglist_nonbackup: "optional?" "scheme?" function_arglist_nonbackup post_event_nofinger  */
#line 1897 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-1], yyvsp[-2]);
	}
#line 5401 "./out/parser.cc"
    break;

  case 254: /* function_arglist_nonbackup: "optional?" "scheme?" function_arglist_nonbackup '-' UNSIGNED  */
#line 1901 "parser.yy"
        {
		SCM n = scm_difference (yyvsp[0], SCM_UNDEFINED);
		if (scm_is_true (ly_call (yyvsp[-3], n)))
			yyval = scm_cons (n, yyvsp[-2]);
		else {
			Music *t = MY_MAKE_MUSIC ("FingeringEvent", (yylsp[0]));
			set_property (t, "digit", yyvsp[0]);
			yyval = check_scheme_arg (parser, (yylsp[-1]), t->unprotect (),
					       yyvsp[-2], yyvsp[-3], n);
		}
	}
#line 5417 "./out/parser.cc"
    break;

  case 255: /* function_arglist_nonbackup: "optional?" "scheme?" function_arglist_nonbackup '-' REAL  */
#line 1913 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[-1]),
				       scm_difference (yyvsp[0], SCM_UNDEFINED),
				       yyvsp[-2], yyvsp[-3]);
	}
#line 5427 "./out/parser.cc"
    break;

  case 256: /* function_arglist_nonbackup: "optional?" "scheme?" function_arglist_nonbackup '-' NUMBER_IDENTIFIER  */
#line 1919 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[-1]),
				       scm_difference (yyvsp[0], SCM_UNDEFINED),
				       yyvsp[-2], yyvsp[-3]);
	}
#line 5437 "./out/parser.cc"
    break;

  case 257: /* function_arglist_nonbackup: "optional?" "scheme?" function_arglist_nonbackup embedded_scm_arg  */
#line 1925 "parser.yy"
        {
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
			yyval = scm_cons (yyvsp[0], yyvsp[-1]);
		else
			yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-1], yyvsp[-2]);
	}
#line 5448 "./out/parser.cc"
    break;

  case 258: /* function_arglist_nonbackup: "optional?" "scheme?" function_arglist_nonbackup bare_number_common  */
#line 1932 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-1], yyvsp[-2]);
	}
#line 5456 "./out/parser.cc"
    break;

  case 259: /* function_arglist_nonbackup: function_arglist_nonbackup_reparse "(reparsed?)" pitch_or_music  */
#line 1936 "parser.yy"
        {
		if (scm_is_true (ly_call (yyvsp[-1], yyvsp[0])))
			yyval = scm_cons (yyvsp[0], yyvsp[-2]);
		else
			yyval = check_scheme_arg (parser, (yylsp[0]),
					       make_music_from_simple
					       (parser, (yylsp[0]), yyvsp[0]),
					       yyvsp[-2], yyvsp[-1], yyvsp[0]);
	}
#line 5470 "./out/parser.cc"
    break;

  case 260: /* function_arglist_nonbackup: function_arglist_nonbackup_reparse "(reparsed?)" duration  */
#line 1946 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 5478 "./out/parser.cc"
    break;

  case 261: /* function_arglist_nonbackup: function_arglist_nonbackup_reparse "(reparsed?)" reparsed_rhythm  */
#line 1950 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 5486 "./out/parser.cc"
    break;

  case 262: /* function_arglist_nonbackup: function_arglist_nonbackup_reparse "(reparsed?)" bare_number_common  */
#line 1954 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 5494 "./out/parser.cc"
    break;

  case 263: /* function_arglist_nonbackup: function_arglist_nonbackup_reparse "(reparsed?)" SCM_ARG  */
#line 1958 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 5502 "./out/parser.cc"
    break;

  case 264: /* function_arglist_nonbackup: function_arglist_nonbackup_reparse "(reparsed?)" lyric_element_music  */
#line 1962 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 5510 "./out/parser.cc"
    break;

  case 265: /* function_arglist_nonbackup: function_arglist_nonbackup_reparse "(reparsed?)" symbol_list_arg  */
#line 1966 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 5518 "./out/parser.cc"
    break;

  case 266: /* reparsed_rhythm: DURATION_ARG dots multipliers post_events  */
#line 1974 "parser.yy"
        {
		SCM d = make_duration (yyvsp[-3], from_scm<int> (yyvsp[-2]), yyvsp[-1]);
		parser->default_duration_ = *unsmob<Duration> (d);
		yyval = make_music_from_simple (parser, (yyloc), d);
		Music *m = unsmob<Music> (yyval);
		assert (m);
		if (scm_is_pair (yyvsp[0]))
			set_property (m, "articulations",
					 scm_reverse_x (yyvsp[0], SCM_EOL));
	}
#line 5533 "./out/parser.cc"
    break;

  case 267: /* function_arglist_nonbackup_reparse: "optional?" "scheme?" function_arglist_nonbackup SCM_IDENTIFIER  */
#line 1988 "parser.yy"
        {
		yyval = yyvsp[-1];
		SCM res = try_string_variants (yyvsp[-2], yyvsp[0]);
		if (!SCM_UNBNDP (res))
			if (scm_is_pair (res))
				MYREPARSE ((yylsp[0]), yyvsp[-2], SYMBOL_LIST, res);
			else
				MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, res);
		else if (scm_is_true
			 (ly_call
			  (yyvsp[-2], make_music_from_simple
			   (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], STRING, yyvsp[0]);
		else
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]);
	}
#line 5554 "./out/parser.cc"
    break;

  case 268: /* function_arglist_nonbackup_reparse: "optional?" "scheme?" function_arglist_nonbackup pitch  */
#line 2005 "parser.yy"
        {
		yyval = yyvsp[-1];
		if (scm_is_true
		    (ly_call
		     (yyvsp[-2], make_music_from_simple
		      (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], PITCH_IDENTIFIER, yyvsp[0]);
		else
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]);
	}
#line 5569 "./out/parser.cc"
    break;

  case 269: /* function_arglist_nonbackup_reparse: "optional?" "scheme?" function_arglist_nonbackup steno_tonic_pitch  */
#line 2016 "parser.yy"
        {
		yyval = yyvsp[-1];
		if (scm_is_true
		    (ly_call
		     (yyvsp[-2], make_music_from_simple
		      (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], TONICNAME_PITCH, yyvsp[0]);
		else
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]);
	}
#line 5584 "./out/parser.cc"
    break;

  case 270: /* function_arglist_nonbackup_reparse: "optional?" "scheme?" function_arglist_nonbackup STRING  */
#line 2027 "parser.yy"
        {
		yyval = yyvsp[-1];
		SCM res = try_string_variants (yyvsp[-2], yyvsp[0]);
		if (!SCM_UNBNDP (res))
			if (scm_is_pair (res))
				MYREPARSE ((yylsp[0]), yyvsp[-2], SYMBOL_LIST, res);
			else
				MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, res);
		else if (scm_is_true
			 (ly_call
			  (yyvsp[-2], make_music_from_simple
			   (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], STRING, yyvsp[0]);
		else
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]);
	}
#line 5605 "./out/parser.cc"
    break;

  case 271: /* function_arglist_nonbackup_reparse: "optional?" "scheme?" function_arglist_nonbackup SYMBOL  */
#line 2044 "parser.yy"
        {
		yyval = yyvsp[-1];
		SCM res = try_word_variants (yyvsp[-2], yyvsp[0]);
		if (!SCM_UNBNDP (res))
			if (scm_is_pair (res))
				MYREPARSE ((yylsp[0]), yyvsp[-2], SYMBOL_LIST, res);
			else
				MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, res);
		else if (scm_is_true
			 (ly_call
			  (yyvsp[-2], make_music_from_simple
			   (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], STRING, yyvsp[0]);
		else
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]);
	}
#line 5626 "./out/parser.cc"
    break;

  case 272: /* function_arglist_nonbackup_reparse: "optional?" "scheme?" function_arglist_nonbackup full_markup  */
#line 2061 "parser.yy"
        {
		yyval = yyvsp[-1];
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]);
		else if (scm_is_true
			 (ly_call
			  (yyvsp[-2], make_music_from_simple
			   (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], STRING, yyvsp[0]);
		else
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]);
	}
#line 5643 "./out/parser.cc"
    break;

  case 273: /* function_arglist_nonbackup_reparse: "optional?" "scheme?" function_arglist_nonbackup UNSIGNED  */
#line 2074 "parser.yy"
        {
		yyval = yyvsp[-1];
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
			// May be 3 \cm or similar
			MYREPARSE ((yylsp[0]), yyvsp[-2], REAL, yyvsp[0]);
		else if (scm_is_true (ly_call (yyvsp[-2], ly_list (yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], SYMBOL_LIST, ly_list (yyvsp[0]));
		else {
			SCM d = make_duration (yyvsp[0]);
			if (!SCM_UNBNDP (d)) {
				if (scm_is_true (ly_call (yyvsp[-2], d)))
					MYREPARSE ((yylsp[0]), yyvsp[-2], DURATION_IDENTIFIER, d);
				else if (scm_is_true
					 (ly_call
					  (yyvsp[-2], make_music_from_simple (parser, (yylsp[0]), d))))
					MYREPARSE ((yylsp[0]), yyvsp[-2], DURATION_ARG, d);
				else
					MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]); // trigger error
			} else
				MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]); // trigger error
		}
	}
#line 5670 "./out/parser.cc"
    break;

  case 274: /* function_arglist_nonbackup_reparse: "optional?" "scheme?" function_arglist_nonbackup DURATION_IDENTIFIER  */
#line 2097 "parser.yy"
        {
		yyval = yyvsp[-1];
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
			MYREPARSE ((yylsp[0]), yyvsp[-2], DURATION_IDENTIFIER, yyvsp[0]);
		else if (scm_is_true
			 (ly_call
			  (yyvsp[-2], make_music_from_simple (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], DURATION_ARG, yyvsp[0]);
		else
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]); // trigger error
	}
#line 5686 "./out/parser.cc"
    break;

  case 276: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup embedded_scm_arg  */
#line 2116 "parser.yy"
        {
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
			yyval = scm_cons (yyvsp[0], yyvsp[-1]);
		else {
			yyval = scm_cons (loc_on_copy (parser, (yylsp[-1]), yyvsp[-3]), yyvsp[-1]);
			MYBACKUP (SCM_ARG, yyvsp[0], (yylsp[0]));
		}
	}
#line 5699 "./out/parser.cc"
    break;

  case 277: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup post_event_nofinger  */
#line 2125 "parser.yy"
        {
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
		{
			yyval = scm_cons (yyvsp[0], yyvsp[-1]);
		} else {
			yyval = scm_cons (loc_on_copy (parser, (yylsp[-1]), yyvsp[-3]), yyvsp[-1]);
			MYBACKUP (EVENT_IDENTIFIER, yyvsp[0], (yylsp[0]));
		}
	}
#line 5713 "./out/parser.cc"
    break;

  case 278: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup pitch  */
#line 2135 "parser.yy"
        {
		if (scm_is_true
		    (ly_call
		     (yyvsp[-2], make_music_from_simple
		      (parser, (yylsp[0]), yyvsp[0]))))
		{
			yyval = yyvsp[-1];
			MYREPARSE ((yylsp[0]), yyvsp[-2], PITCH_IDENTIFIER, yyvsp[0]);
		} else if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
			yyval = scm_cons (yyvsp[0], yyvsp[-1]);
		else {
			yyval = scm_cons (loc_on_copy (parser, (yylsp[-1]), yyvsp[-3]), yyvsp[-1]);
			MYBACKUP (PITCH_IDENTIFIER, yyvsp[0], (yylsp[0]));
		}
	}
#line 5733 "./out/parser.cc"
    break;

  case 279: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup steno_tonic_pitch  */
#line 2151 "parser.yy"
        {
		if (scm_is_true
		    (ly_call
		     (yyvsp[-2], make_music_from_simple
		      (parser, (yylsp[0]), yyvsp[0]))))
		{
			yyval = yyvsp[-1];
			MYREPARSE ((yylsp[0]), yyvsp[-2], TONICNAME_PITCH, yyvsp[0]);
		} else if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
			yyval = scm_cons (yyvsp[0], yyvsp[-1]);
		else {
			yyval = scm_cons (loc_on_copy (parser, (yylsp[-1]), yyvsp[-3]), yyvsp[-1]);
			MYBACKUP (TONICNAME_PITCH, yyvsp[0], (yylsp[0]));
		}
	}
#line 5753 "./out/parser.cc"
    break;

  case 280: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup full_markup  */
#line 2167 "parser.yy"
        {
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
			yyval = scm_cons (yyvsp[0], yyvsp[-1]);
		else {
			yyval = scm_cons (loc_on_copy (parser, (yylsp[-1]), yyvsp[-3]), yyvsp[-1]);
			MYBACKUP (SCM_IDENTIFIER, yyvsp[0], (yylsp[0]));
		}
	}
#line 5766 "./out/parser.cc"
    break;

  case 281: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup UNSIGNED  */
#line 2176 "parser.yy"
        {
		yyval = yyvsp[-1];
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
			// May be 3 \cm or similar
			MYREPARSE ((yylsp[0]), yyvsp[-2], REAL, yyvsp[0]);
		else if (scm_is_true (ly_call (yyvsp[-2], ly_list (yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], SYMBOL_LIST, ly_list (yyvsp[0]));
		else {
			SCM d = make_duration (yyvsp[0]);
			if (!SCM_UNBNDP (d)) {
				if (scm_is_true (ly_call (yyvsp[-2], d)))
					MYREPARSE ((yylsp[0]), yyvsp[-2], DURATION_IDENTIFIER, d);
				else if (scm_is_true
					 (ly_call
					  (yyvsp[-2], make_music_from_simple (parser, (yylsp[0]), d))))
					MYREPARSE ((yylsp[0]), yyvsp[-2], DURATION_ARG, d);
				else {
					yyval = scm_cons (loc_on_copy (parser, (yylsp[-1]), yyvsp[-3]), yyvsp[-1]);
					MYBACKUP (UNSIGNED, yyvsp[0], (yylsp[0]));
				}
			} else {
				yyval = scm_cons (loc_on_copy (parser, (yylsp[-1]), yyvsp[-3]), yyvsp[-1]);
				MYBACKUP (UNSIGNED, yyvsp[0], (yylsp[0]));
			}
		}
	}
#line 5797 "./out/parser.cc"
    break;

  case 282: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup REAL  */
#line 2203 "parser.yy"
        {
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
		{
			yyval = yyvsp[-1];
			MYREPARSE ((yylsp[0]), yyvsp[-2], REAL, yyvsp[0]);
		} else {
			yyval = scm_cons (loc_on_copy (parser, (yylsp[-1]), yyvsp[-3]), yyvsp[-1]);
			MYBACKUP (REAL, yyvsp[0], (yylsp[0]));
		}
	}
#line 5812 "./out/parser.cc"
    break;

  case 283: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup NUMBER_IDENTIFIER  */
#line 2214 "parser.yy"
        {
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
		{
			yyval = scm_cons (yyvsp[0], yyvsp[-1]);
		} else {
			yyval = scm_cons (loc_on_copy (parser, (yylsp[-1]), yyvsp[-3]), yyvsp[-1]);
			MYBACKUP (NUMBER_IDENTIFIER, yyvsp[0], (yylsp[0]));
		}
	}
#line 5826 "./out/parser.cc"
    break;

  case 284: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup '-' UNSIGNED  */
#line 2224 "parser.yy"
        {
		SCM n = scm_difference (yyvsp[0], SCM_UNDEFINED);
		if (scm_is_true (ly_call (yyvsp[-3], n))) {
			yyval = yyvsp[-2];
			MYREPARSE ((yylsp[0]), yyvsp[-3], REAL, n);
		} else {
			Music *t = MY_MAKE_MUSIC ("FingeringEvent", (yylsp[0]));
			set_property (t, "digit", yyvsp[0]);
			yyval = t->unprotect ();
			if (scm_is_true (ly_call (yyvsp[-3], yyval)))
				yyval = scm_cons (yyval, yyvsp[-2]);
			else {
				yyval = scm_cons (loc_on_copy (parser, (yylsp[-2]), yyvsp[-4]), yyvsp[-2]);
				MYBACKUP (UNSIGNED, yyvsp[0], (yylsp[0]));
				parser->lexer_->push_extra_token ((yylsp[-1]), '-');
			}
		}
	}
#line 5849 "./out/parser.cc"
    break;

  case 285: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup '-' REAL  */
#line 2243 "parser.yy"
        {
		SCM n = scm_difference (yyvsp[0], SCM_UNDEFINED);
		if (scm_is_true (ly_call (yyvsp[-3], n))) {
			MYREPARSE ((yylsp[0]), yyvsp[-3], REAL, n);
			yyval = yyvsp[-2];
		} else {
			yyval = scm_cons (loc_on_copy (parser, (yylsp[-2]), yyvsp[-4]), yyvsp[-2]);
			MYBACKUP (REAL, n, (yylsp[0]));
		}
	}
#line 5864 "./out/parser.cc"
    break;

  case 286: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup '-' NUMBER_IDENTIFIER  */
#line 2254 "parser.yy"
        {
		SCM n = scm_difference (yyvsp[0], SCM_UNDEFINED);
		if (scm_is_true (ly_call (yyvsp[-3], n))) {
			yyval = scm_cons (n, yyvsp[-2]);
		} else {
			yyval = scm_cons (loc_on_copy (parser, (yylsp[-2]), yyvsp[-4]), yyvsp[-2]);
			MYBACKUP (NUMBER_IDENTIFIER, n, (yylsp[0]));
		}
	}
#line 5878 "./out/parser.cc"
    break;

  case 287: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup DURATION_IDENTIFIER  */
#line 2264 "parser.yy"
        {
		yyval = yyvsp[-1];
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
			MYREPARSE ((yylsp[0]), yyvsp[-2], DURATION_IDENTIFIER, yyvsp[0]);
		else if (scm_is_true
			 (ly_call
			  (yyvsp[-2], make_music_from_simple (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], DURATION_ARG, yyvsp[0]);
		else {
			yyval = scm_cons (loc_on_copy (parser, (yylsp[-1]), yyvsp[-3]), yyvsp[-1]);
			MYBACKUP (DURATION_IDENTIFIER, yyvsp[0], (yylsp[0]));
		}
	}
#line 5896 "./out/parser.cc"
    break;

  case 288: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup SCM_IDENTIFIER  */
#line 2278 "parser.yy"
        {
		SCM res = try_string_variants (yyvsp[-2], yyvsp[0]);
		if (!SCM_UNBNDP (res))
			if (scm_is_pair (res)) {
				yyval = yyvsp[-1];
				MYREPARSE ((yylsp[0]), yyvsp[-2], SYMBOL_LIST, res);
			}
			else
				yyval = scm_cons (res, yyvsp[-1]);
		else {
			yyval = scm_cons (loc_on_copy (parser, (yylsp[-1]), yyvsp[-3]), yyvsp[-1]);
			MYBACKUP (SCM_IDENTIFIER, yyvsp[0], (yylsp[0]));
		}
	}
#line 5915 "./out/parser.cc"
    break;

  case 289: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup STRING  */
#line 2293 "parser.yy"
        {
		SCM res = try_string_variants (yyvsp[-2], yyvsp[0]);
		if (!SCM_UNBNDP (res))
			if (scm_is_pair (res)) {
				yyval = yyvsp[-1];
				MYREPARSE ((yylsp[0]), yyvsp[-2], SYMBOL_LIST, res);
			}
			else
				yyval = scm_cons (res, yyvsp[-1]);
		else {
			yyval = scm_cons (loc_on_copy (parser, (yylsp[-1]), yyvsp[-3]), yyvsp[-1]);
			MYBACKUP (STRING, yyvsp[0], (yylsp[0]));
		}
	}
#line 5934 "./out/parser.cc"
    break;

  case 290: /* function_arglist_backup: "optional?" "scheme?" function_arglist_backup SYMBOL  */
#line 2308 "parser.yy"
        {
		SCM res = try_word_variants (yyvsp[-2], yyvsp[0]);
		if (!SCM_UNBNDP (res))
			if (scm_is_pair (res)) {
				yyval = yyvsp[-1];
				MYREPARSE ((yylsp[0]), yyvsp[-2], SYMBOL_LIST, res);
			}
			else
				yyval = scm_cons (res, yyvsp[-1]);
		else {
			yyval = scm_cons (loc_on_copy (parser, (yylsp[-1]), yyvsp[-3]), yyvsp[-1]);
			MYBACKUP (STRING, yyvsp[0], (yylsp[0]));
		}
	}
#line 5953 "./out/parser.cc"
    break;

  case 291: /* function_arglist_backup: function_arglist_backup "(reparsed?)" pitch_or_music  */
#line 2323 "parser.yy"
        {
		if (scm_is_true (ly_call (yyvsp[-1], yyvsp[0])))
			yyval = scm_cons (yyvsp[0], yyvsp[-2]);
		else
			yyval = check_scheme_arg (parser, (yylsp[0]),
					       make_music_from_simple
					       (parser, (yylsp[0]), yyvsp[0]),
					       yyvsp[-2], yyvsp[-1]);
	}
#line 5967 "./out/parser.cc"
    break;

  case 292: /* function_arglist_backup: function_arglist_backup "(reparsed?)" bare_number_common  */
#line 2333 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]),
				       yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 5976 "./out/parser.cc"
    break;

  case 293: /* function_arglist_backup: function_arglist_backup "(reparsed?)" duration  */
#line 2338 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]),
				       yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 5985 "./out/parser.cc"
    break;

  case 294: /* function_arglist_backup: function_arglist_backup "(reparsed?)" reparsed_rhythm  */
#line 2343 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]),
				       yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 5994 "./out/parser.cc"
    break;

  case 295: /* function_arglist_backup: function_arglist_backup "(reparsed?)" symbol_list_arg  */
#line 2348 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 6002 "./out/parser.cc"
    break;

  case 297: /* function_arglist: "optional?" "scheme?" function_arglist_skip_nonbackup "\\default"  */
#line 2356 "parser.yy"
        {
		yyval = scm_cons (loc_on_copy (parser, (yylsp[0]), yyvsp[-3]), yyvsp[-1]);
	}
#line 6010 "./out/parser.cc"
    break;

  case 299: /* function_arglist_skip_nonbackup: "optional?" "scheme?" function_arglist_skip_nonbackup  */
#line 2364 "parser.yy"
        {
		yyval = scm_cons (loc_on_copy (parser, (yylsp[0]), yyvsp[-2]), yyvsp[0]);
	}
#line 6018 "./out/parser.cc"
    break;

  case 300: /* function_arglist_partial: "scheme?" function_arglist_optional  */
#line 2383 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 6026 "./out/parser.cc"
    break;

  case 301: /* function_arglist_partial: "scheme?" function_arglist_partial_optional  */
#line 2387 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 6034 "./out/parser.cc"
    break;

  case 302: /* function_arglist_partial: "optional?" "scheme?" function_arglist_nonbackup  */
#line 2391 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 6042 "./out/parser.cc"
    break;

  case 303: /* function_arglist_partial: "optional?" "scheme?" function_arglist_partial  */
#line 2395 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 6050 "./out/parser.cc"
    break;

  case 304: /* function_arglist_partial_optional: "scheme?" function_arglist_optional  */
#line 2402 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 6058 "./out/parser.cc"
    break;

  case 305: /* function_arglist_partial_optional: "scheme?" function_arglist_partial_optional  */
#line 2406 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 6066 "./out/parser.cc"
    break;

  case 306: /* function_arglist_partial_optional: "optional?" "scheme?" function_arglist_backup  */
#line 2410 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 6074 "./out/parser.cc"
    break;

  case 307: /* function_arglist_partial_optional: "optional?" "scheme?" function_arglist_partial_optional  */
#line 2414 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 6082 "./out/parser.cc"
    break;

  case 308: /* function_arglist_common: EXPECT_NO_MORE_ARGS  */
#line 2420 "parser.yy"
                            {
		yyval = SCM_EOL;
	}
#line 6090 "./out/parser.cc"
    break;

  case 309: /* function_arglist_common: "scheme?" function_arglist_optional embedded_scm_arg  */
#line 2424 "parser.yy"
        {
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
			yyval = scm_cons (yyvsp[0], yyvsp[-1]);
		else
			yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-1], yyvsp[-2]);
	}
#line 6101 "./out/parser.cc"
    break;

  case 310: /* function_arglist_common: "scheme?" function_arglist_optional bare_number_common  */
#line 2431 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]),
				       yyvsp[0], yyvsp[-1], yyvsp[-2]);
	}
#line 6110 "./out/parser.cc"
    break;

  case 311: /* function_arglist_common: "scheme?" function_arglist_optional post_event_nofinger  */
#line 2436 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]),
				       yyvsp[0], yyvsp[-1], yyvsp[-2]);
	}
#line 6119 "./out/parser.cc"
    break;

  case 312: /* function_arglist_common: "scheme?" function_arglist_optional '-' NUMBER_IDENTIFIER  */
#line 2441 "parser.yy"
        {
		SCM n = scm_difference (yyvsp[0], SCM_UNDEFINED);
		yyval = check_scheme_arg (parser, (yylsp[0]), n, yyvsp[-2], yyvsp[-3]);
	}
#line 6128 "./out/parser.cc"
    break;

  case 313: /* function_arglist_common: function_arglist_common_reparse "(reparsed?)" SCM_ARG  */
#line 2446 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]),
				       yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 6137 "./out/parser.cc"
    break;

  case 314: /* function_arglist_common: function_arglist_common_reparse "(reparsed?)" lyric_element_music  */
#line 2451 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]),
				       yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 6146 "./out/parser.cc"
    break;

  case 315: /* function_arglist_common: function_arglist_common_reparse "(reparsed?)" pitch_or_music  */
#line 2456 "parser.yy"
        {
		if (scm_is_true (ly_call (yyvsp[-1], yyvsp[0])))
			yyval = scm_cons (yyvsp[0], yyvsp[-2]);
		else
			yyval = check_scheme_arg (parser, (yylsp[0]),
					       make_music_from_simple
					       (parser, (yylsp[0]), yyvsp[0]),
					       yyvsp[-2], yyvsp[-1], yyvsp[0]);
	}
#line 6160 "./out/parser.cc"
    break;

  case 316: /* function_arglist_common: function_arglist_common_reparse "(reparsed?)" bare_number_common  */
#line 2466 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]),
				       yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 6169 "./out/parser.cc"
    break;

  case 317: /* function_arglist_common: function_arglist_common_reparse "(reparsed?)" duration  */
#line 2471 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]),
				       yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 6178 "./out/parser.cc"
    break;

  case 318: /* function_arglist_common: function_arglist_common_reparse "(reparsed?)" reparsed_rhythm  */
#line 2476 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]),
				       yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 6187 "./out/parser.cc"
    break;

  case 319: /* function_arglist_common: function_arglist_common_reparse "(reparsed?)" symbol_list_arg  */
#line 2481 "parser.yy"
        {
		yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-2], yyvsp[-1]);
	}
#line 6195 "./out/parser.cc"
    break;

  case 320: /* function_arglist_common_reparse: "scheme?" function_arglist_optional SCM_IDENTIFIER  */
#line 2488 "parser.yy"
        {
		yyval = yyvsp[-1];
		SCM res = try_string_variants (yyvsp[-2], yyvsp[0]);
		if (!SCM_UNBNDP (res))
			if (scm_is_pair (res))
				MYREPARSE ((yylsp[0]), yyvsp[-2], SYMBOL_LIST, res);
			else
				MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, res);
		else if (scm_is_true
			 (ly_call
			  (yyvsp[-2], make_music_from_simple (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], LYRIC_ELEMENT, yyvsp[0]);
		else
			// This is going to flag a syntax error, we
			// know the predicate to be false.
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]);
	}
#line 6217 "./out/parser.cc"
    break;

  case 321: /* function_arglist_common_reparse: "scheme?" function_arglist_optional pitch  */
#line 2506 "parser.yy"
        {
		yyval = yyvsp[-1];
		if (scm_is_true
		    (ly_call
		     (yyvsp[-2], make_music_from_simple
		      (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], PITCH_IDENTIFIER, yyvsp[0]);
		else
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]);
	}
#line 6232 "./out/parser.cc"
    break;

  case 322: /* function_arglist_common_reparse: "scheme?" function_arglist_optional steno_tonic_pitch  */
#line 2517 "parser.yy"
        {
		yyval = yyvsp[-1];
		if (scm_is_true
		    (ly_call
		     (yyvsp[-2], make_music_from_simple
		      (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], TONICNAME_PITCH, yyvsp[0]);
		else
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]);
	}
#line 6247 "./out/parser.cc"
    break;

  case 323: /* function_arglist_common_reparse: "scheme?" function_arglist_optional STRING  */
#line 2528 "parser.yy"
        {
		yyval = yyvsp[-1];
		SCM res = try_string_variants (yyvsp[-2], yyvsp[0]);
		if (!SCM_UNBNDP (res))
			if (scm_is_pair (res))
				MYREPARSE ((yylsp[0]), yyvsp[-2], SYMBOL_LIST, res);
			else
				MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, res);
		else if (scm_is_true
			 (ly_call
			  (yyvsp[-2], make_music_from_simple (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], LYRIC_ELEMENT, yyvsp[0]);
		else
			// This is going to flag a syntax error, we
			// know the predicate to be false.
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]);
	}
#line 6269 "./out/parser.cc"
    break;

  case 324: /* function_arglist_common_reparse: "scheme?" function_arglist_optional SYMBOL  */
#line 2546 "parser.yy"
        {
		yyval = yyvsp[-1];
		SCM res = try_word_variants (yyvsp[-2], yyvsp[0]);
		if (!SCM_UNBNDP (res))
			if (scm_is_pair (res))
				MYREPARSE ((yylsp[0]), yyvsp[-2], SYMBOL_LIST, res);
			else
				MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, res);
		else if (scm_is_true
			 (ly_call
			  (yyvsp[-2], make_music_from_simple (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], LYRIC_ELEMENT, yyvsp[0]);
		else
			// This is going to flag a syntax error, we
			// know the predicate to be false.
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]);
	}
#line 6291 "./out/parser.cc"
    break;

  case 325: /* function_arglist_common_reparse: "scheme?" function_arglist_optional full_markup  */
#line 2564 "parser.yy"
        {
		yyval = yyvsp[-1];
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]);
		else if (scm_is_true
			 (ly_call
			  (yyvsp[-2], make_music_from_simple (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], LYRIC_ELEMENT, yyvsp[0]);
		else
			// This is going to flag a syntax error, we
			// know the predicate to be false.
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]);
	}
#line 6309 "./out/parser.cc"
    break;

  case 326: /* function_arglist_common_reparse: "scheme?" function_arglist_optional UNSIGNED  */
#line 2578 "parser.yy"
        {
		yyval = yyvsp[-1];
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
			// May be 3 \cm or similar
			MYREPARSE ((yylsp[0]), yyvsp[-2], REAL, yyvsp[0]);
		else if (scm_is_true (ly_call (yyvsp[-2], ly_list (yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], SYMBOL_LIST, ly_list (yyvsp[0]));
		else {
			SCM d = make_duration (yyvsp[0]);
			if (!SCM_UNBNDP (d)) {
				if (scm_is_true (ly_call (yyvsp[-2], d)))
					MYREPARSE ((yylsp[0]), yyvsp[-2], DURATION_IDENTIFIER, d);
				else if (scm_is_true
					 (ly_call
					  (yyvsp[-2], make_music_from_simple (parser, (yylsp[0]), d))))
					MYREPARSE ((yylsp[0]), yyvsp[-2], DURATION_ARG, d);
				else
					MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]); // trigger error
			} else
				MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]); // trigger error
		}
	}
#line 6336 "./out/parser.cc"
    break;

  case 327: /* function_arglist_common_reparse: "scheme?" function_arglist_optional DURATION_IDENTIFIER  */
#line 2601 "parser.yy"
        {
		yyval = yyvsp[-1];
		if (scm_is_true (ly_call (yyvsp[-2], yyvsp[0])))
			MYREPARSE ((yylsp[0]), yyvsp[-2], DURATION_IDENTIFIER, yyvsp[0]);
		else if (scm_is_true
			 (ly_call
			  (yyvsp[-2], make_music_from_simple (parser, (yylsp[0]), yyvsp[0]))))
			MYREPARSE ((yylsp[0]), yyvsp[-2], DURATION_ARG, yyvsp[0]);
		else
			MYREPARSE ((yylsp[0]), yyvsp[-2], SCM_ARG, yyvsp[0]); // trigger error
	}
#line 6352 "./out/parser.cc"
    break;

  case 328: /* function_arglist_common_reparse: "scheme?" function_arglist_optional '-' UNSIGNED  */
#line 2613 "parser.yy"
        {
		yyval = yyvsp[-2];
		SCM n = scm_difference (yyvsp[0], SCM_UNDEFINED);
		if (scm_is_true (ly_call (yyvsp[-3], n)))
			MYREPARSE ((yylsp[0]), yyvsp[-3], REAL, n);
		else {
			Music *t = MY_MAKE_MUSIC ("FingeringEvent", (yylsp[0]));
			set_property (t, "digit", yyvsp[0]);
			SCM m = t->unprotect ();
			if (scm_is_true (ly_call (yyvsp[-3], m)))
				MYREPARSE ((yylsp[0]), yyvsp[-3], SCM_ARG, m);
			else
				MYREPARSE ((yylsp[0]), yyvsp[-3], SCM_ARG, yyvsp[0]);
		}
	}
#line 6372 "./out/parser.cc"
    break;

  case 329: /* function_arglist_common_reparse: "scheme?" function_arglist_optional '-' REAL  */
#line 2629 "parser.yy"
        {
		yyval = yyvsp[-2];
		SCM n = scm_difference (yyvsp[0], SCM_UNDEFINED);
		MYREPARSE ((yylsp[0]), yyvsp[-3], REAL, n);
	}
#line 6382 "./out/parser.cc"
    break;

  case 331: /* function_arglist_optional: "optional?" "scheme?" function_arglist_skip_backup "\\default"  */
#line 2639 "parser.yy"
        {
		yyval = scm_cons (loc_on_copy (parser, (yylsp[0]), yyvsp[-3]), yyvsp[-1]);
	}
#line 6390 "./out/parser.cc"
    break;

  case 334: /* function_arglist_skip_backup: "optional?" "scheme?" function_arglist_skip_backup  */
#line 2648 "parser.yy"
        {
		yyval = scm_cons (loc_on_copy (parser, (yylsp[0]), yyvsp[-2]), yyvsp[0]);
	}
#line 6398 "./out/parser.cc"
    break;

  case 335: /* music_function_call: MUSIC_FUNCTION function_arglist  */
#line 2654 "parser.yy"
                                        {
		yyval = MAKE_SYNTAX (music_function, (yyloc),
				  yyvsp[-1], yyvsp[0]);
	}
#line 6407 "./out/parser.cc"
    break;

  case 336: /* optional_id: %empty  */
#line 2662 "parser.yy"
             { yyval = SCM_EOL; }
#line 6413 "./out/parser.cc"
    break;

  case 337: /* optional_id: '=' simple_string  */
#line 2663 "parser.yy"
                            {
		yyval = yyvsp[0];
	}
#line 6421 "./out/parser.cc"
    break;

  case 338: /* $@10: %empty  */
#line 2675 "parser.yy"
        {
		parser->lexer_->push_lyric_state ();
	}
#line 6429 "./out/parser.cc"
    break;

  case 339: /* lyric_mode_music: $@10 grouped_music_list  */
#line 2678 "parser.yy"
        {
		parser->lexer_->pop_state ();
		yyval = yyvsp[0];
	}
#line 6438 "./out/parser.cc"
    break;

  case 341: /* mode_changed_music: mode_changing_head grouped_music_list  */
#line 2686 "parser.yy"
                                              {
		if (scm_is_eq (yyvsp[-1], ly_symbol2scm ("chords")))
		{
		  yyval = MAKE_SYNTAX (unrelativable_music, (yyloc), yyvsp[0]);
		}
		else
		{
		  yyval = yyvsp[0];
		}
		parser->lexer_->pop_state ();
	}
#line 6454 "./out/parser.cc"
    break;

  case 342: /* mode_changed_music: mode_changing_head_with_context optional_context_mods grouped_music_list  */
#line 2697 "parser.yy"
                                                                                   {
		yyval = MAKE_SYNTAX (context_create, (yyloc), yyvsp[-2], SCM_EOL, yyvsp[-1], yyvsp[0]);
		if (scm_is_eq (yyvsp[-2], ly_symbol2scm ("ChordNames")))
		{
		  yyval = MAKE_SYNTAX (unrelativable_music, (yyloc), yyval);
		}
		parser->lexer_->pop_state ();
	}
#line 6467 "./out/parser.cc"
    break;

  case 343: /* mode_changing_head: "\\notemode"  */
#line 2708 "parser.yy"
                 {
		parser->lexer_->push_note_state ();

		yyval = ly_symbol2scm ("notes");
	}
#line 6477 "./out/parser.cc"
    break;

  case 344: /* mode_changing_head: "\\drummode"  */
#line 2714 "parser.yy"
                {
		parser->lexer_->push_drum_state ();
		yyval = ly_symbol2scm ("drums");
	}
#line 6486 "./out/parser.cc"
    break;

  case 345: /* mode_changing_head: "\\figuremode"  */
#line 2718 "parser.yy"
                     {
		parser->lexer_->push_figuredbass_state ();

		yyval = ly_symbol2scm ("figures");
	}
#line 6496 "./out/parser.cc"
    break;

  case 346: /* mode_changing_head: "\\chordmode"  */
#line 2723 "parser.yy"
                    {
		SCM mods = parser->lexer_->lookup_identifier_symbol (ly_symbol2scm ("chordmodifiers"));
		parser->lexer_->chordmodifier_tab_ = Hash_table::alist_to_hashq_table (mods);
		parser->lexer_->push_chord_state ();
		yyval = ly_symbol2scm ("chords");

	}
#line 6508 "./out/parser.cc"
    break;

  case 347: /* mode_changing_head: "\\lyricmode"  */
#line 2731 "parser.yy"
                { parser->lexer_->push_lyric_state ();
		yyval = ly_symbol2scm ("lyrics");
	}
#line 6516 "./out/parser.cc"
    break;

  case 348: /* mode_changing_head_with_context: "\\drums"  */
#line 2737 "parser.yy"
              {
		parser->lexer_->push_drum_state();
		yyval = ly_symbol2scm ("DrumStaff");
	}
#line 6525 "./out/parser.cc"
    break;

  case 349: /* mode_changing_head_with_context: "\\figures"  */
#line 2741 "parser.yy"
                  {
		parser->lexer_->push_figuredbass_state ();

		yyval = ly_symbol2scm ("FiguredBass");
	}
#line 6535 "./out/parser.cc"
    break;

  case 350: /* mode_changing_head_with_context: "\\chords"  */
#line 2746 "parser.yy"
                 {
		SCM mods = parser->lexer_->lookup_identifier_symbol (ly_symbol2scm ("chordmodifiers"));
		parser->lexer_->chordmodifier_tab_ = Hash_table::alist_to_hashq_table (mods);
		parser->lexer_->push_chord_state ();
		yyval = ly_symbol2scm ("ChordNames");
	}
#line 6546 "./out/parser.cc"
    break;

  case 351: /* mode_changing_head_with_context: "\\lyrics"  */
#line 2753 "parser.yy"
                { parser->lexer_->push_lyric_state ();
		yyval = ly_symbol2scm ("Lyrics");
	}
#line 6554 "./out/parser.cc"
    break;

  case 352: /* context_change: "\\change" symbol '=' simple_string  */
#line 2759 "parser.yy"
                                         {
		yyval = MAKE_SYNTAX (context_change, (yyloc), yyvsp[-2], yyvsp[0]);
	}
#line 6562 "./out/parser.cc"
    break;

  case 353: /* property_path: symbol_list_rev  */
#line 2766 "parser.yy"
                         {
		yyval = scm_reverse_x (yyvsp[0], SCM_EOL);
	}
#line 6570 "./out/parser.cc"
    break;

  case 354: /* property_operation: symbol '=' scalar  */
#line 2772 "parser.yy"
                          {
		yyval = ly_list (ly_symbol2scm ("assign"), yyvsp[-2], yyvsp[0]);
	}
#line 6578 "./out/parser.cc"
    break;

  case 355: /* property_operation: "\\unset" symbol  */
#line 2775 "parser.yy"
                       {
		yyval = ly_list (ly_symbol2scm ("unset"), yyvsp[0]);
	}
#line 6586 "./out/parser.cc"
    break;

  case 356: /* property_operation: "\\override" revert_arg '=' scalar  */
#line 2778 "parser.yy"
                                         {
		if (scm_ilength (yyvsp[-2]) < 2) {
			parser->parser_error ((yylsp[-2]), _("bad grob property path"));
			yyval = SCM_UNDEFINED;
		} else {
			yyval = scm_cons (ly_symbol2scm ("push"),
				       scm_cons2 (scm_car (yyvsp[-2]),
						  yyvsp[0],
						  scm_cdr (yyvsp[-2])));
		}
	}
#line 6602 "./out/parser.cc"
    break;

  case 357: /* property_operation: "\\revert" revert_arg  */
#line 2789 "parser.yy"
                            {
		yyval = scm_cons (ly_symbol2scm ("pop"), yyvsp[0]);
	}
#line 6610 "./out/parser.cc"
    break;

  case 358: /* revert_arg: revert_arg_backup "(backed-up?)" symbol_list_arg  */
#line 2819 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 6618 "./out/parser.cc"
    break;

  case 359: /* revert_arg_backup: revert_arg_part  */
#line 2826 "parser.yy"
        {
		if (scm_is_null (yyvsp[0])
		    || scm_is_null (scm_cdr (yyvsp[0])))
			MYBACKUP (SCM_ARG, yyvsp[0], (yylsp[0]));
		else
			MYBACKUP (SYMBOL_LIST, scm_reverse_x (yyvsp[0], SCM_EOL), (yylsp[0]));
	}
#line 6630 "./out/parser.cc"
    break;

  case 361: /* revert_arg_part: revert_arg_backup "(backed-up?)" SCM_ARG '.' symbol_list_part  */
#line 2839 "parser.yy"
        {
		yyval = scm_append_x (ly_list (yyvsp[0], yyvsp[-2]));
	}
#line 6638 "./out/parser.cc"
    break;

  case 362: /* revert_arg_part: revert_arg_backup "(backed-up?)" SCM_ARG ',' symbol_list_part  */
#line 2843 "parser.yy"
        {
		yyval = scm_append_x (ly_list (yyvsp[0], yyvsp[-2]));
	}
#line 6646 "./out/parser.cc"
    break;

  case 363: /* revert_arg_part: revert_arg_backup "(backed-up?)" SCM_ARG symbol_list_part  */
#line 2847 "parser.yy"
        {
		yyval = scm_append_x (ly_list (yyvsp[0], yyvsp[-1]));
		property_path_dot_warning ((yylsp[0]), scm_reverse (yyval));
	}
#line 6655 "./out/parser.cc"
    break;

  case 364: /* context_def_mod: "\\consists"  */
#line 2854 "parser.yy"
                 { yyval = ly_symbol2scm ("consists"); }
#line 6661 "./out/parser.cc"
    break;

  case 365: /* context_def_mod: "\\remove"  */
#line 2855 "parser.yy"
                 { yyval = ly_symbol2scm ("remove"); }
#line 6667 "./out/parser.cc"
    break;

  case 366: /* context_def_mod: "\\accepts"  */
#line 2857 "parser.yy"
                  { yyval = ly_symbol2scm ("accepts"); }
#line 6673 "./out/parser.cc"
    break;

  case 367: /* context_def_mod: "\\defaultchild"  */
#line 2858 "parser.yy"
                       { yyval = ly_symbol2scm ("default-child"); }
#line 6679 "./out/parser.cc"
    break;

  case 368: /* context_def_mod: "\\denies"  */
#line 2859 "parser.yy"
                 { yyval = ly_symbol2scm ("denies"); }
#line 6685 "./out/parser.cc"
    break;

  case 369: /* context_def_mod: "\\alias"  */
#line 2861 "parser.yy"
                { yyval = ly_symbol2scm ("alias"); }
#line 6691 "./out/parser.cc"
    break;

  case 370: /* context_def_mod: "\\type"  */
#line 2862 "parser.yy"
               { yyval = ly_symbol2scm ("translator-type"); }
#line 6697 "./out/parser.cc"
    break;

  case 371: /* context_def_mod: "\\description"  */
#line 2863 "parser.yy"
                      { yyval = ly_symbol2scm ("description"); }
#line 6703 "./out/parser.cc"
    break;

  case 372: /* context_def_mod: "\\name"  */
#line 2864 "parser.yy"
               { yyval = ly_symbol2scm ("context-name"); }
#line 6709 "./out/parser.cc"
    break;

  case 373: /* context_mod: property_operation  */
#line 2868 "parser.yy"
                           { yyval = yyvsp[0]; }
#line 6715 "./out/parser.cc"
    break;

  case 374: /* context_mod: context_def_mod STRING  */
#line 2869 "parser.yy"
                                 {
		yyval = ly_list (yyvsp[-1], yyvsp[0]);
	}
#line 6723 "./out/parser.cc"
    break;

  case 375: /* context_mod: context_def_mod SYMBOL  */
#line 2872 "parser.yy"
                                 {
		yyval = ly_list (yyvsp[-1], yyvsp[0]);
	}
#line 6731 "./out/parser.cc"
    break;

  case 376: /* context_mod: context_def_mod embedded_scm  */
#line 2876 "parser.yy"
        {
		if (!scm_is_string (yyvsp[0])
		    && !scm_is_eq (yyvsp[-1], ly_symbol2scm ("consists"))
		    && !scm_is_eq (yyvsp[-1], ly_symbol2scm ("remove")))
		{
			yyval = SCM_EOL;
			parser->parser_error ((yylsp[-1]), _ ("only \\consists and \\remove take non-string argument."));
		}
		else
		{
			yyval = ly_list (yyvsp[-1], yyvsp[0]);
		}
	}
#line 6749 "./out/parser.cc"
    break;

  case 377: /* grob_prop_spec: symbol_list_rev  */
#line 2893 "parser.yy"
        {
		yyval = scm_reverse_x (yyvsp[0], SCM_EOL);
	}
#line 6757 "./out/parser.cc"
    break;

  case 378: /* grob_prop_path: grob_prop_spec  */
#line 2901 "parser.yy"
        {
		if (scm_is_pair (yyvsp[0])
		    && from_scm<bool>
		    (scm_object_property (scm_car (yyvsp[0]),
					  ly_symbol2scm ("is-grob?"))))
			yyval = scm_cons (ly_symbol2scm ("Bottom"), yyvsp[0]);
		if (!scm_is_pair (yyval)
		    || !scm_is_pair (scm_cdr (yyval))
		    || !scm_is_pair (scm_cddr (yyval)))
		{
			parser->parser_error ((yylsp[0]), _ ("bad grob property path"));
			yyval = SCM_UNDEFINED;
		}
	}
#line 6776 "./out/parser.cc"
    break;

  case 379: /* grob_prop_path: grob_prop_spec property_path  */
#line 2916 "parser.yy"
        {
		if (scm_is_pair (yyvsp[-1])
		    && from_scm<bool>
		    (scm_object_property (scm_car (yyvsp[-1]),
					  ly_symbol2scm ("is-grob?"))))
			yyval = scm_cons (ly_symbol2scm ("Bottom"), yyvsp[-1]);
		if (!scm_is_pair (yyval)
		    || !scm_is_pair (scm_cdr (yyval))
		    || scm_is_pair (scm_cddr (yyval))
		    || !scm_is_pair (yyvsp[0]))
		{
			parser->parser_error ((yylsp[-1]), _ ("bad grob property path"));
			yyval = SCM_UNDEFINED;
		} else {
			property_path_dot_warning ((yylsp[0]), ly_append (yyvsp[-1], yyvsp[0]));
			yyval = scm_append_x (ly_list (yyval, yyvsp[0]));
		}
	}
#line 6799 "./out/parser.cc"
    break;

  case 380: /* context_prop_spec: symbol_list_rev  */
#line 2939 "parser.yy"
        {
		SCM l = scm_reverse_x (yyvsp[0], SCM_EOL);
		switch (scm_ilength (l)) {
		case 1:
			l = scm_cons (ly_symbol2scm ("Bottom"), l);
		case 2:
			break;
		default:
			parser->parser_error ((yylsp[0]), _ ("bad context property path"));
			l = SCM_UNDEFINED;
		}
		yyval = l;
	}
#line 6817 "./out/parser.cc"
    break;

  case 381: /* simple_revert_context: symbol_list_part  */
#line 2971 "parser.yy"
        {
		yyvsp[0] = scm_reverse_x (yyvsp[0], SCM_EOL);
		if (scm_is_null (yyvsp[0])
		    || from_scm<bool>
		    (scm_object_property (scm_car (yyvsp[0]),
					  ly_symbol2scm ("is-grob?")))) {
			yyval = ly_symbol2scm ("Bottom");
			parser->lexer_->push_extra_token ((yylsp[0]), SCM_IDENTIFIER, yyvsp[0]);
		} else {
			yyval = scm_car (yyvsp[0]);
			parser->lexer_->push_extra_token ((yylsp[0]), SCM_IDENTIFIER,
							  scm_cdr (yyvsp[0]));
		}
	}
#line 6836 "./out/parser.cc"
    break;

  case 382: /* music_property_def: "\\override" grob_prop_path '=' scalar  */
#line 2988 "parser.yy"
                                           {
		if (SCM_UNBNDP (yyvsp[-2]))
			yyval = MAKE_SYNTAX (unspecified_music, (yyloc));
		else
			yyval = MAKE_SYNTAX (property_override, (yyloc),
					  scm_car (yyvsp[-2]),
					  scm_cdr (yyvsp[-2]),
					  yyvsp[0]);
	}
#line 6850 "./out/parser.cc"
    break;

  case 383: /* music_property_def: "\\revert" simple_revert_context revert_arg  */
#line 2997 "parser.yy"
                                                  {
		yyval = MAKE_SYNTAX (property_revert, (yyloc), yyvsp[-1], yyvsp[0]);
	}
#line 6858 "./out/parser.cc"
    break;

  case 384: /* music_property_def: "\\set" context_prop_spec '=' scalar  */
#line 3000 "parser.yy"
                                           {
		if (SCM_UNBNDP (yyvsp[-2]))
			yyval = MAKE_SYNTAX (unspecified_music, (yyloc));
		else
			yyval = MAKE_SYNTAX (property_set, (yyloc),
					  scm_car (yyvsp[-2]),
					  scm_cadr (yyvsp[-2]),
					  yyvsp[0]);
	}
#line 6872 "./out/parser.cc"
    break;

  case 385: /* music_property_def: "\\unset" context_prop_spec  */
#line 3009 "parser.yy"
                                  {
		if (SCM_UNBNDP (yyvsp[0]))
			yyval = MAKE_SYNTAX (unspecified_music, (yyloc));
		else
			yyval = MAKE_SYNTAX (property_unset, (yyloc),
					  scm_car (yyvsp[0]),
					  scm_cadr (yyvsp[0]));
	}
#line 6885 "./out/parser.cc"
    break;

  case 392: /* text: embedded_scm_bare  */
#line 3030 "parser.yy"
        {
		if (Text_interface::is_markup (yyvsp[0])) {
			yyval = yyvsp[0];
		} else {
			parser->parser_error ((yylsp[0]), (_ ("markup expected")));
			yyval = scm_string (SCM_EOL);
		}
	}
#line 6898 "./out/parser.cc"
    break;

  case 395: /* simple_string: embedded_scm_bare  */
#line 3043 "parser.yy"
        {
		if (scm_is_string (yyvsp[0])) {
			yyval = yyvsp[0];
		} else {
			parser->parser_error ((yylsp[0]), (_ ("simple string expected")));
			yyval = scm_string (SCM_EOL);
		}
	}
#line 6911 "./out/parser.cc"
    break;

  case 396: /* symbol: STRING  */
#line 3054 "parser.yy"
               {
		yyval = scm_string_to_symbol (yyvsp[0]);
	}
#line 6919 "./out/parser.cc"
    break;

  case 397: /* symbol: SYMBOL  */
#line 3058 "parser.yy"
        {
		if (!is_regular_identifier (yyvsp[0], false))
			parser->parser_error ((yylsp[0]), (_ ("symbol expected")));
		yyval = scm_string_to_symbol (yyvsp[0]);
	}
#line 6929 "./out/parser.cc"
    break;

  case 398: /* symbol: embedded_scm_bare  */
#line 3064 "parser.yy"
        {
		// This is a bit of overkill but makes the same
		// routine responsible for all symbol interpretations.
		yyval = try_string_variants (Guile_user::symbol_p, yyvsp[0]);
		if (SCM_UNBNDP (yyval))
		{
			parser->parser_error ((yylsp[0]), (_ ("symbol expected")));
			// Generate a unique symbol in case it is used
			// for an assignment or similar
			yyval = scm_make_symbol (to_scm ("undefined"));
		}
	}
#line 6946 "./out/parser.cc"
    break;

  case 403: /* scalar: '-' bare_number  */
#line 3091 "parser.yy"
        {
		yyval = scm_difference (yyvsp[0], SCM_UNDEFINED);
	}
#line 6954 "./out/parser.cc"
    break;

  case 405: /* scalar: symbol_list_part_bare '.' property_path  */
#line 3096 "parser.yy"
        {
		yyval = scm_reverse_x (yyvsp[-2], yyvsp[0]);
	}
#line 6962 "./out/parser.cc"
    break;

  case 406: /* scalar: symbol_list_part_bare ',' property_path  */
#line 3100 "parser.yy"
        {
		yyval = scm_reverse_x (yyvsp[-2], yyvsp[0]);
	}
#line 6970 "./out/parser.cc"
    break;

  case 407: /* event_chord: simple_element post_events  */
#line 3106 "parser.yy"
                                   {
		// Let the rhythmic music iterator sort this mess out.
		if (scm_is_pair (yyvsp[0])) {
			set_property (unsmob<Music> (yyval), "articulations",
							 scm_reverse_x (yyvsp[0], SCM_EOL));
		}
	}
#line 6982 "./out/parser.cc"
    break;

  case 408: /* event_chord: CHORD_REPETITION optional_notemode_duration post_events  */
#line 3113 "parser.yy"
                                                                  {
		yyval = MAKE_SYNTAX (repetition_chord, (yyloc),
				  yyvsp[-1], scm_reverse_x (yyvsp[0], SCM_EOL));
	}
#line 6991 "./out/parser.cc"
    break;

  case 409: /* event_chord: MULTI_MEASURE_REST optional_notemode_duration post_events  */
#line 3117 "parser.yy"
                                                                    {
		yyval = MAKE_SYNTAX (multi_measure_rest, (yyloc), yyvsp[-1],
				  scm_reverse_x (yyvsp[0], SCM_EOL));
	}
#line 7000 "./out/parser.cc"
    break;

  case 412: /* note_chord_element: chord_body optional_notemode_duration post_events  */
#line 3128 "parser.yy"
        {
		Music *m = unsmob<Music> (yyvsp[-2]);
		SCM dur = unsmob<Duration> (yyvsp[-1])->smobbed_copy ();
		SCM es = get_property (m, "elements");
		SCM postevs = scm_reverse_x (yyvsp[0], SCM_EOL);

		for (SCM s = es; scm_is_pair (s); s = scm_cdr (s))
			set_property (unsmob<Music> (scm_car (s)), "duration", dur);
		es = ly_append (es, postevs);

		set_property (m, "elements", es);
		m->set_spot (parser->lexer_->override_input ((yyloc)));
		yyval = m->self_scm ();
	}
#line 7019 "./out/parser.cc"
    break;

  case 413: /* chord_body: "<" chord_body_elements ">"  */
#line 3146 "parser.yy"
        {
		yyval = MAKE_SYNTAX (event_chord, (yyloc),
				  reverse_music_list (parser, (yyloc),
						      yyvsp[-1], false, false));
	}
#line 7029 "./out/parser.cc"
    break;

  case 414: /* chord_body: FIGURE_OPEN figure_list FIGURE_CLOSE  */
#line 3152 "parser.yy"
        {
		yyval = MAKE_SYNTAX (event_chord, (yyloc), scm_reverse_x (yyvsp[-1], SCM_EOL));
	}
#line 7037 "./out/parser.cc"
    break;

  case 415: /* chord_body_elements: %empty  */
#line 3158 "parser.yy"
                                { yyval = SCM_EOL; }
#line 7043 "./out/parser.cc"
    break;

  case 416: /* chord_body_elements: chord_body_elements chord_body_element  */
#line 3159 "parser.yy"
                                                 {
		if (unsmob<Music> (yyvsp[0]))
			yyval = scm_cons (yyvsp[0], yyvsp[-1]);
	}
#line 7052 "./out/parser.cc"
    break;

  case 417: /* chord_body_element: pitch_or_tonic_pitch exclamations questions octave_check post_events  */
#line 3167 "parser.yy"
        {
		bool q = from_scm<bool> (yyvsp[-2]);
		bool ex = from_scm<bool> (yyvsp[-3]);
		SCM check = yyvsp[-1];
		SCM post = yyvsp[0];

		Music *n = MY_MAKE_MUSIC ("NoteEvent", (yyloc));
		set_property (n, "pitch", yyvsp[-4]);
		if (q)
			set_property (n, "cautionary", SCM_BOOL_T);
                if (ex || q)
			set_property (n, "force-accidental", SCM_BOOL_T);

		if (scm_is_pair (post)) {
			SCM arts = scm_reverse_x (post, SCM_EOL);
			set_property (n, "articulations", arts);
		}
		if (scm_is_number (check))
		{
			int q = from_scm<int> (check);
			set_property (n, "absolute-octave", to_scm (q-1));
		}

		yyval = n->unprotect ();
	}
#line 7082 "./out/parser.cc"
    break;

  case 418: /* chord_body_element: DRUM_PITCH post_events  */
#line 3192 "parser.yy"
                                           {
		Music *n = MY_MAKE_MUSIC ("NoteEvent", (yyloc));
		set_property (n, "drum-type", yyvsp[-1]);

		if (scm_is_pair (yyvsp[0])) {
			SCM arts = scm_reverse_x (yyvsp[0], SCM_EOL);
			set_property (n, "articulations", arts);
		}
		yyval = n->unprotect ();
	}
#line 7097 "./out/parser.cc"
    break;

  case 419: /* chord_body_element: music_function_chord_body  */
#line 3203 "parser.yy"
        {
		Music *m = unsmob<Music> (yyvsp[0]);

		if (m && !m->is_mus_type ("post-event")) {
			while (m && m->is_mus_type ("music-wrapper-music")) {
				yyval = get_property (m, "element");
				m = unsmob<Music> (yyval);
			}

			if (!(m && m->is_mus_type ("rhythmic-event"))) {
				parser->parser_error ((yyloc), _ ("not a rhythmic event"));
				yyval = SCM_UNSPECIFIED;
			}
		}
	}
#line 7117 "./out/parser.cc"
    break;

  case 424: /* event_function_event: EVENT_FUNCTION function_arglist  */
#line 3228 "parser.yy"
                                        {
		yyval = MAKE_SYNTAX (music_function, (yyloc),
				  yyvsp[-1], yyvsp[0]);
	}
#line 7126 "./out/parser.cc"
    break;

  case 425: /* post_events: %empty  */
#line 3235 "parser.yy"
                    {
		yyval = SCM_EOL;
	}
#line 7134 "./out/parser.cc"
    break;

  case 426: /* post_events: post_events post_event  */
#line 3238 "parser.yy"
                                 {
		yyval = post_event_cons (yyvsp[0], yyvsp[-1]);
	}
#line 7142 "./out/parser.cc"
    break;

  case 427: /* post_event_nofinger: direction_less_event  */
#line 3244 "parser.yy"
                             {
		yyval = yyvsp[0];
	}
#line 7150 "./out/parser.cc"
    break;

  case 428: /* post_event_nofinger: script_dir music_function_call  */
#line 3247 "parser.yy"
                                         {
		Music *m = unsmob<Music> (yyvsp[0]);
		if (!m->is_mus_type ("post-event")) {
			parser->parser_error ((yylsp[0]), _ ("post-event expected"));
			yyval = SCM_UNSPECIFIED;
		} else {
			m->set_spot (parser->lexer_->override_input ((yyloc)));
			if (!SCM_UNBNDP (yyvsp[-1]))
				set_property (m, "direction", yyvsp[-1]);
			yyval = yyvsp[0];
		}
	}
#line 7167 "./out/parser.cc"
    break;

  case 429: /* post_event_nofinger: "--"  */
#line 3259 "parser.yy"
                 {
		if (!parser->lexer_->is_lyric_state ())
			parser->parser_error ((yylsp[0]), _ ("have to be in Lyric mode for lyrics"));
		yyval = MY_MAKE_MUSIC ("HyphenEvent", (yyloc))->unprotect ();
	}
#line 7177 "./out/parser.cc"
    break;

  case 430: /* post_event_nofinger: "__"  */
#line 3264 "parser.yy"
                   {
		if (!parser->lexer_->is_lyric_state ())
			parser->parser_error ((yylsp[0]), _ ("have to be in Lyric mode for lyrics"));
		yyval = MY_MAKE_MUSIC ("ExtenderEvent", (yyloc))->unprotect ();
	}
#line 7187 "./out/parser.cc"
    break;

  case 431: /* post_event_nofinger: script_dir direction_reqd_event  */
#line 3269 "parser.yy"
                                          {
		if (Music *m = unsmob<Music> (yyvsp[0])) {
			m->set_spot (parser->lexer_->override_input ((yyloc)));
			if (!SCM_UNBNDP (yyvsp[-1]))
			{
				set_property (m, "direction", yyvsp[-1]);
			}
		}
		yyval = yyvsp[0];
	}
#line 7202 "./out/parser.cc"
    break;

  case 432: /* post_event_nofinger: script_dir direction_less_event  */
#line 3279 "parser.yy"
                                          {
		if (auto *m = unsmob<Music> (yyvsp[0])) {
			m->set_spot (parser->lexer_->override_input ((yyloc)));
			if (!SCM_UNBNDP (yyvsp[-1])) {
				set_property (m, "direction", yyvsp[-1]);
			}
		}
		yyval = yyvsp[0];
	}
#line 7216 "./out/parser.cc"
    break;

  case 433: /* post_event_nofinger: '^' fingering  */
#line 3289 "parser.yy"
        {
		Music *m = unsmob<Music> (yyvsp[0]);
		m->set_spot (parser->lexer_->override_input ((yyloc)));
		set_property (m, "direction", to_scm (UP));
		yyval = yyvsp[0];
	}
#line 7227 "./out/parser.cc"
    break;

  case 434: /* post_event_nofinger: '_' fingering  */
#line 3296 "parser.yy"
        {
		Music *m = unsmob<Music> (yyvsp[0]);
		m->set_spot (parser->lexer_->override_input ((yyloc)));
		set_property (m, "direction", to_scm (DOWN));
		yyval = yyvsp[0];
	}
#line 7238 "./out/parser.cc"
    break;

  case 436: /* post_event: '-' fingering  */
#line 3306 "parser.yy"
                        {
		unsmob<Music> (yyvsp[0])->set_spot (parser->lexer_->override_input ((yyloc)));
		yyval = yyvsp[0];
	}
#line 7247 "./out/parser.cc"
    break;

  case 437: /* string_number_event: E_UNSIGNED  */
#line 3313 "parser.yy"
                   {
		Music *s = MY_MAKE_MUSIC ("StringNumberEvent", (yyloc));
		set_property (s, "string-number", yyvsp[0]);
		yyval = s->unprotect ();
	}
#line 7257 "./out/parser.cc"
    break;

  case 439: /* direction_less_event: EVENT_IDENTIFIER  */
#line 3322 "parser.yy"
                                {
		yyval = yyvsp[0];
	}
#line 7265 "./out/parser.cc"
    break;

  case 440: /* direction_less_event: tremolo_type  */
#line 3325 "parser.yy"
                        {
               Music *a = MY_MAKE_MUSIC ("TremoloEvent", (yyloc));
               set_property (a, "tremolo-type", yyvsp[0]);
               yyval = a->unprotect ();
        }
#line 7275 "./out/parser.cc"
    break;

  case 442: /* direction_reqd_event: gen_text_def  */
#line 3334 "parser.yy"
                     {
		yyval = yyvsp[0];
	}
#line 7283 "./out/parser.cc"
    break;

  case 443: /* direction_reqd_event: script_abbreviation  */
#line 3337 "parser.yy"
                              {
		SCM sym = ly_symbol2scm ("dash" + from_scm<std::string> (yyvsp[0]));
		SCM s = parser->lexer_->lookup_identifier_symbol (sym);
		Music *original = unsmob<Music> (s);
		if (original && original->is_mus_type ("post-event")) {
			Music *a = original->clone ();
			// origin will be set by post_event_nofinger
			yyval = a->unprotect ();
		} else {
			parser->parser_error ((yylsp[0]), _ ("expecting post-event as script definition"));
			yyval = SCM_UNSPECIFIED;
		}
	}
#line 7301 "./out/parser.cc"
    break;

  case 444: /* octave_check: %empty  */
#line 3353 "parser.yy"
             { yyval = SCM_EOL; }
#line 7307 "./out/parser.cc"
    break;

  case 445: /* octave_check: '=' quotes  */
#line 3354 "parser.yy"
                     { yyval = yyvsp[0]; }
#line 7313 "./out/parser.cc"
    break;

  case 446: /* quotes: %empty  */
#line 3359 "parser.yy"
        {
                yyval = SCM_INUM0;
	}
#line 7321 "./out/parser.cc"
    break;

  case 449: /* erroneous_quotes: quotes  */
#line 3368 "parser.yy"
               {
		if (scm_is_eq (SCM_INUM0, yyvsp[0]))
			yyval = SCM_UNDEFINED;
	}
#line 7330 "./out/parser.cc"
    break;

  case 450: /* sup_quotes: '\''  */
#line 3375 "parser.yy"
             {
		yyval = to_scm (1);
	}
#line 7338 "./out/parser.cc"
    break;

  case 451: /* sup_quotes: sup_quotes '\''  */
#line 3378 "parser.yy"
                          {
		yyval = scm_oneplus (yyvsp[-1]);
	}
#line 7346 "./out/parser.cc"
    break;

  case 452: /* sub_quotes: ','  */
#line 3384 "parser.yy"
            {
		yyval = to_scm (-1);
	}
#line 7354 "./out/parser.cc"
    break;

  case 453: /* sub_quotes: sub_quotes ','  */
#line 3387 "parser.yy"
                         {
		yyval = scm_oneminus (yyvsp[-1]);
	}
#line 7362 "./out/parser.cc"
    break;

  case 454: /* steno_pitch: NOTENAME_PITCH quotes  */
#line 3393 "parser.yy"
                              {
                if (!scm_is_eq (SCM_INUM0, yyvsp[0]))
                {
                        Pitch p = *unsmob<Pitch> (yyvsp[-1]);
                        p = p.transposed (Pitch (from_scm<int> (yyvsp[0]), 0));
                        yyval = p.smobbed_copy ();
                }
	}
#line 7375 "./out/parser.cc"
    break;

  case 455: /* steno_tonic_pitch: TONICNAME_PITCH quotes  */
#line 3408 "parser.yy"
                               {
                if (!scm_is_eq (SCM_INUM0, yyvsp[0]))
                {
                        Pitch p = *unsmob<Pitch> (yyvsp[-1]);
                        p = p.transposed (Pitch (from_scm<int> (yyvsp[0]), 0));
                        yyval = p.smobbed_copy ();
                }
	}
#line 7388 "./out/parser.cc"
    break;

  case 457: /* pitch: PITCH_IDENTIFIER quotes  */
#line 3420 "parser.yy"
                                  {
                if (!scm_is_eq (SCM_INUM0, yyvsp[0]))
                {
                        Pitch p = *unsmob<Pitch> (yyvsp[-1]);
                        p = p.transposed (Pitch (from_scm<int> (yyvsp[0]), 0));
                        yyval = p.smobbed_copy ();
                }
	}
#line 7401 "./out/parser.cc"
    break;

  case 460: /* gen_text_def: full_markup  */
#line 3436 "parser.yy"
                    {
		Music *t = MY_MAKE_MUSIC ("TextScriptEvent", (yyloc));
		set_property (t, "text", yyvsp[0]);
		yyval = t->unprotect ();
	}
#line 7411 "./out/parser.cc"
    break;

  case 461: /* gen_text_def: STRING  */
#line 3441 "parser.yy"
                 {
		Music *t = MY_MAKE_MUSIC ("TextScriptEvent", (yyloc));
		set_property (t, "text",
			make_simple_markup (yyvsp[0]));
		yyval = t->unprotect ();
	}
#line 7422 "./out/parser.cc"
    break;

  case 462: /* gen_text_def: SYMBOL  */
#line 3447 "parser.yy"
                 {
		// Flag a warning? could be unintentional
		Music *t = MY_MAKE_MUSIC ("TextScriptEvent", (yyloc));
		set_property (t, "text",
			make_simple_markup (yyvsp[0]));
		yyval = t->unprotect ();
	}
#line 7434 "./out/parser.cc"
    break;

  case 463: /* gen_text_def: embedded_scm  */
#line 3455 "parser.yy"
        {
		// Could be using this for every gen_text_def but for speed
		yyval = MAKE_SYNTAX (create_script, (yylsp[0]), yyvsp[0]);
	}
#line 7443 "./out/parser.cc"
    break;

  case 464: /* fingering: UNSIGNED  */
#line 3462 "parser.yy"
                 {
		Music *t = MY_MAKE_MUSIC ("FingeringEvent", (yyloc));
		set_property (t, "digit", yyvsp[0]);
		yyval = t->unprotect ();
	}
#line 7453 "./out/parser.cc"
    break;

  case 465: /* script_abbreviation: '^'  */
#line 3470 "parser.yy"
                        {
		yyval = scm_from_latin1_string ("Hat");
	}
#line 7461 "./out/parser.cc"
    break;

  case 466: /* script_abbreviation: '+'  */
#line 3473 "parser.yy"
                        {
		yyval = scm_from_latin1_string ("Plus");
	}
#line 7469 "./out/parser.cc"
    break;

  case 467: /* script_abbreviation: '-'  */
#line 3476 "parser.yy"
                        {
		yyval = scm_from_latin1_string ("Dash");
	}
#line 7477 "./out/parser.cc"
    break;

  case 468: /* script_abbreviation: '!'  */
#line 3479 "parser.yy"
                        {
		yyval = scm_from_latin1_string ("Bang");
	}
#line 7485 "./out/parser.cc"
    break;

  case 469: /* script_abbreviation: ">"  */
#line 3482 "parser.yy"
                        {
		yyval = scm_from_latin1_string ("Larger");
	}
#line 7493 "./out/parser.cc"
    break;

  case 470: /* script_abbreviation: '.'  */
#line 3485 "parser.yy"
                        {
		yyval = scm_from_latin1_string ("Dot");
	}
#line 7501 "./out/parser.cc"
    break;

  case 471: /* script_abbreviation: '_'  */
#line 3488 "parser.yy"
              {
		yyval = scm_from_latin1_string ("Underscore");
	}
#line 7509 "./out/parser.cc"
    break;

  case 472: /* script_dir: '_'  */
#line 3494 "parser.yy"
                { yyval = to_scm (DOWN); }
#line 7515 "./out/parser.cc"
    break;

  case 473: /* script_dir: '^'  */
#line 3495 "parser.yy"
                { yyval = to_scm (UP); }
#line 7521 "./out/parser.cc"
    break;

  case 474: /* script_dir: '-'  */
#line 3496 "parser.yy"
                { yyval = SCM_UNDEFINED; }
#line 7527 "./out/parser.cc"
    break;

  case 475: /* maybe_notemode_duration: %empty  */
#line 3500 "parser.yy"
        {
		yyval = SCM_UNDEFINED;
	}
#line 7535 "./out/parser.cc"
    break;

  case 476: /* maybe_notemode_duration: duration  */
#line 3503 "parser.yy"
                        {
		yyval = yyvsp[0];
		parser->default_duration_ = *unsmob<Duration> (yyval);
	}
#line 7544 "./out/parser.cc"
    break;

  case 477: /* optional_notemode_duration: maybe_notemode_duration  */
#line 3512 "parser.yy"
        {
		if (SCM_UNBNDP (yyval))
			yyval = parser->default_duration_.smobbed_copy ();
	}
#line 7553 "./out/parser.cc"
    break;

  case 478: /* steno_duration: UNSIGNED dots  */
#line 3519 "parser.yy"
                                {
		yyval = make_duration (yyvsp[-1], from_scm<int> (yyvsp[0]));
		if (SCM_UNBNDP (yyval))
		{
			parser->parser_error ((yylsp[-1]), _ ("not a duration"));
			yyval = Duration ().smobbed_copy ();
		}
	}
#line 7566 "./out/parser.cc"
    break;

  case 479: /* steno_duration: DURATION_IDENTIFIER dots  */
#line 3527 "parser.yy"
                                        {
		yyval = make_duration (yyvsp[-1], from_scm<int> (yyvsp[0]));
	}
#line 7574 "./out/parser.cc"
    break;

  case 480: /* duration: steno_duration multipliers  */
#line 3533 "parser.yy"
                                   {
		yyval = make_duration (yyvsp[-1], 0, yyvsp[0]);
	}
#line 7582 "./out/parser.cc"
    break;

  case 481: /* dots: %empty  */
#line 3539 "parser.yy"
                        {
		yyval = SCM_INUM0;
	}
#line 7590 "./out/parser.cc"
    break;

  case 482: /* dots: dots '.'  */
#line 3542 "parser.yy"
                   {
		yyval = scm_oneplus (yyvsp[-1]);
	}
#line 7598 "./out/parser.cc"
    break;

  case 485: /* multipliers: %empty  */
#line 3554 "parser.yy"
        {
		yyval = SCM_UNDEFINED;
	}
#line 7606 "./out/parser.cc"
    break;

  case 486: /* multipliers: multipliers '*' UNSIGNED  */
#line 3558 "parser.yy"
        {
		if (!SCM_UNBNDP (yyvsp[-2]))
			yyval = scm_product (yyvsp[-2], yyvsp[0]);
		else
			yyval = yyvsp[0];
	}
#line 7617 "./out/parser.cc"
    break;

  case 487: /* multipliers: multipliers '*' FRACTION  */
#line 3565 "parser.yy"
        {
		if (!SCM_UNBNDP (yyvsp[-2]))
			yyval = scm_product (yyvsp[-2], scm_divide (scm_car (yyvsp[0]),
							  scm_cdr (yyvsp[0])));
		else
			yyval = scm_divide (scm_car (yyvsp[0]), scm_cdr (yyvsp[0]));
	}
#line 7629 "./out/parser.cc"
    break;

  case 488: /* multipliers: multipliers '*' multiplier_scm  */
#line 3573 "parser.yy"
        {
		if (scm_is_false (Lily::scale_p (yyvsp[0])))
		{
			parser->parser_error ((yylsp[0]), _ ("not a multiplier"));
		} else if (SCM_UNBNDP (yyvsp[-2]))
			yyval = Lily::scale_to_factor (yyvsp[0]);
		else
			yyval = scm_product (yyvsp[-2], Lily::scale_to_factor (yyvsp[0]));
	}
#line 7643 "./out/parser.cc"
    break;

  case 489: /* tremolo_type: ':'  */
#line 3585 "parser.yy"
                {
		yyval = to_scm (parser->default_tremolo_type_);
	}
#line 7651 "./out/parser.cc"
    break;

  case 490: /* tremolo_type: ':' UNSIGNED  */
#line 3588 "parser.yy"
                       {
		if (SCM_UNBNDP (make_duration (yyvsp[0]))) {
			parser->parser_error ((yylsp[0]), _ ("not a duration"));
			yyval = to_scm (parser->default_tremolo_type_);
		} else {
			yyval = yyvsp[0];
			parser->default_tremolo_type_ = from_scm<int> (yyvsp[0]);
		}
	}
#line 7665 "./out/parser.cc"
    break;

  case 495: /* bass_number: embedded_scm_bare  */
#line 3605 "parser.yy"
        {
		// as an integer, it needs to be non-negative, and otherwise
		// it needs to be suitable as a markup.
		if (scm_is_integer (yyvsp[0])
		    ? scm_is_true (scm_negative_p (yyvsp[0]))
		    : !Text_interface::is_markup (yyvsp[0]))
		{
			parser->parser_error ((yylsp[0]), _ ("bass number expected"));
			yyval = SCM_INUM0;
		}
	}
#line 7681 "./out/parser.cc"
    break;

  case 496: /* bass_figure: "_"  */
#line 3619 "parser.yy"
                     {
		Music *bfr = MY_MAKE_MUSIC ("BassFigureEvent", (yyloc));
		yyval = bfr->unprotect ();
	}
#line 7690 "./out/parser.cc"
    break;

  case 497: /* bass_figure: bass_number  */
#line 3623 "parser.yy"
                       {
		Music *bfr = MY_MAKE_MUSIC ("BassFigureEvent", (yyloc));
		yyval = bfr->self_scm ();

		if (scm_is_number (yyvsp[0]))
			set_property (bfr, "figure", yyvsp[0]);
		else if (Text_interface::is_markup (yyvsp[0]))
			set_property (bfr, "text", yyvsp[0]);

		bfr->unprotect ();
	}
#line 7706 "./out/parser.cc"
    break;

  case 498: /* bass_figure: bass_figure ']'  */
#line 3634 "parser.yy"
                          {
		yyval = yyvsp[-1];
		set_property (unsmob<Music> (yyvsp[-1]), "bracket-stop", SCM_BOOL_T);
	}
#line 7715 "./out/parser.cc"
    break;

  case 499: /* bass_figure: bass_figure FIGURE_ALTERATION_EXPR  */
#line 3638 "parser.yy"
                                             {
		Music *m = unsmob<Music> (yyvsp[-1]);

		if (scm_is_number (get_property (m, "alteration")))
			m->warning (_f ("Dropping surplus alteration symbols for bass figure."));
		else {
			auto alter_expr = from_scm<std::string> (yyvsp[0]);
			Rational alter (0);
			bool bracket = false;

			for (string::iterator it=alter_expr.begin(); it != alter_expr.end (); it++)
			{
				int c = *it & 0xff;

				/* The friendly lexer guarantees that '[' has its matching ']',
				   so we don't have to check here. */
				if (c == '[')		bracket = true;

				/* "!" resets the counter: we mimic this traditional (pre-2.23.4) behavior. */
				else if (c == '!')	alter = 0;
				else if (c == '+')	alter += SHARP_ALTERATION;
				else if (c == '-')	alter += FLAT_ALTERATION;
			}

			set_property (m, "alteration", to_scm (alter));
			if (bracket)
				set_property (m, "alteration-bracket", SCM_BOOL_T);
		}
	}
#line 7749 "./out/parser.cc"
    break;

  case 500: /* bass_figure: bass_figure figured_bass_modification  */
#line 3667 "parser.yy"
                                                 {
		Music *m = unsmob<Music> (yyvsp[-1]);
		set_property (m, yyvsp[0], SCM_BOOL_T);
	}
#line 7758 "./out/parser.cc"
    break;

  case 501: /* figured_bass_modification: "\\+"  */
#line 3675 "parser.yy"
                        {
		yyval = ly_symbol2scm ("augmented");
	}
#line 7766 "./out/parser.cc"
    break;

  case 502: /* figured_bass_modification: "\\!"  */
#line 3678 "parser.yy"
                        {
		yyval = ly_symbol2scm ("no-continuation");
	}
#line 7774 "./out/parser.cc"
    break;

  case 503: /* figured_bass_modification: '/'  */
#line 3681 "parser.yy"
                        {
		yyval = ly_symbol2scm ("diminished");
	}
#line 7782 "./out/parser.cc"
    break;

  case 504: /* figured_bass_modification: "\\\\"  */
#line 3684 "parser.yy"
                      {
		yyval = ly_symbol2scm ("augmented-slash");
	}
#line 7790 "./out/parser.cc"
    break;

  case 505: /* br_bass_figure: bass_figure  */
#line 3690 "parser.yy"
                    {
		yyval = yyvsp[0];
	}
#line 7798 "./out/parser.cc"
    break;

  case 506: /* br_bass_figure: '[' bass_figure  */
#line 3693 "parser.yy"
                          {
		yyval = yyvsp[0];
		set_property (unsmob<Music> (yyval), "bracket-start", SCM_BOOL_T);
	}
#line 7807 "./out/parser.cc"
    break;

  case 507: /* figure_list: %empty  */
#line 3700 "parser.yy"
                        {
		yyval = SCM_EOL;
	}
#line 7815 "./out/parser.cc"
    break;

  case 508: /* figure_list: figure_list br_bass_figure  */
#line 3703 "parser.yy"
                                     {
		yyval = scm_cons (yyvsp[0], yyvsp[-1]);
	}
#line 7823 "./out/parser.cc"
    break;

  case 509: /* optional_rest: %empty  */
#line 3709 "parser.yy"
               { yyval = SCM_BOOL_F; }
#line 7829 "./out/parser.cc"
    break;

  case 510: /* optional_rest: "\\rest"  */
#line 3710 "parser.yy"
               { yyval = SCM_BOOL_T; }
#line 7835 "./out/parser.cc"
    break;

  case 511: /* pitch_or_music: pitch exclamations questions octave_check maybe_notemode_duration erroneous_quotes optional_rest post_events  */
#line 3718 "parser.yy"
                                                                                                                     {
		if (!parser->lexer_->is_note_state ())
			parser->parser_error ((yylsp[-7]), _ ("have to be in Note mode for notes"));
		if (!SCM_UNBNDP (yyvsp[-2]))
		{
			// It's possible to get here without a
			// duration, like when there is no
			// octave_check but a question mark.  But we
			// point out the most frequent error of an
			// interspersed duration specifically
			if (!SCM_UNBNDP (yyvsp[-3]))
				parser->parser_error ((yylsp[-2]), _ ("octave marks must precede duration"));
			else
				parser->parser_error ((yylsp[-2]), _ ("badly placed octave marks"));
			// Try sorting the quotes to where they likely belong
			if (scm_is_number (yyvsp[-4])) {
				yyvsp[-4] = scm_sum (yyvsp[-4], yyvsp[-2]);
			} else {
				yyvsp[-7] = unsmob<Pitch> (yyvsp[-7])->transposed
					(Pitch (from_scm<int> (yyvsp[-2]), 0)).smobbed_copy ();
			}
		}

		if (!SCM_UNBNDP (yyvsp[-6])
                    || !SCM_UNBNDP (yyvsp[-5])
                    || scm_is_number (yyvsp[-4])
                    || !SCM_UNBNDP (yyvsp[-3])
		    || scm_is_true (yyvsp[-1])
		    || scm_is_pair (yyvsp[0]))
		{
			Music *n = 0;
			if (scm_is_true (yyvsp[-1]))
				n = MY_MAKE_MUSIC ("RestEvent", (yyloc));
			else
				n = MY_MAKE_MUSIC ("NoteEvent", (yyloc));

			set_property (n, "pitch", yyvsp[-7]);
			if (SCM_UNBNDP (yyvsp[-3]))
				set_property (n, "duration",
						 parser->default_duration_.smobbed_copy ());
			else
				set_property (n, "duration", yyvsp[-3]);

			if (scm_is_number (yyvsp[-4]))
			{
				int q = from_scm<int> (yyvsp[-4]);
				set_property (n, "absolute-octave", to_scm (q-1));
			}

			if (from_scm<bool> (yyvsp[-5]))
				set_property (n, "cautionary", SCM_BOOL_T);
			if (from_scm<bool> (yyvsp[-6]) || from_scm<bool> (yyvsp[-5]))
				set_property (n, "force-accidental", SCM_BOOL_T);
			if (scm_is_pair (yyvsp[0]))
				set_property (n, "articulations",
						 scm_reverse_x (yyvsp[0], SCM_EOL));
			yyval = n->unprotect ();
		}
	}
#line 7899 "./out/parser.cc"
    break;

  case 512: /* pitch_or_music: new_chord post_events  */
#line 3777 "parser.yy"
                                {
		if (!parser->lexer_->is_chord_state ())
                        parser->parser_error ((yylsp[-1]), _ ("have to be in Chord mode for chords"));
		if (scm_is_pair (yyvsp[0])) {
			if (unsmob<Pitch> (yyvsp[-1]))
				yyvsp[-1] = make_chord_elements ((yylsp[-1]),
							  yyvsp[-1],
							  parser->default_duration_.smobbed_copy (),
							  SCM_EOL);

			SCM elts = ly_append (yyvsp[-1], scm_reverse_x (yyvsp[0], SCM_EOL));

			yyval = MAKE_SYNTAX (event_chord, (yylsp[-1]), elts);
		} else if (!unsmob<Pitch> (yyvsp[-1]))
			yyval = MAKE_SYNTAX (event_chord, (yylsp[-1]), yyvsp[-1]);
		// A mere pitch drops through.
	}
#line 7921 "./out/parser.cc"
    break;

  case 513: /* simple_element: DRUM_PITCH optional_notemode_duration  */
#line 3797 "parser.yy"
                                              {
		Music *n = MY_MAKE_MUSIC ("NoteEvent", (yyloc));
		set_property (n, "duration", yyvsp[0]);
		set_property (n, "drum-type", yyvsp[-1]);

		yyval = n->unprotect ();
	}
#line 7933 "./out/parser.cc"
    break;

  case 514: /* simple_element: RESTNAME optional_notemode_duration  */
#line 3804 "parser.yy"
                                                        {
		Music *ev = 0;
 		if (from_scm<std::string> (yyvsp[-1]) == "s") {
			/* Space */
			ev = MY_MAKE_MUSIC ("SkipEvent", (yyloc));
		  }
		else {
			ev = MY_MAKE_MUSIC ("RestEvent", (yyloc));

		    }
		set_property (ev, "duration", yyvsp[0]);
 		yyval = ev->unprotect ();
	}
#line 7951 "./out/parser.cc"
    break;

  case 515: /* lyric_element: full_markup  */
#line 3820 "parser.yy"
                    {
		if (!parser->lexer_->is_lyric_state ())
			parser->parser_error ((yylsp[0]), _ ("markup outside of text script or \\lyricmode"));
		yyval = yyvsp[0];
	}
#line 7961 "./out/parser.cc"
    break;

  case 516: /* lyric_element: SYMBOL  */
#line 3825 "parser.yy"
                 {
		if (!parser->lexer_->is_lyric_state ())
			parser->parser_error ((yylsp[0]), _f ("not a note name: %s", from_scm<std::string> (yyvsp[0])));
		yyval = yyvsp[0];
	}
#line 7971 "./out/parser.cc"
    break;

  case 517: /* lyric_element: STRING  */
#line 3830 "parser.yy"
                 {
		if (!parser->lexer_->is_lyric_state ())
			parser->parser_error ((yylsp[0]), _ ("string outside of text script or \\lyricmode"));
		yyval = yyvsp[0];
	}
#line 7981 "./out/parser.cc"
    break;

  case 519: /* lyric_element_music: lyric_element optional_notemode_duration post_events  */
#line 3839 "parser.yy"
                                                             {
		yyval = MAKE_SYNTAX (lyric_event, (yyloc), yyvsp[-2], yyvsp[-1]);
		if (scm_is_pair (yyvsp[0]))
			set_property
				(unsmob<Music> (yyval), "articulations", scm_reverse_x (yyvsp[0], SCM_EOL));
	}
#line 7992 "./out/parser.cc"
    break;

  case 520: /* new_chord: steno_tonic_pitch maybe_notemode_duration  */
#line 3849 "parser.yy"
                                                    {
		if (SCM_UNBNDP (yyvsp[0]))
			yyval = yyvsp[-1];
		else
			yyval = make_chord_elements ((yyloc), yyvsp[-1], yyvsp[0], SCM_EOL);
	}
#line 8003 "./out/parser.cc"
    break;

  case 521: /* new_chord: steno_tonic_pitch optional_notemode_duration chord_separator chord_items  */
#line 3855 "parser.yy"
                                                                                   {
		SCM its = scm_reverse_x (yyvsp[0], SCM_EOL);
		yyval = make_chord_elements ((yyloc), yyvsp[-3], yyvsp[-2], scm_cons (yyvsp[-1], its));
	}
#line 8012 "./out/parser.cc"
    break;

  case 522: /* chord_items: %empty  */
#line 3862 "parser.yy"
             {
		yyval = SCM_EOL;
	}
#line 8020 "./out/parser.cc"
    break;

  case 523: /* chord_items: chord_items chord_item  */
#line 3865 "parser.yy"
                                 {
		yyval = scm_cons (yyvsp[0], yyval);
	}
#line 8028 "./out/parser.cc"
    break;

  case 524: /* chord_separator: ":"  */
#line 3871 "parser.yy"
                    {
		yyval = ly_symbol2scm ("chord-colon");
	}
#line 8036 "./out/parser.cc"
    break;

  case 525: /* chord_separator: "^"  */
#line 3874 "parser.yy"
                      {
		yyval = ly_symbol2scm ("chord-caret");
	}
#line 8044 "./out/parser.cc"
    break;

  case 526: /* chord_separator: "/" steno_tonic_pitch  */
#line 3877 "parser.yy"
                                        {
 		yyval = ly_list (ly_symbol2scm ("chord-slash"), yyvsp[0]);
	}
#line 8052 "./out/parser.cc"
    break;

  case 527: /* chord_separator: "/+" steno_tonic_pitch  */
#line 3880 "parser.yy"
                                       {
		yyval = ly_list (ly_symbol2scm ("chord-bass"), yyvsp[0]);
	}
#line 8060 "./out/parser.cc"
    break;

  case 528: /* chord_item: chord_separator  */
#line 3886 "parser.yy"
                        {
		yyval = yyvsp[0];
	}
#line 8068 "./out/parser.cc"
    break;

  case 529: /* chord_item: step_numbers  */
#line 3889 "parser.yy"
                       {
		yyval = scm_reverse_x (yyvsp[0], SCM_EOL);
	}
#line 8076 "./out/parser.cc"
    break;

  case 530: /* chord_item: CHORD_MODIFIER  */
#line 3892 "parser.yy"
                          {
		yyval = yyvsp[0];
	}
#line 8084 "./out/parser.cc"
    break;

  case 531: /* step_numbers: step_number  */
#line 3898 "parser.yy"
                    { yyval = scm_cons (yyvsp[0], SCM_EOL); }
#line 8090 "./out/parser.cc"
    break;

  case 532: /* step_numbers: step_numbers '.' step_number  */
#line 3899 "parser.yy"
                                       {
		yyval = scm_cons (yyvsp[0], yyval);
	}
#line 8098 "./out/parser.cc"
    break;

  case 533: /* step_number: UNSIGNED  */
#line 3905 "parser.yy"
                 {
		yyval = make_chord_step (yyvsp[0], 0);
        }
#line 8106 "./out/parser.cc"
    break;

  case 534: /* step_number: UNSIGNED '+'  */
#line 3908 "parser.yy"
                       {
		yyval = make_chord_step (yyvsp[-1], SHARP_ALTERATION);
	}
#line 8114 "./out/parser.cc"
    break;

  case 535: /* step_number: UNSIGNED "-"  */
#line 3911 "parser.yy"
                               {
		yyval = make_chord_step (yyvsp[-1], FLAT_ALTERATION);
	}
#line 8122 "./out/parser.cc"
    break;

  case 536: /* tempo_range: exact_unsigned_number  */
#line 3917 "parser.yy"
                              {
		yyval = yyvsp[0];
	}
#line 8130 "./out/parser.cc"
    break;

  case 537: /* tempo_range: exact_unsigned_number '-' exact_unsigned_number  */
#line 3920 "parser.yy"
                                                          {
		yyval = scm_cons (yyvsp[-2], yyvsp[0]);
	}
#line 8138 "./out/parser.cc"
    break;

  case 538: /* number_expression: number_expression '+' number_term  */
#line 3932 "parser.yy"
                                          {
		yyval = scm_sum (yyvsp[-2], yyvsp[0]);
	}
#line 8146 "./out/parser.cc"
    break;

  case 539: /* number_expression: number_expression '-' number_term  */
#line 3935 "parser.yy"
                                            {
		yyval = scm_difference (yyvsp[-2], yyvsp[0]);
	}
#line 8154 "./out/parser.cc"
    break;

  case 541: /* number_term: number_factor  */
#line 3942 "parser.yy"
                      {
		yyval = yyvsp[0];
	}
#line 8162 "./out/parser.cc"
    break;

  case 542: /* number_term: number_factor '*' number_factor  */
#line 3945 "parser.yy"
                                          {
		yyval = scm_product (yyvsp[-2], yyvsp[0]);
	}
#line 8170 "./out/parser.cc"
    break;

  case 543: /* number_term: number_factor '/' number_factor  */
#line 3948 "parser.yy"
                                          {
		yyval = scm_divide (yyvsp[-2], yyvsp[0]);
	}
#line 8178 "./out/parser.cc"
    break;

  case 544: /* number_factor: '-' number_factor  */
#line 3954 "parser.yy"
                           { /* %prec UNARY_MINUS */
		yyval = scm_difference (yyvsp[0], SCM_UNDEFINED);
	}
#line 8186 "./out/parser.cc"
    break;

  case 548: /* bare_number_common: REAL NUMBER_IDENTIFIER  */
#line 3964 "parser.yy"
        {
		yyval = scm_product (yyvsp[-1], yyvsp[0]);
	}
#line 8194 "./out/parser.cc"
    break;

  case 551: /* bare_number: UNSIGNED NUMBER_IDENTIFIER  */
#line 3972 "parser.yy"
                                        {
		yyval = scm_product (yyvsp[-1], yyvsp[0]);
	}
#line 8202 "./out/parser.cc"
    break;

  case 553: /* exact_unsigned_number: NUMBER_IDENTIFIER  */
#line 3980 "parser.yy"
        {
		if (!scm_is_exact (yyvsp[0])
		    || scm_is_true (scm_negative_p (yyvsp[0])))
		{
			parser->parser_error ((yylsp[0]), _("not an exact unsigned number"));
			yyval = SCM_INUM0;
		}
	}
#line 8215 "./out/parser.cc"
    break;

  case 554: /* exact_unsigned_number: embedded_scm  */
#line 3989 "parser.yy"
        {
		if (!scm_is_number (yyvsp[0])
		    || !scm_is_exact (yyvsp[0])
		    || scm_is_true (scm_negative_p (yyvsp[0])))
		{
			parser->parser_error ((yylsp[0]), _("not an exact unsigned number"));
			yyval = SCM_INUM0;
		}
	}
#line 8229 "./out/parser.cc"
    break;

  case 556: /* unsigned_integer: NUMBER_IDENTIFIER  */
#line 4003 "parser.yy"
        {
		if (!scm_is_integer (yyvsp[0])
		    || scm_is_true (scm_negative_p (yyvsp[0])))
		{
			parser->parser_error ((yylsp[0]), _("not an unsigned integer"));
			yyval = SCM_INUM0;
		}
	}
#line 8242 "./out/parser.cc"
    break;

  case 557: /* unsigned_integer: embedded_scm  */
#line 4012 "parser.yy"
        {
		if (!scm_is_integer (yyvsp[0])
		    || scm_is_true (scm_negative_p (yyvsp[0])))
		{
			parser->parser_error ((yylsp[0]), _("not an unsigned integer"));
			yyval = SCM_INUM0;
		}
	}
#line 8255 "./out/parser.cc"
    break;

  case 558: /* exclamations: %empty  */
#line 4023 "parser.yy"
                { yyval = SCM_UNDEFINED; }
#line 8261 "./out/parser.cc"
    break;

  case 559: /* exclamations: exclamations '!'  */
#line 4025 "parser.yy"
        {
                if (SCM_UNBNDP (yyvsp[-1]))
                        yyval = SCM_BOOL_T;
                else
                        yyval = scm_not (yyvsp[-1]);
        }
#line 8272 "./out/parser.cc"
    break;

  case 560: /* questions: %empty  */
#line 4040 "parser.yy"
        { yyval = SCM_UNDEFINED; }
#line 8278 "./out/parser.cc"
    break;

  case 561: /* questions: questions '?'  */
#line 4042 "parser.yy"
        {
                if (SCM_UNBNDP (yyvsp[-1]))
                        yyval = SCM_BOOL_T;
                else
                        yyval = scm_not (yyvsp[-1]);
        }
#line 8289 "./out/parser.cc"
    break;

  case 562: /* $@11: %empty  */
#line 4052 "parser.yy"
                { parser->lexer_->push_markup_state (); }
#line 8295 "./out/parser.cc"
    break;

  case 563: /* full_markup_list: "\\markuplist" $@11 markup_list  */
#line 4053 "parser.yy"
                    {
		yyval = yyvsp[0];
		parser->lexer_->pop_state ();
	}
#line 8304 "./out/parser.cc"
    break;

  case 564: /* markup_mode: "\\markup"  */
#line 4061 "parser.yy"
        {
		parser->lexer_->push_markup_state ();
	}
#line 8312 "./out/parser.cc"
    break;

  case 565: /* markup_mode_word: markup_mode markup_word  */
#line 4077 "parser.yy"
        {
		yyval = yyvsp[0];
		parser->lexer_->pop_state ();
	}
#line 8321 "./out/parser.cc"
    break;

  case 566: /* full_markup: markup_mode markup_top  */
#line 4085 "parser.yy"
                               {
		yyval = yyvsp[0];
		parser->lexer_->pop_state ();
	}
#line 8330 "./out/parser.cc"
    break;

  case 567: /* full_markup: markup_mode_word  */
#line 4090 "parser.yy"
        {
		yyval = make_simple_markup (yyvsp[0]);
	}
#line 8338 "./out/parser.cc"
    break;

  case 568: /* partial_markup: markup_mode markup_partial_function "\\etc"  */
#line 4097 "parser.yy"
        {
		yyval = MAKE_SYNTAX (partial_markup, (yylsp[-1]), yyvsp[-1]);
		parser->lexer_->pop_state ();
	}
#line 8347 "./out/parser.cc"
    break;

  case 569: /* markup_top: markup_list  */
#line 4104 "parser.yy"
                    {
		yyval = ly_list (Lily::line_markup,  yyvsp[0]);
	}
#line 8355 "./out/parser.cc"
    break;

  case 570: /* markup_top: markup_head_1_list simple_markup  */
#line 4108 "parser.yy"
        {
		yyval = scm_car (MAKE_SYNTAX (composed_markup_list,
					   (yylsp[0]), yyvsp[-1], ly_list (yyvsp[0])));
	}
#line 8364 "./out/parser.cc"
    break;

  case 571: /* markup_top: simple_markup_noword  */
#line 4112 "parser.yy"
                               {
		yyval = yyvsp[0];
	}
#line 8372 "./out/parser.cc"
    break;

  case 572: /* $@12: %empty  */
#line 4119 "parser.yy"
        {
		if (Text_interface::is_markup (yyvsp[0]))
			MYBACKUP (MARKUP_IDENTIFIER, yyvsp[0], (yylsp[0]));
		else if (Text_interface::is_markup_list (yyvsp[0]))
			MYBACKUP (MARKUPLIST_IDENTIFIER, yyvsp[0], (yylsp[0]));
		else if (scm_is_eq (yyvsp[0], SCM_UNSPECIFIED))
			MYBACKUP (MARKUPLIST_IDENTIFIER, SCM_EOL, (yylsp[0]));
		else {
			parser->parser_error ((yylsp[0]), _ ("not a markup"));
			MYBACKUP (MARKUP_IDENTIFIER, scm_string (SCM_EOL), (yylsp[0]));
		}
	}
#line 8389 "./out/parser.cc"
    break;

  case 574: /* markup_list: markup_composed_list  */
#line 4135 "parser.yy"
                             {
		yyval = yyvsp[0];
	}
#line 8397 "./out/parser.cc"
    break;

  case 576: /* markup_uncomposed_list: markup_braced_list  */
#line 4142 "parser.yy"
                           {
		yyval = yyvsp[0];
	}
#line 8405 "./out/parser.cc"
    break;

  case 577: /* markup_uncomposed_list: markup_command_list  */
#line 4145 "parser.yy"
                              {
		yyval = ly_list (yyvsp[0]);
	}
#line 8413 "./out/parser.cc"
    break;

  case 578: /* markup_uncomposed_list: markup_scm MARKUPLIST_IDENTIFIER  */
#line 4149 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 8421 "./out/parser.cc"
    break;

  case 579: /* $@13: %empty  */
#line 4152 "parser.yy"
                     {
		parser->lexer_->push_note_state ();
	}
#line 8429 "./out/parser.cc"
    break;

  case 580: /* markup_uncomposed_list: "\\score-lines" $@13 '{' score_body '}'  */
#line 4154 "parser.yy"
                             {
		Score *sc = unsmob<Score> (yyvsp[-1]);
		sc->origin ()->set_spot ((yyloc));
		if (sc->defs_.empty ()) {
			Output_def *od = get_layout (parser);
			sc->add_output_def (od);
			od->unprotect ();
		}
		yyval = ly_list (ly_list (Lily::score_lines_markup_list, yyvsp[-1]));
		parser->lexer_->pop_state ();
	}
#line 8445 "./out/parser.cc"
    break;

  case 581: /* markup_composed_list: markup_head_1_list markup_uncomposed_list  */
#line 4168 "parser.yy"
                                                  {
		yyval = MAKE_SYNTAX (composed_markup_list,
				  (yylsp[0]), yyvsp[-1], yyvsp[0]);
	}
#line 8454 "./out/parser.cc"
    break;

  case 582: /* markup_braced_list: '{' markup_braced_list_body '}'  */
#line 4175 "parser.yy"
                                        {
		yyval = scm_reverse_x (yyvsp[-1], SCM_EOL);
	}
#line 8462 "./out/parser.cc"
    break;

  case 583: /* markup_braced_list_body: %empty  */
#line 4181 "parser.yy"
                        {  yyval = SCM_EOL; }
#line 8468 "./out/parser.cc"
    break;

  case 584: /* markup_braced_list_body: markup_braced_list_body markup  */
#line 4182 "parser.yy"
                                         {
		yyval = scm_cons (yyvsp[0], yyvsp[-1]);
	}
#line 8476 "./out/parser.cc"
    break;

  case 585: /* markup_braced_list_body: markup_braced_list_body markup_list  */
#line 4185 "parser.yy"
                                              {
		yyval = Srfi_1::append_reverse (yyvsp[0], yyvsp[-1]);
	}
#line 8484 "./out/parser.cc"
    break;

  case 586: /* markup_command_list: MARKUP_LIST_FUNCTION markup_command_list_arguments  */
#line 4191 "parser.yy"
                                                           {
	  yyval = scm_cons (yyvsp[-1], scm_reverse_x (yyvsp[0], SCM_EOL));
	}
#line 8492 "./out/parser.cc"
    break;

  case 587: /* markup_command_basic_arguments: "markup-list?" markup_command_list_arguments markup_list  */
#line 4197 "parser.yy"
                                                                     {
		yyval = scm_cons (yyvsp[0], yyvsp[-1]);
	}
#line 8500 "./out/parser.cc"
    break;

  case 588: /* markup_command_basic_arguments: "scheme?" markup_command_list_arguments embedded_scm  */
#line 4200 "parser.yy"
                                                                {
		yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-1], yyvsp[-2]);
	}
#line 8508 "./out/parser.cc"
    break;

  case 589: /* $@14: %empty  */
#line 4211 "parser.yy"
                                                       {
		parser->lexer_->push_note_state ();
	}
#line 8516 "./out/parser.cc"
    break;

  case 590: /* markup_command_basic_arguments: "scheme?" markup_command_list_arguments '{' $@14 embedded_lilypond '}'  */
#line 4213 "parser.yy"
                                {
		yyval = SCM_UNDEFINED;
		if (scm_is_false (ly_call (yyvsp[-5], yyvsp[-1]))) {
			// The conversion to a note event needs the lexer to be still
			// in note state.
			SCM maybe_music = make_music_from_simple (parser, (yylsp[-1]), yyvsp[-1]);
			if (unsmob<Music> (maybe_music)
			    && scm_is_true (ly_call (yyvsp[-5], maybe_music))) {
				yyval = scm_cons (maybe_music, yyvsp[-4]);
				if (Duration *dur = unsmob<Duration> (yyvsp[-1])) {
					parser->default_duration_ = *dur;
				}
			}
		}
		if (SCM_UNBNDP (yyval)) {
			yyval = check_scheme_arg (parser, (yylsp[-1]), yyvsp[-1], yyvsp[-4], yyvsp[-5]);
		}
		parser->lexer_->pop_state ();
	}
#line 8540 "./out/parser.cc"
    break;

  case 591: /* markup_command_basic_arguments: "scheme?" markup_command_list_arguments mode_changed_music  */
#line 4232 "parser.yy"
                                                                      {
		yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-1], yyvsp[-2]);
	}
#line 8548 "./out/parser.cc"
    break;

  case 592: /* markup_command_basic_arguments: "scheme?" markup_command_list_arguments MUSIC_IDENTIFIER  */
#line 4235 "parser.yy"
                                                                    {
		yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-1], yyvsp[-2]);
	}
#line 8556 "./out/parser.cc"
    break;

  case 593: /* markup_command_basic_arguments: "scheme?" markup_command_list_arguments STRING  */
#line 4238 "parser.yy"
                                                          {
		yyval = check_scheme_arg (parser, (yylsp[0]), yyvsp[0], yyvsp[-1], yyvsp[-2]);
	}
#line 8564 "./out/parser.cc"
    break;

  case 594: /* markup_command_basic_arguments: EXPECT_NO_MORE_ARGS  */
#line 4241 "parser.yy"
                              {
		yyval = SCM_EOL;
	}
#line 8572 "./out/parser.cc"
    break;

  case 595: /* markup_command_list_arguments: markup_command_basic_arguments  */
#line 4247 "parser.yy"
                                       { yyval = yyvsp[0]; }
#line 8578 "./out/parser.cc"
    break;

  case 596: /* markup_command_list_arguments: "markup?" markup_command_list_arguments markup  */
#line 4248 "parser.yy"
                                                             {
	  yyval = scm_cons (yyvsp[0], yyvsp[-1]);
	}
#line 8586 "./out/parser.cc"
    break;

  case 597: /* markup_partial_function: MARKUP_FUNCTION markup_arglist_partial  */
#line 4255 "parser.yy"
        {
		yyval = ly_list (scm_cons (yyvsp[-1], scm_reverse_x (yyvsp[0], SCM_EOL)));
	}
#line 8594 "./out/parser.cc"
    break;

  case 598: /* markup_partial_function: markup_head_1_list MARKUP_FUNCTION markup_arglist_partial  */
#line 4259 "parser.yy"
        {
		yyval = scm_cons (scm_cons (yyvsp[-1], scm_reverse_x (yyvsp[0], SCM_EOL)),
			       yyvsp[-2]);
	}
#line 8603 "./out/parser.cc"
    break;

  case 599: /* markup_arglist_partial: "markup?" markup_arglist_partial  */
#line 4267 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 8611 "./out/parser.cc"
    break;

  case 600: /* markup_arglist_partial: "markup-list?" markup_arglist_partial  */
#line 4271 "parser.yy"
        {
		yyval= yyvsp[0];
	}
#line 8619 "./out/parser.cc"
    break;

  case 601: /* markup_arglist_partial: "scheme?" markup_arglist_partial  */
#line 4275 "parser.yy"
        {
		yyval= yyvsp[0];
	}
#line 8627 "./out/parser.cc"
    break;

  case 602: /* markup_arglist_partial: "markup?" markup_command_list_arguments  */
#line 4281 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 8635 "./out/parser.cc"
    break;

  case 603: /* markup_arglist_partial: "markup-list?" markup_command_list_arguments  */
#line 4285 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 8643 "./out/parser.cc"
    break;

  case 604: /* markup_arglist_partial: "scheme?" markup_command_list_arguments  */
#line 4289 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 8651 "./out/parser.cc"
    break;

  case 605: /* markup_head_1_item: MARKUP_FUNCTION "markup?" markup_command_list_arguments  */
#line 4295 "parser.yy"
                                                                    {
	  yyval = scm_cons (yyvsp[-2], scm_reverse_x (yyvsp[0], SCM_EOL));
	}
#line 8659 "./out/parser.cc"
    break;

  case 606: /* markup_head_1_list: markup_head_1_item  */
#line 4301 "parser.yy"
                                {
		yyval = ly_list (yyvsp[0]);
	}
#line 8667 "./out/parser.cc"
    break;

  case 607: /* markup_head_1_list: markup_head_1_list markup_head_1_item  */
#line 4304 "parser.yy"
                                                {
		yyval = scm_cons (yyvsp[0], yyvsp[-1]);
	}
#line 8675 "./out/parser.cc"
    break;

  case 610: /* simple_markup: markup_word  */
#line 4316 "parser.yy"
        {
		yyval = make_simple_markup (yyvsp[0]);
	}
#line 8683 "./out/parser.cc"
    break;

  case 612: /* $@15: %empty  */
#line 4323 "parser.yy"
              {
		parser->lexer_->push_note_state ();
	}
#line 8691 "./out/parser.cc"
    break;

  case 613: /* simple_markup_noword: "\\score" $@15 '{' score_body '}'  */
#line 4325 "parser.yy"
                             {
		Score *sc = unsmob<Score> (yyvsp[-1]);
		sc->origin ()->set_spot ((yyloc));
		if (sc->defs_.empty ()) {
			Output_def *od = get_layout (parser);
			sc->add_output_def (od);
			od->unprotect ();
		}
		yyval = ly_list (Lily::score_markup, yyvsp[-1]);
		parser->lexer_->pop_state ();
	}
#line 8707 "./out/parser.cc"
    break;

  case 614: /* simple_markup_noword: MARKUP_FUNCTION markup_command_basic_arguments  */
#line 4336 "parser.yy"
                                                         {
		yyval = scm_cons (yyvsp[-1], scm_reverse_x (yyvsp[0], SCM_EOL));
	}
#line 8715 "./out/parser.cc"
    break;

  case 615: /* simple_markup_noword: markup_scm MARKUP_IDENTIFIER  */
#line 4340 "parser.yy"
        {
		yyval = yyvsp[0];
	}
#line 8723 "./out/parser.cc"
    break;

  case 616: /* markup: markup_head_1_list simple_markup  */
#line 4347 "parser.yy"
        {
		yyval = scm_car (MAKE_SYNTAX (composed_markup_list,
					   (yylsp[0]), yyvsp[-1], ly_list (yyvsp[0])));
	}
#line 8732 "./out/parser.cc"
    break;

  case 617: /* markup: simple_markup  */
#line 4351 "parser.yy"
                        {
		yyval = yyvsp[0];
	}
#line 8740 "./out/parser.cc"
    break;


#line 8744 "./out/parser.cc"

      default: break;
    }
  /* User semantic actions sometimes alter yychar, and that requires
     that yytoken be updated with the new translation.  We take the
     approach of translating immediately before every use of yytoken.
     One alternative is translating here after every semantic action,
     but that translation would be missed if the semantic action invokes
     YYABORT, YYACCEPT, or YYERROR immediately after altering yychar or
     if it invokes YYBACKUP.  In the case of YYABORT or YYACCEPT, an
     incorrect destructor might then be invoked immediately.  In the
     case of YYERROR or YYBACKUP, subsequent parser actions might lead
     to an incorrect destructor call or verbose syntax error message
     before the lookahead is translated.  */
  YY_SYMBOL_PRINT ("-> $$ =", YY_CAST (yysymbol_kind_t, yyr1[yyn]), &yyval, &yyloc);

  YYPOPSTACK (yylen);
  yylen = 0;

  *++yyvsp = yyval;
  *++yylsp = yyloc;

  /* Now 'shift' the result of the reduction.  Determine what state
     that goes to, based on the state we popped back to and the rule
     number reduced by.  */
  {
    const int yylhs = yyr1[yyn] - YYNTOKENS;
    const int yyi = yypgoto[yylhs] + *yyssp;
    yystate = (0 <= yyi && yyi <= YYLAST && yycheck[yyi] == *yyssp
               ? yytable[yyi]
               : yydefgoto[yylhs]);
  }

  goto yynewstate;


/*--------------------------------------.
| yyerrlab -- here on detecting error.  |
`--------------------------------------*/
yyerrlab:
  /* Make sure we have latest lookahead translation.  See comments at
     user semantic actions for why this is necessary.  */
  yytoken = yychar == YYEMPTY ? YYSYMBOL_YYEMPTY : YYTRANSLATE (yychar);
  /* If not already recovering from an error, report this error.  */
  if (!yyerrstatus)
    {
      ++yynerrs;
      {
        yypcontext_t yyctx
          = {yyssp, yytoken, &yylloc};
        char const *yymsgp = YY_("syntax error");
        int yysyntax_error_status;
        yysyntax_error_status = yysyntax_error (&yymsg_alloc, &yymsg, &yyctx);
        if (yysyntax_error_status == 0)
          yymsgp = yymsg;
        else if (yysyntax_error_status == -1)
          {
            if (yymsg != yymsgbuf)
              YYSTACK_FREE (yymsg);
            yymsg = YY_CAST (char *,
                             YYSTACK_ALLOC (YY_CAST (YYSIZE_T, yymsg_alloc)));
            if (yymsg)
              {
                yysyntax_error_status
                  = yysyntax_error (&yymsg_alloc, &yymsg, &yyctx);
                yymsgp = yymsg;
              }
            else
              {
                yymsg = yymsgbuf;
                yymsg_alloc = sizeof yymsgbuf;
                yysyntax_error_status = YYENOMEM;
              }
          }
        yyerror (&yylloc, parser, retval, yymsgp);
        if (yysyntax_error_status == YYENOMEM)
          YYNOMEM;
      }
    }

  yyerror_range[1] = yylloc;
  if (yyerrstatus == 3)
    {
      /* If just tried and failed to reuse lookahead token after an
         error, discard it.  */

      if (yychar <= END_OF_FILE)
        {
          /* Return failure if at end of input.  */
          if (yychar == END_OF_FILE)
            YYABORT;
        }
      else
        {
          yydestruct ("Error: discarding",
                      yytoken, &yylval, &yylloc, parser, retval);
          yychar = YYEMPTY;
        }
    }

  /* Else will try to reuse lookahead token after shifting the error
     token.  */
  goto yyerrlab1;


/*---------------------------------------------------.
| yyerrorlab -- error raised explicitly by YYERROR.  |
`---------------------------------------------------*/
yyerrorlab:
  /* Pacify compilers when the user code never invokes YYERROR and the
     label yyerrorlab therefore never appears in user code.  */
  if (0)
    YYERROR;
  ++yynerrs;

  /* Do not reclaim the symbols of the rule whose action triggered
     this YYERROR.  */
  YYPOPSTACK (yylen);
  yylen = 0;
  YY_STACK_PRINT (yyss, yyssp);
  yystate = *yyssp;
  goto yyerrlab1;


/*-------------------------------------------------------------.
| yyerrlab1 -- common code for both syntax error and YYERROR.  |
`-------------------------------------------------------------*/
yyerrlab1:
  yyerrstatus = 3;      /* Each real token shifted decrements this.  */

  /* Pop stack until we find a state that shifts the error token.  */
  for (;;)
    {
      yyn = yypact[yystate];
      if (!yypact_value_is_default (yyn))
        {
          yyn += YYSYMBOL_YYerror;
          if (0 <= yyn && yyn <= YYLAST && yycheck[yyn] == YYSYMBOL_YYerror)
            {
              yyn = yytable[yyn];
              if (0 < yyn)
                break;
            }
        }

      /* Pop the current state because it cannot handle the error token.  */
      if (yyssp == yyss)
        YYABORT;

      yyerror_range[1] = *yylsp;
      yydestruct ("Error: popping",
                  YY_ACCESSING_SYMBOL (yystate), yyvsp, yylsp, parser, retval);
      YYPOPSTACK (1);
      yystate = *yyssp;
      YY_STACK_PRINT (yyss, yyssp);
    }

  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  *++yyvsp = yylval;
  YY_IGNORE_MAYBE_UNINITIALIZED_END

  yyerror_range[2] = yylloc;
  ++yylsp;
  YYLLOC_DEFAULT (*yylsp, yyerror_range, 2);

  /* Shift the error token.  */
  YY_SYMBOL_PRINT ("Shifting", YY_ACCESSING_SYMBOL (yyn), yyvsp, yylsp);

  yystate = yyn;
  goto yynewstate;


/*-------------------------------------.
| yyacceptlab -- YYACCEPT comes here.  |
`-------------------------------------*/
yyacceptlab:
  yyresult = 0;
  goto yyreturnlab;


/*-----------------------------------.
| yyabortlab -- YYABORT comes here.  |
`-----------------------------------*/
yyabortlab:
  yyresult = 1;
  goto yyreturnlab;


/*-----------------------------------------------------------.
| yyexhaustedlab -- YYNOMEM (memory exhaustion) comes here.  |
`-----------------------------------------------------------*/
yyexhaustedlab:
  yyerror (&yylloc, parser, retval, YY_("memory exhausted"));
  yyresult = 2;
  goto yyreturnlab;


/*----------------------------------------------------------.
| yyreturnlab -- parsing is finished, clean up and return.  |
`----------------------------------------------------------*/
yyreturnlab:
  if (yychar != YYEMPTY)
    {
      /* Make sure we have latest lookahead translation.  See comments at
         user semantic actions for why this is necessary.  */
      yytoken = YYTRANSLATE (yychar);
      yydestruct ("Cleanup: discarding lookahead",
                  yytoken, &yylval, &yylloc, parser, retval);
    }
  /* Do not reclaim the symbols of the rule whose action triggered
     this YYABORT or YYACCEPT.  */
  YYPOPSTACK (yylen);
  YY_STACK_PRINT (yyss, yyssp);
  while (yyssp != yyss)
    {
      yydestruct ("Cleanup: popping",
                  YY_ACCESSING_SYMBOL (+*yyssp), yyvsp, yylsp, parser, retval);
      YYPOPSTACK (1);
    }
#ifndef yyoverflow
  if (yyss != yyssa)
    YYSTACK_FREE (yyss);
#endif
  if (yymsg != yymsgbuf)
    YYSTACK_FREE (yymsg);
  return yyresult;
}

#line 4356 "parser.yy"


void
Lily_parser::set_yydebug (bool x)
{
	yydebug = x;
}

SCM
Lily_parser::do_yyparse ()
{
	return scm_c_with_fluid (Lily::f_parser,
				 self_scm (),
				 do_yyparse_trampoline,
				 static_cast <void *>(this));
}

SCM
Lily_parser::do_yyparse_trampoline (void *parser)
{
	SCM retval = SCM_UNDEFINED;
	yyparse (static_cast <Lily_parser *>(parser), &retval);
	return retval;
}



/*

It is a little strange to have this function in this file, but
otherwise, we have to import music classes into the lexer.

*/
int
Lily_lexer::try_special_identifiers (SCM *destination, SCM sid)
{
	if (unsmob<Book> (sid)) {
		Book *book =  unsmob<Book> (sid)->clone ();
		*destination = book->self_scm ();
		book->unprotect ();

		return BOOK_IDENTIFIER;
	} else if (scm_is_number (sid)) {
		*destination = sid;
		return NUMBER_IDENTIFIER;
	} else if (unsmob<Context_def> (sid))
	{
		*destination = unsmob<Context_def> (sid)->clone ()->unprotect ();
		return SCM_IDENTIFIER;
        } else if (unsmob<Context_mod> (sid)) {
                *destination = unsmob<Context_mod> (sid)->smobbed_copy ();
                return SCM_IDENTIFIER;
	} else if (Music *mus = unsmob<Music> (sid)) {
		mus = mus->clone ();
		*destination = mus->self_scm ();
		bool is_event = mus->is_mus_type ("post-event");
		mus->unprotect ();
		return is_event ? EVENT_IDENTIFIER : MUSIC_IDENTIFIER;
	} else if (unsmob<Pitch> (sid)) {
		*destination = unsmob<Pitch> (sid)->smobbed_copy ();
		return PITCH_IDENTIFIER;
	} else if (unsmob<Duration> (sid)) {
		*destination = unsmob<Duration> (sid)->smobbed_copy ();
		return DURATION_IDENTIFIER;
	} else if (unsmob<Output_def> (sid)) {
		*destination = unsmob<Output_def> (sid)->clone ()->unprotect ();
		return SCM_IDENTIFIER;
	} else if (unsmob<Score> (sid)) {
		*destination = unsmob<Score> (sid)->clone ()->unprotect ();
		return SCM_IDENTIFIER;
	} else if (scm_is_pair (sid)
		   && scm_is_pair (scm_car (sid))
		   && scm_is_true (Lily::key_p (scm_caar (sid)))) {
		*destination = sid;
		return LOOKUP_IDENTIFIER;
	} else if (scm_is_true (Guile_user::module_p (sid))) {
		*destination = sid;
		return MODULE_IDENTIFIER;
	}
	return -1;
}

// check_scheme_arg checks one argument with a given predicate for use
// in an argument list and throws a syntax error if it is unusable.
// The argument is prepended to the argument list in any case.  After
// throwing a syntax error, the argument list is terminated with #f as
// its last cdr in order to mark it as uncallable while not losing
// track of its total length.
//
// There are a few special considerations: if optional argument disp
// is given (otherwise it defaults to SCM_UNDEFINED), it will be used
// instead of arg in a prospective error message.  This is useful if
// arg is not the actual argument but rather a transformation of it.
//
// If arg itself is SCM_UNDEFINED, the predicate is considered false
// and an error message using disp is produced unconditionally.

SCM check_scheme_arg (Lily_parser *parser, Input loc,
		      SCM arg, SCM args, SCM pred, SCM disp)
{
	if (SCM_UNBNDP (arg))
		args = scm_cons (disp, args);
	else {
		args = scm_cons (arg, args);
		if (scm_is_true (ly_call (pred, arg)))
			return args;
	}
	scm_set_cdr_x (scm_last_pair (args), SCM_EOL);
	MAKE_SYNTAX (argument_error, loc, scm_length (args), pred,
		     SCM_UNBNDP (disp) ? arg : disp);
	scm_set_cdr_x (scm_last_pair (args), SCM_BOOL_F);
	return args;
}

SCM loc_on_copy (Lily_parser *parser, Input loc, SCM arg)
{
	if (Music *m = unsmob<Music> (arg))
	{
		m = m->clone ();
		m->set_spot (parser->lexer_->override_input (loc));
		return m->unprotect ();
	}
	if (Book *b = unsmob<Book> (arg))
	{
		b = b->clone ();
		b->origin ()->set_spot (parser->lexer_->override_input (loc));
		return b->unprotect ();
	}
	if (Context_def *cd = unsmob<Context_def> (arg))
	{
		cd = cd->clone ();
		cd->origin ()->set_spot (parser->lexer_->override_input (loc));
		return cd->unprotect ();
	}
	if (Output_def *od = unsmob<Output_def> (arg))
	{
		od = od->clone ();
		od->input_origin_ = parser->lexer_->override_input (loc);
		return od->unprotect ();
	}
	if (Score *s = unsmob<Score> (arg))
	{
		s = s->clone ();
		s->origin ()->set_spot (parser->lexer_->override_input (loc));
		return s->unprotect ();
	}
	if (Context_mod *cm = unsmob<Context_mod> (arg))
	{
		return cm->smobbed_copy ();
	}
	return arg;
}

SCM
make_reverse_key_list (SCM keys)
{
	if (scm_is_true (Lily::key_p (keys)))
		return ly_list (keys);
	if (scm_is_string (keys))
		return ly_list (scm_string_to_symbol (keys));
	if (!ly_is_list (keys))
		return SCM_UNDEFINED;
	SCM res = SCM_EOL;
	for (; scm_is_pair (keys); keys = scm_cdr (keys))
	{
		SCM elt = scm_car (keys);
		if (scm_is_true (Lily::key_p (elt)))
			res = scm_cons (elt, res);
		else if (scm_is_string (elt))
			res = scm_cons (scm_string_to_symbol (elt), res);
		else return SCM_UNDEFINED;
	}
	return res;
}

SCM
try_string_variants (SCM pred, SCM str)
{
	// a matching predicate is always ok
	if (scm_is_true (ly_call (pred, str)))
		return str;
	// a key may be interpreted as a list of keys if it helps
	if (scm_is_true (Lily::key_p (str))) {
		str = ly_list (str);
		if (scm_is_true (ly_call (pred, str)))
			return str;
		return SCM_UNDEFINED;
	}

	if (!scm_is_string (str))
		return SCM_UNDEFINED;

	// Let's attempt the symbol list interpretation first.

	str = scm_string_to_symbol (str);

	SCM lst = ly_list (str);

	if (scm_is_true (ly_call (pred, lst)))
		return lst;

	// Try the single symbol interpretation

	if (scm_is_true (ly_call (pred, str)))
		return str;

	return SCM_UNDEFINED;
}

SCM
try_word_variants (SCM pred, SCM str)
{
	// str is always a string when we come here

	if (scm_is_true (ly_call (pred, str)))
		return str;

	// If this cannot be a string representation of a symbol list,
	// we are through.

	if (!is_regular_identifier (str, true))
		return SCM_UNDEFINED;

	str = scm_string_split (str, SCM_MAKE_CHAR ('.'));
	for (SCM &p : as_ly_scm_list (str))
		p = scm_string_split (p, SCM_MAKE_CHAR (','));
	str = scm_append_x (str);
	for (SCM &p : as_ly_scm_list (str))
		p = scm_string_to_symbol (p);

	// Let's attempt the symbol list interpretation first.

	if (scm_is_true (ly_call (pred, str)))
		return str;

	// If there is just one symbol in the list, we might interpret
	// it as a single symbol

	if (scm_is_null (scm_cdr (str)))
	{
		str = scm_car (str);
		if (scm_is_true (ly_call (pred, str)))
			return str;
	}

	return SCM_UNDEFINED;
}

bool
is_regular_identifier (SCM id, bool multiple)
{
  if (!scm_is_string (id))
	  return false;

  auto str = from_scm<std::string> (id);

  bool middle = false;

  for (string::iterator it=str.begin(); it != str.end (); it++)
  {
	  int c = *it & 0xff;
	  if ((c >= 'a' && c <= 'z')
	      || (c >= 'A' && c <= 'Z')
	      || c > 0x7f)
		  middle = true;
	  else if (middle && (c == '-' || c == '_' || (multiple &&
						       (c == '.' || c == ','))))
		  middle = false;
	  else
		  return false;
  }
  return middle;
}

SCM
make_music_from_simple (Lily_parser *parser, Input loc, SCM simple)
{
	if (unsmob<Music> (simple))
		return simple;

	if (scm_is_symbol (simple))
	{
		SCM out = SCM_UNDEFINED;
		switch (parser->lexer_->scan_word (out, simple))
		{
		case DRUM_PITCH:
		{
			Music *n = MY_MAKE_MUSIC ("NoteEvent", loc);
			set_property (n, "duration", parser->default_duration_.smobbed_copy ());
			set_property (n, "drum-type", out);
			return n->unprotect ();
		}
		case NOTENAME_PITCH:
		case TONICNAME_PITCH:
			// Take the parsed pitch
			simple = out;
			break;
		// Don't scan CHORD_MODIFIER etc.
		}
	}

	if (parser->lexer_->is_note_state ()) {
		if (unsmob<Pitch> (simple)) {
			Music *n = MY_MAKE_MUSIC ("NoteEvent", loc);
			set_property (n, "duration", parser->default_duration_.smobbed_copy ());
			set_property (n, "pitch", simple);
			return n->unprotect ();
		}
		SCM d = simple;
		if (scm_is_integer (simple))
			d = make_duration (simple);
		if (unsmob<Duration> (d)) {
			Music *n = MY_MAKE_MUSIC ("NoteEvent", loc);
			set_property (n, "duration", d);
			return n->unprotect ();
		}
		return simple;
	} else if (parser->lexer_->is_lyric_state ()) {
		if (Text_interface::is_markup (simple))
			return MAKE_SYNTAX (lyric_event, loc, simple,
					    parser->default_duration_.smobbed_copy ());
	} else if (parser->lexer_->is_chord_state ()) {
		if (unsmob<Pitch> (simple))
			return MAKE_SYNTAX
				(event_chord,
				 loc,
				 make_chord_elements (loc, simple,
						      parser->default_duration_.smobbed_copy (),
						      SCM_EOL));
	}
	return simple;
}

Music *
make_music_with_input (SCM name, Input where)
{
       Music *m = make_music_by_name (name);
       m->set_spot (where);
       return m;
}

SCM
make_simple_markup (SCM a)
{
	return a;
}

SCM
make_duration (SCM d, int dots, SCM factor)
{
	Duration k;

	if (Duration *dur = unsmob<Duration> (d)) {
		if (!dots && SCM_UNBNDP (factor))
			return d;
		k = *dur;
		if (dots)
			k = Duration (k.duration_log (), k.dot_count () + dots)
				.compressed (k.factor ());
	} else {
		int t = from_scm<int> (d);
		if (t > 0 && (t & (t-1)) == 0)
			k = Duration (intlog2 (t), dots);
		else
			return SCM_UNDEFINED;
	}

	if (!SCM_UNBNDP (factor))
		k = k.compressed (from_scm<Rational> (factor));

	return k.smobbed_copy ();
}

SCM
make_chord_step (SCM step_scm, Rational alter)
{
	Pitch m (0, from_scm<int> (step_scm) - 1, alter);

	// Notename/octave are normalized
	if (m.get_notename () == 6)
		m = m.transposed (Pitch (0, 0, FLAT_ALTERATION));

	return m.smobbed_copy ();
}


SCM
make_chord_elements (Input loc, SCM pitch, SCM dur, SCM modification_list)
{
	SCM res = Lily::construct_chord_elements (pitch, dur, modification_list);
	for (SCM s = res; scm_is_pair (s); s = scm_cdr (s))
	{
		unsmob<Music> (scm_car (s))->set_spot (loc);
	}
	return res;
}

// Return true if there are post events unaccounted for
bool
add_post_events (Music *m, SCM events)
{
	if (!scm_is_pair (events))
		return false;	// successfully added -- nothing

	while (m) {
		if (m->is_mus_type ("rhythmic-event")
		    || m->is_mus_type ("caesura-event")) {
			set_property
				(m, "articulations",
				 scm_append_x (ly_list
					       (get_property (m, "articulations"),
						events)));
			return false;
		}
		if (m->is_mus_type ("event-chord")) {
			set_property
				(m, "elements",
				 scm_append_x (ly_list
					       (get_property (m, "elements"),
						events)));
			return false;
		}
		if (m->is_mus_type ("sequential-music")) {
			SCM lp = scm_last_pair (get_property (m, "elements"));
			if (scm_is_pair (lp)) {
				m = unsmob<Music> (scm_car (lp));
				continue;
			}
			return true;
		}
		if (m->is_mus_type ("music-wrapper-music")
		    || m->is_mus_type ("time-scaled-music")) {
			m = unsmob<Music> (get_property (m, "element"));
			continue;
		}
		break;
	}
	return true;
}

// Returns either a list or a post-event
//
// If PRESERVE is true, unattachable post-events are not thrown away
// but rather added attached to empty chords.  If COMPRESS is true, a
// sequence consisting only of post-events may be returned as a single
// post-event.
SCM reverse_music_list (Lily_parser *parser, Input loc, SCM lst, bool preserve, bool compress)
{
	SCM res = SCM_EOL;	// Resulting reversed list
	SCM bad = SCM_EOL;	// Bad post events
	SCM post = SCM_EOL;	// current unattached events
	for (; scm_is_pair (lst); lst = scm_cdr (lst)) {
		SCM elt = scm_car (lst);
		Music *m = unsmob<Music> (elt);
		assert (m);
		if (m->is_mus_type ("post-event")) {
			post = post_event_cons (elt, post);
			continue;
		}
		if (add_post_events (m, post)) {
			bad = scm_cons (scm_car (post), bad);
			if (preserve) {
				Music *p = unsmob<Music> (scm_car (post));
				res = scm_cons (MAKE_SYNTAX (event_chord,
							     *p->origin (),
							     post),
						res);
			}
		}
		post = SCM_EOL;
		res = scm_cons (elt, res);
	}
	if (scm_is_pair (post)) {
		if (scm_is_null (res) && compress) { // pure postevent list
			if (scm_is_null (scm_cdr (post)))
				return scm_car (post);
			Music *m = MY_MAKE_MUSIC ("PostEvents", loc);
			set_property (m, "elements", post);
			return m->unprotect ();
		}
		bad = ly_append (post, bad);
		if (preserve) {
			Music *p = unsmob<Music> (scm_car (post));
			res = scm_cons (MAKE_SYNTAX (event_chord,
						     *p->origin (),
						     post),
					res);
		}
	}
	for (; scm_is_pair (bad); bad = scm_cdr (bad))
	{
		Music *what = unsmob<Music> (scm_car (bad));
		if (preserve)
			what->warning (_f ("Unattached %s", what->name ()));
		else
			what->warning (_f ("Dropping unattachable %s", what->name ()));
	}
	return res;
}

SCM post_event_cons (SCM post_event, SCM tail)
{
	Music *ev = unsmob<Music> (post_event);
	if (!ev)
		return tail;
	if (!ev->is_mus_type ("post-event-wrapper"))
		return scm_cons (post_event, tail);
	SCM elts = SCM_UNDEFINED;
	SCM props = SCM_EOL;
	SCM tweaks = SCM_UNDEFINED;
	for (SCM p = ev->get_property_alist (true);
	     scm_is_pair (p);
	     p = scm_cdr (p))
	{
		SCM pair = scm_car (p);
		SCM sym = scm_car (pair);
		if (scm_is_eq (sym, ly_symbol2scm ("origin")))
			continue;
		else if (scm_is_eq (sym, ly_symbol2scm ("elements"))
			 && SCM_UNBNDP (elts))
			elts = scm_cdr (pair);
		else if (scm_is_eq (sym, ly_symbol2scm ("tweaks"))
			 && SCM_UNBNDP (tweaks))
			tweaks = scm_cdr (pair);
		else
			props = scm_cons (pair, props);
	}
	if (!scm_is_pair (elts))
		return tail;
	elts = scm_reverse_x (elts, SCM_EOL);
	for (SCM p = elts; scm_is_pair (p); p = scm_cdr (p))
	{
		Music *ev = unsmob<Music> (scm_car (p));
		// tweaks are always collected in-order, newer tweaks
		// nearer to the front of the list
		if (scm_is_pair (tweaks))
			set_property (ev,
			              "tweaks",
			              Srfi_1::append_reverse (tweaks,
				                              get_property (ev, "tweaks")));
		// other properties are applied last to first so that
		// in case of duplicate properties, the actually
		// current one survives
		for (SCM q = props; scm_is_pair (q); q = scm_cdr (q))
			set_property (ev, scm_caar (q), scm_cdar (q));
	}
	return ly_append (elts, tail);
}

void property_path_dot_warning (Input loc, SCM lst)
{
	// if lst is empty, don't even venture a guess...
	if (scm_is_pair (lst)) {
		std::string out = ly_symbol2string (scm_car (lst));
		for (lst = scm_cdr (lst); scm_is_pair (lst); lst = scm_cdr (lst)) {
			out += ".";
			out += ly_symbol2string (scm_car (lst));
		}
		loc.warning (_f ("deprecated: missing `.' in property path %s", out));
	}
}


int
yylex (YYSTYPE *s, YYLTYPE *loc, Lily_parser *parser)
{
	Lily_lexer *lex = parser->lexer_;

	lex->lexval_ = s;
	lex->lexloc_ = loc;
	int tok = lex->pop_extra_token ();
	if (tok >= 0)
		return tok;
	lex->prepare_for_next_token ();
	return lex->yylex ();
}
